(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    435897,      12488]
NotebookOptionsPosition[    382450,      11007]
NotebookOutlinePosition[    400706,      11394]
CellTagsIndexPosition[    400071,      11371]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Boolean Algebra", "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 12}}],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
 "In this chapter we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to model Boolean algebra. In the first section, we demonstrate the basic \
functions that will be used in this chapter. In the second section, we will \
focus on the disjunctive normal form of a logical expression. We will see how \
to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s functions for finding a disjunctive normal form expression for a Boolean \
function and for finding a representation for a function defined by a table \
of values. In Section 3, we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to model logical circuits, including how to go about \
transforming a circuit diagram into a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " expression. We also provide a function that will transform a logical \
expression into a model of a circuit. In the final section of the chapter, we \
consider simplification of logical expressions, and we develop an \
implementation of the Quine-McCluskey method."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["12.1 Boolean Functions", "Section",
 CellTags->"Section1"],

Cell[TextData[{
 "In this section we will see how to work with Boolean expressions and how to \
create Boolean functions. We will also use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to verify identities in Boolean algebra and to compute the dual of an \
expression."
}], "Text"],

Cell[CellGroupData[{

Cell["Preliminaries", "Subsection",
 CellTags->"Section1a"],

Cell[TextData[{
 "In Chapter 1 of this manual, we discussed ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s logical expressions. The Boolean values ",
 Cell[BoxData[
  FormBox["true", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["false", TraditionalForm]]],
 " are represented by the symbols ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 ". To ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", these are constant values, like the numbers ",
 StyleBox["2", "Input"],
 " or ",
 StyleBox[ButtonBox["Pi",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Pi"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "We also saw in Chapter 1 the logical operators ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 " (",
 StyleBox["&&", "Input"],
 "), ",
 StyleBox[ButtonBox["Or",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Or"], "FunctionLink"],
 " (",
 StyleBox["||", "Input"],
 "), and ",
 StyleBox[ButtonBox["Not",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Not"], "FunctionLink"],
 " (",
 StyleBox["!", "Input"],
 "). These can be used in functional form or as operators. The two \
expressions below both compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", "\[And]", 
    RowBox[{"\[Not]", 
     RowBox[{"(", 
      RowBox[{"T", "\[Or]", "F"}], ")"}]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"True", "&&", 
  RowBox[{"!", 
   RowBox[{"(", 
    RowBox[{"True", "||", "False"}], ")"}]}]}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"And", "[", 
  RowBox[{"True", ",", 
   RowBox[{"Not", "[", 
    RowBox[{"Or", "[", 
     RowBox[{"True", ",", "False"}], "]"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " obeys the usual order of precedence for logical operators, namely negation \
followed by conjunction, then disjunction, and finally implication. "
}], "Text"],

Cell[TextData[{
 "Other logical operators supported by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " include: the exclusive or, ",
 StyleBox[ButtonBox["Xor",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Xor"], "FunctionLink"],
 ", implication, ",
 StyleBox[ButtonBox["Implies",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Implies"], "FunctionLink"],
 ", and the biconditional, ",
 StyleBox[ButtonBox["Equivalent",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equivalent"], "FunctionLink"],
 ". Each logical connective can be entered as the usual mathematical symbol \
by using an escape sequence, as shown in the table below. "
}], "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {"And", "\[EscapeKey]and\[EscapeKey]", "\[And]"},
   {"Or", "\[EscapeKey]or\[EscapeKey]", "\[Or]"},
   {"Not", "\[EscapeKey]not\[EscapeKey]", "\[Not]"},
   {"Xor", "\[EscapeKey]xor\[EscapeKey]", "\[Xor]"},
   {"Implies", 
    StyleBox[
     RowBox[{"\[EscapeKey]", "=", 
      RowBox[{">", "\[EscapeKey]"}]}], "Text",
     AutoSpacing->False], "\[Implies]"},
   {"Equivalent", "\[EscapeKey]equiv\[EscapeKey]", "\[Equivalent]"}
  },
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]],
 FontFamily->"Times"]], "Text"],

Cell[TextData[{
 "In this manual, we will typically enter operators using either the simple \
keyboard character operators for ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 " (",
 StyleBox["&&", "Input"],
 "), ",
 StyleBox[ButtonBox["Or",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Or"], "FunctionLink"],
 " (",
 StyleBox["||", "Input"],
 "), and ",
 StyleBox[ButtonBox["Not",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Not"], "FunctionLink"],
 " (",
 StyleBox["!", "Input"],
 ") or in functional form, rather than using the escape sequences."
}], "Text"],

Cell[TextData[{
 "For Boolean algebra, the textbook uses the objects 0 and 1 with operators \
",
 Cell[BoxData[
  FormBox["+", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["\[CenterDot]", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["\[VeryThinSpace]", "_"], TraditionalForm]]],
 " instead of their logical counterparts. It is tempting to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s bit functions, ",
 StyleBox[ButtonBox["BitAnd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitAnd"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["BitOr",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitOr"], "FunctionLink"],
 ", and ",
 StyleBox[ButtonBox["BitNot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitNot"], "FunctionLink"],
 ", in order to replicate the 0-1 form of Boolean expressions. However, the \
bit functions behave differently than the corresponding Boolean operators \
would, in particular ",
 StyleBox[ButtonBox["BitNot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BitNot"], "FunctionLink"],
 " does not switch between 0 and 1."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BitNot", "[", "0", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "1"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BitNot", "[", "1", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "2"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "In the remainder of this manual, we will stick to the logical forms of \
Boolean expressions. However, some readers may be interested to know that it \
is possible to create operators to mirror the kinds of Boolean algebra \
expressions used in the text. To do so, we use symbols that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will interpret as operators but which have no built-in definition. For \
example, we could use circle times (\[CircleTimes], entered \[EscapeKey]c*\
\[EscapeKey]) and circle plus (\[CirclePlus], entered \[EscapeKey]c+\
\[EscapeKey]) for the ",
 StyleBox["and",
  FontWeight->"Bold"],
 " and ",
 StyleBox["or",
  FontWeight->"Bold"],
 " operators, and the unary minus-plus (\[MinusPlus], entered \[EscapeKey]-+\
\[EscapeKey]) for negation. Then the expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "\[CenterDot]", "0"}], "+", 
    OverscriptBox[
     RowBox[{"(", 
      RowBox[{"0", "+", "1"}], ")"}], "_"]}], TraditionalForm]]],
 " would be entered as shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"1", "\[CircleTimes]", "0"}], "\[CirclePlus]", 
  RowBox[{"\[MinusPlus]", 
   RowBox[{"(", 
    RowBox[{"0", "\[CirclePlus]", "1"}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"1", "\[CircleTimes]", "0"}], "\[CirclePlus]", 
  RowBox[{"\[MinusPlus]", 
   RowBox[{"(", 
    RowBox[{"0", "\[CirclePlus]", "1"}], ")"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To get ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to evaluate such expressions properly, you just need to make definitions \
to the symbols. By setting the ",
 StyleBox[ButtonBox["Flat",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flat"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Listable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Listable"], "FunctionLink"],
 " attributes first, the operator will be associative. For example, \
\[CircleTimes] can be defined by setting values for ",
 StyleBox[ButtonBox["CircleTimes",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CircleTimes"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"CircleTimes", ",", 
    RowBox[{"{", 
     RowBox[{"Flat", ",", "Listable"}], "}"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CircleTimes", "[", 
    RowBox[{"1", ",", "1"}], "]"}], "=", "1"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CircleTimes", "[", 
    RowBox[{"1", ",", "0"}], "]"}], "=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CircleTimes", "[", 
    RowBox[{"0", ",", "1"}], "]"}], "=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CircleTimes", "[", 
    RowBox[{"0", ",", "0"}], "]"}], "=", "0"}], ";"}]}], "Input"],

Cell[TextData[{
 "Now ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will automatically simplify \[CircleTimes], so if we enter the previous \
expression again:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"1", "\[CircleTimes]", "0"}], "\[CirclePlus]", 
  RowBox[{"\[MinusPlus]", 
   RowBox[{"(", 
    RowBox[{"0", "\[CirclePlus]", "1"}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"0", "\[CirclePlus]", 
  RowBox[{"\[MinusPlus]", 
   RowBox[{"(", 
    RowBox[{"0", "\[CirclePlus]", "1"}], ")"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Definitions of the other operations are left to the interested reader. In \
this manual, we will not use this approach, since the logical form of Boolean \
expressions is more naturally supported by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Boolean Expressions and Boolean Functions", "Subsection",
 CellTags->"Section1b"],

Cell[TextData[{
 "Consider Example 1 from the text, which asks that we compute the value of \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "\[CenterDot]", "0"}], "+", 
    OverscriptBox[
     RowBox[{"(", 
      RowBox[{"0", "+", "1"}], ")"}], "_"]}], TraditionalForm]]],
 ". To perform this computation in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we first translate it into a logical statement. We do this by changing 1 \
into ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ", 0 into ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 ", the multiplication into ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 " (",
 StyleBox["&&", "Input"],
 "), the addition into ",
 StyleBox[ButtonBox["Or",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Or"], "FunctionLink"],
 " (",
 StyleBox["||", "Input"],
 "), and the bar into ",
 StyleBox[ButtonBox["Not",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Not"], "FunctionLink"],
 " (",
 StyleBox["!", "Input"],
 ")."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"True", "&&", "False"}], "||", 
  RowBox[{"!", 
   RowBox[{"(", 
    RowBox[{"False", "||", "True"}], ")"}]}]}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["\<\
Of course, you can enter Boolean expressions involving variables, assuming \
the symbols have not previously been assigned values.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Implies", "[", 
  RowBox[{
   RowBox[{"p", "&&", "q"}], ",", "r"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"p", "&&", "q"}], "\[Implies]", "r"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "And, just as with arithmetic expressions, you can evaluate these \
expressions for specific values by applying the ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") operator."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Implies", "[", 
   RowBox[{
    RowBox[{"p", "&&", "q"}], ",", "r"}], "]"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"p", "\[Rule]", "True"}], ",", 
    RowBox[{"q", "\[Rule]", "True"}], ",", 
    RowBox[{"r", "\[Rule]", "False"}]}], "}"}]}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Representing Boolean Functions", "Subsubsection"],

Cell[TextData[{
 "You define a Boolean function in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in the same way as any other function."
}], "Text"],

Cell[TextData[{
 "Consider, for example, the Boolean function ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", 
     RowBox[{"x", ",", "y", ",", "z"}], ")"}], "=", 
    RowBox[{
     RowBox[{"x", " ", "y"}], "+", 
     RowBox[{"y", " ", "z"}], "+", 
     RowBox[{"z", " ", "x"}]}]}], TraditionalForm]]],
 " (written in the 0-1 notation)."
}], "Text"],

Cell[TextData[{
 "This can be modeled in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by the function defined below."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", 
   RowBox[{"x_", ",", "y_", ",", "z_"}], "]"}], ":=", 
  RowBox[{"Or", "[", 
   RowBox[{
    RowBox[{"And", "[", 
     RowBox[{"x", ",", "y"}], "]"}], ",", 
    RowBox[{"And", "[", 
     RowBox[{"y", ",", "z"}], "]"}], ",", 
    RowBox[{"And", "[", 
     RowBox[{"z", ",", "x"}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "You can work with ",
 StyleBox["f", "Input"],
 " in the usual way. The following applies ",
 StyleBox["f", "Input"],
 " to ",
 StyleBox["p", "Input"],
 ", ",
 StyleBox["q", "Input"],
 ", and ",
 StyleBox["r", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "[", 
  RowBox[{"p", ",", "q", ",", "r"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"p", "&&", "q"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"q", "&&", "r"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"r", "&&", "p"}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "When ",
 StyleBox["f", "Input"],
 " is applied to truth values, it is evaluated."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "[", 
  RowBox[{"True", ",", "False", ",", "True"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can also mix truth values and symbols. In this case, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will simplify the expression, given the partial information."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "[", 
  RowBox[{"True", ",", "q", ",", "r"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"q", "||", 
  RowBox[{"(", 
   RowBox[{"q", "&&", "r"}], ")"}], "||", "r"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You may notice that this expression is logically equivalent to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "\[Or]", "r"}], TraditionalForm]]],
 ". Applying the ",
 StyleBox[ButtonBox["Simplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Simplify"], "FunctionLink"],
 " function will ask ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to more fully simplify the output."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", 
   RowBox[{"True", ",", "q", ",", "r"}], "]"}], "//", "Simplify"}]], "Input"],

Cell[BoxData[
 RowBox[{"q", "||", "r"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Values of Boolean Functions", "Subsubsection"],

Cell[TextData[{
 "Examples 4 and 5 of Section 12.1 illustrate how the values of a Boolean \
function, in the 0-1 format, can be displayed in a table. In the logical \
form, this is equivalent to a truth table for the Boolean function. In \
Chapter 1 of this manual, we illustrated the use of the ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 " function for creating truth tables."
}], "Text"],

Cell[TextData[{
 "Recall that ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 " accepts two arguments: a Boolean expression and a list of the variables. \
The output is a list of the truth values for the expression obtained by \
substituting every possible combination of truth values into the variables."
}], "Text"],

Cell[TextData[{
 "For example, we will display the table of values for the Boolean function \
",
 StyleBox["f", "Input"],
 " defined above. The first argument to  ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 " will be a list containing the three variables and the function applied to \
them. Giving the first argument as this list means that the output will \
indicate the values of the individual variables, and not just the result. The \
second argument will be the list of variables."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BooleanTable", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "q", ",", "r", ",", 
      RowBox[{"f", "[", 
       RowBox[{"p", ",", "q", ",", "r"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "q", ",", "r"}], "}"}]}], "]"}], "//", 
  "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {"True", "True", "True", "True"},
    {"True", "True", "False", "True"},
    {"True", "False", "True", "True"},
    {"True", "False", "False", "False"},
    {"False", "True", "True", "True"},
    {"False", "True", "False", "False"},
    {"False", "False", "True", "False"},
    {"False", "False", "False", "False"}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "If you wish, you can use this function to produce output in the 0-1 form by \
applying the ",
 StyleBox[ButtonBox["Boole",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Boole"], "FunctionLink"],
 " function. ",
 StyleBox[ButtonBox["Boole",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Boole"], "FunctionLink"],
 " is a built-in function that transforms the truth values ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " into the values 1 and 0. Since it threads over lists, it can be applied to \
the output from ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Boole", "[", 
   RowBox[{"BooleanTable", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"p", ",", "q", ",", "r", ",", 
       RowBox[{"f", "[", 
        RowBox[{"p", ",", "q", ",", "r"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"p", ",", "q", ",", "r"}], "}"}]}], "]"}], "]"}], "//", 
  "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {"1", "1", "1", "1"},
    {"1", "1", "0", "1"},
    {"1", "0", "1", "1"},
    {"1", "0", "0", "0"},
    {"0", "1", "1", "1"},
    {"0", "1", "0", "0"},
    {"0", "0", "1", "0"},
    {"0", "0", "0", "0"}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can further refine the output by using the ",
 StyleBox[ButtonBox["TableHeadings",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableHeadings"], "FunctionLink"],
 " option for ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 ". ",
 StyleBox[ButtonBox["TableHeadings",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableHeadings"], "FunctionLink"],
 " is assigned to a pair representing the row and column headings, with ",
 StyleBox[ButtonBox["None",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/None"], "FunctionLink"],
 " used when a group of headings is not wanted."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{
   RowBox[{"Boole", "[", 
    RowBox[{"BooleanTable", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"p", ",", "q", ",", "r", ",", 
        RowBox[{"f", "[", 
         RowBox[{"p", ",", "q", ",", "r"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"p", ",", "q", ",", "r"}], "}"}]}], "]"}], "]"}], ",", 
   RowBox[{"TableHeadings", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"None", ",", 
      RowBox[{"{", 
       RowBox[{"p", ",", "q", ",", "r", ",", "f"}], "}"}]}], "}"}]}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      TagBox["p",
       HoldForm], 
      TagBox["q",
       HoldForm], 
      TagBox["r",
       HoldForm], 
      TagBox["f",
       HoldForm]},
     {"1", "1", "1", "1"},
     {"1", "1", "0", "1"},
     {"1", "0", "1", "1"},
     {"1", "0", "0", "0"},
     {"0", "1", "1", "1"},
     {"0", "1", "0", "0"},
     {"0", "0", "1", "0"},
     {"0", "0", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxDividers->{
     "Columns" -> {{False}}, "ColumnsIndexed" -> {}, 
      "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[2.0999999999999996`]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   {None, OutputFormsDump`HeadedColumns}],
  Function[BoxForm`e$, 
   TableForm[
   BoxForm`e$, 
    TableHeadings -> {
     None, {$CellContext`p, $CellContext`q, $CellContext`r, \
$CellContext`f}}]]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Operations on Boolean Functions", "Subsubsection"],

Cell["\<\
As with functions on real numbers, Boolean functions can be combined using \
basic operations. The complement of a Boolean function and the Boolean sum \
and product of functions are defined in the text. \
\>", "Text"],

Cell[TextData[{
 "To compute complements, sums, and products of Boolean functions, you must \
define a new function in terms of the original. For example, consider the \
function ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"G", "(", 
     RowBox[{"x", ",", "y"}], ")"}], "=", 
    RowBox[{"x", "\[CenterDot]", "y"}]}], TraditionalForm]]],
 ". In logical notation, this is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"G", "(", 
     RowBox[{"x", ",", "y"}], ")"}], "=", 
    RowBox[{"x", "\[And]", "y"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"G", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"x", "&&", "y"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The complement of ",
 StyleBox["G", "Input"],
 ", which we'll call ",
 StyleBox["notG", "Input"],
 ", is created as follows. The arguments of ",
 StyleBox["notG", "Input"],
 " are the same as ",
 StyleBox["G", "Input"],
 ". The formula that defines ",
 StyleBox["notG", "Input"],
 " is ",
 Cell[BoxData[
  RowBox[{"!", 
   RowBox[{"G", "[", 
    RowBox[{"x", ",", "y"}], "]"}]}]], "myCode"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"notG", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"!", 
   RowBox[{"G", "[", 
    RowBox[{"x", ",", "y"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Observe that if we evaluate ",
 StyleBox["notG", "Input"],
 " at a pair of variables, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " returns the expected result."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"notG", "[", 
  RowBox[{"x", ",", "y"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"!", 
  RowBox[{"(", 
   RowBox[{"x", "&&", "y"}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Let us define another function, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"H", "(", 
     RowBox[{"x", ",", "y"}], ")"}], "=", 
    RowBox[{"x", "\[CenterDot]", 
     OverscriptBox["y", "_"]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"H", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"x", "&&", 
   RowBox[{"!", "y"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "To compute the Boolean sum ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "+", "H"}], TraditionalForm]]],
 ", we combine the functions with the ",
 StyleBox[ButtonBox["Or",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Or"], "FunctionLink"],
 " (",
 StyleBox["||", "Input"],
 ") operator. More precisely, we define a function ",
 StyleBox["GpH", "Input"],
 " with the formula ",
 StyleBox["G[x,y]||H[x,y]", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"GpH", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"G", "[", 
    RowBox[{"x", ",", "y"}], "]"}], "||", 
   RowBox[{"H", "[", 
    RowBox[{"x", ",", "y"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Applying this to a pair of variables and simplifying, we obtain the \
following formula for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"G", "+", "H"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GpH", "[", 
   RowBox[{"x", ",", "y"}], "]"}], "//", "Simplify"}]], "Input"],

Cell[BoxData["x"], "Output"]
}, Open  ]],

Cell[TextData[{
 "This result indicates that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"x", "\[CenterDot]", "y"}], "+", 
     RowBox[{"x", "\[CenterDot]", 
      OverscriptBox["y", "_"]}]}], "=", "x"}], TraditionalForm]]],
 ". This can also be verified using the identities in Table 5 of Section 12.1."
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Identities of Boolean Algebra", "Subsection",
 CellTags->"Section1c"],

Cell[TextData[{
 "We can check identities, equivalence of Boolean expressions, and equality \
of Boolean functions using the ",
 StyleBox[ButtonBox["Equivalent",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equivalent"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 " functions."
}], "Text"],

Cell[TextData[{
 "We will use the distributive law ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "(", 
     RowBox[{"y", "+", "z"}], ")"}], "=", 
    RowBox[{
     RowBox[{"x", "\[CenterDot]", "y"}], "+", 
     RowBox[{
      RowBox[{"x", "\[CenterDot]", "z"}], Cell[""]}]}]}], TraditionalForm]]],
 " as an example. First we must translate the statement into a logical \
equivalence: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[And]", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"y", "\[Or]", "z"}], ")"}], "\[Congruent]", 
      RowBox[{"(", 
       RowBox[{"x", "\[And]", "y"}], ")"}]}]}], "\[Or]", 
    RowBox[{"(", 
     RowBox[{"x", "\[And]", "z"}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
Now we will assign the expressions on either side of the equivalence to \
symbols. This is not necessary, but it will make later expressions easier to \
read.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"distributiveL", "=", 
   RowBox[{"x", "&&", 
    RowBox[{"(", 
     RowBox[{"y", "||", "z"}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"distributiveR", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "&&", "y"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{"x", "&&", "z"}], ")"}]}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "To confirm the equivalence of the two Boolean expressions, we combine them \
into a biconditional using the ",
 StyleBox[ButtonBox["Equivalent",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equivalent"], "FunctionLink"],
 " function. We then apply the ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 " function to the biconditional and a list of the Boolean variables \
appearing in the expression."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TautologyQ", "[", 
  RowBox[{
   RowBox[{"Equivalent", "[", 
    RowBox[{"distributiveL", ",", "distributiveR"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell["This verifies the given distributive law.", "Text"],

Cell[TextData[{
 "In the case that the two expressions are not equivalent, you can use the ",
 StyleBox[ButtonBox["SatisfiabilityInstances",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiabilityInstances"], "FunctionLink"],
 " function to find a list of assignments of truth values to the variables in \
the expression that demonstrates that the expressions are not equivalent."
}], "Text"],

Cell[TextData[{
 "Consider the non-equivalence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "+", 
     RowBox[{"x", "\[CenterDot]", "y"}]}], "\[NotEqual]", "y"}], 
   TraditionalForm]]],
 ". In logical form, this is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[Or]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", "\[And]", "y"}], ")"}], "\[NotCongruent]", "y"}]}], 
   TraditionalForm]]],
 ". First observe that ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 " returns false."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TautologyQ", "[", 
  RowBox[{
   RowBox[{"Equivalent", "[", 
    RowBox[{
     RowBox[{"x", "||", 
      RowBox[{"(", 
       RowBox[{"x", "&&", "y"}], ")"}]}], ",", "y"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now apply ",
 StyleBox[ButtonBox["SatisfiabilityInstances",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiabilityInstances"], "FunctionLink"],
 " to the negation of the equivalence."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SatisfiabilityInstances", "[", 
  RowBox[{
   RowBox[{"!", 
    RowBox[{"Equivalent", "[", 
     RowBox[{
      RowBox[{"x", "||", 
       RowBox[{"(", 
        RowBox[{"x", "&&", "y"}], ")"}]}], ",", "y"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"True", ",", "False"}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This output means that setting ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " equal to ",
 Cell[BoxData[
  FormBox[
   StyleBox["true",
    FontWeight->"Bold"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " equal to ",
 Cell[BoxData[
  FormBox[
   StyleBox["false",
    FontWeight->"Bold"], TraditionalForm]]],
 " provides a demonstration, by counterexample, that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "\[Or]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", "\[And]", "y"}], ")"}], "\[NotCongruent]", "y"}]}], 
   TraditionalForm]]],
 ". Indeed, substituting ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", "true"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", "=", "false"}], TraditionalForm]]],
 " on the left hand side produces ",
 Cell[BoxData[
  FormBox[
   RowBox[{"true", "\[Or]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"true", "\[And]", "false"}], ")"}], "\[Congruent]", "true"}], 
    "\[Or]", 
    RowBox[{"false", "\[Congruent]", "true"}]}], TraditionalForm]]],
 ". That is not the same as the right hand side, ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 ", which is assigned ",
 Cell[BoxData[
  FormBox[
   StyleBox["false",
    FontWeight->"Bold"], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Note that the output from ",
 StyleBox[ButtonBox["SatisfiabilityInstances",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SatisfiabilityInstances"], "FunctionLink"],
 " is a list of assignments. Ordinarily only one truth value assignment will \
be returned. But if you provide a positive integer as an optional third \
argument, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will attempt to find that number of different assignments. Below, we ask \
for three assignments, but only two exist and so two are returned."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SatisfiabilityInstances", "[", 
  RowBox[{
   RowBox[{"!", 
    RowBox[{"Equivalent", "[", 
     RowBox[{
      RowBox[{"x", "||", 
       RowBox[{"(", 
        RowBox[{"x", "&&", "y"}], ")"}]}], ",", "y"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"True", ",", "False"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"False", ",", "True"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Equality of Boolean functions can also be checked with the ",
 StyleBox[ButtonBox["Equivalent",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equivalent"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 " functions."
}], "Text"],

Cell["Consider the following Boolean functions.", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", "1"], "(", 
    RowBox[{"x", ",", "y"}], ")"}], "=", 
   OverscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "\[CenterDot]", "y"}], ")"}], "_"]}], 
  TraditionalForm]], "Equation"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", "2"], "(", 
    RowBox[{"x", ",", "y"}], ")"}], "=", 
   RowBox[{
    OverscriptBox["x", "_"], "+", 
    OverscriptBox["y", "_"]}]}], TraditionalForm]], "Equation"],

Cell["Define the corresponding functions:", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"f1", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
   RowBox[{"!", 
    RowBox[{"(", 
     RowBox[{"x", "&&", "y"}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"f2", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"!", "x"}], "||", 
    RowBox[{"!", "y"}]}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We can test the assertion that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["f", "1"], "(", 
     RowBox[{"x", ",", "y"}], ")"}], "=", 
    RowBox[{
     SubscriptBox["f", "2"], "(", 
     RowBox[{"x", ",", "y"}], ")"}]}], TraditionalForm]]],
 " by applying ",
 "the ",
 StyleBox[ButtonBox["Equivalent",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equivalent"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 " functions as shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TautologyQ", "[", 
  RowBox[{
   RowBox[{"Equivalent", "[", 
    RowBox[{
     RowBox[{"f1", "[", 
      RowBox[{"x", ",", "y"}], "]"}], ",", 
     RowBox[{"f2", "[", 
      RowBox[{"x", ",", "y"}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Duality", "Subsection",
 CellTags->"Section1d"],

Cell[TextData[{
 "We conclude this section by showing how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to compute the dual of an expression. We will define a \
function, ",
 StyleBox["dual", "Input"],
 ", to achieve this."
}], "Text"],

Cell[TextData[{
 "Recall that the dual of a Boolean expression is the expression obtained by \
interchanging conjunctions and disjunctions and interchanging trues and \
falses. We can achieve this in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " by applying ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") with a list of rules effecting the interchanges."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dual", "[", "expr_", "]"}], ":=", 
  RowBox[{"expr", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"And", "\[Rule]", "Or"}], ",", 
     RowBox[{"Or", "\[Rule]", "And"}], ",", 
     RowBox[{"False", "\[Rule]", "True"}], ",", 
     RowBox[{"True", "\[Rule]", "False"}]}], "}"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that this will not produce an infinite loop, since ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") operates by looking at each part of the expression only once applying the \
first rule in the list that is valid."
}], "Text"],

Cell[TextData[{
 "For example, consider the expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", 
     OverscriptBox["y", "_"]}], "+", 
    RowBox[{"y", "\[CenterDot]", 
     OverscriptBox["z", "_"]}], "+", 
    RowBox[{
     OverscriptBox["x", "_"], "\[CenterDot]", "z"}]}], TraditionalForm]]],
 ". As a logical expression, this can be written as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "\[And]", 
      RowBox[{"\[Not]", "y"}]}], ")"}], "\[Or]", 
    RowBox[{"(", 
     RowBox[{"y", "\[And]", 
      RowBox[{"\[Not]", "z"}]}], ")"}], "\[Or]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[Not]", "x"}], "\[And]", "z"}], ")"}]}], TraditionalForm]]],
 ". We calculate the dual by applying the ",
 StyleBox["dual", "Input"],
 " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dual1", "=", 
  RowBox[{"dual", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "&&", 
      RowBox[{"!", "y"}]}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{"y", "&&", 
      RowBox[{"!", "z"}]}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "x"}], "&&", "z"}], ")"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"x", "||", 
    RowBox[{"!", "y"}]}], ")"}], "&&", 
  RowBox[{"(", 
   RowBox[{"y", "||", 
    RowBox[{"!", "z"}]}], ")"}], "&&", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "x"}], "||", "z"}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Similarly, the dual of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     OverscriptBox["x", "_"], "\[CenterDot]", "y"}], "+", 
    RowBox[{
     OverscriptBox["y", "_"], "\[CenterDot]", "z"}], "+", 
    RowBox[{"x", "\[CenterDot]", 
     OverscriptBox["z", "_"]}]}], TraditionalForm]]],
 " can be computed by"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dual2", "=", 
  RowBox[{"dual", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "x"}], "&&", "y"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{"x", "&&", 
      RowBox[{"!", "z"}]}], ")"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "x"}], "||", "y"}], ")"}], "&&", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "y"}], "||", "z"}], ")"}], "&&", 
  RowBox[{"(", 
   RowBox[{"x", "||", 
    RowBox[{"!", "z"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Exercise 13 of Section 12.1 asks you to prove that the expressions ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"x", "\[CenterDot]", 
     OverscriptBox["y", "_"]}], "+", 
    RowBox[{"y", "\[CenterDot]", 
     OverscriptBox["z", "_"]}], "+", 
    RowBox[{
     OverscriptBox["x", "_"], "\[CenterDot]", "z"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     OverscriptBox["x", "_"], "\[CenterDot]", "y"}], "+", 
    RowBox[{
     OverscriptBox["y", "_"], "\[CenterDot]", "z"}], "+", 
    RowBox[{"x", "\[CenterDot]", 
     OverscriptBox["z", "_"]}]}], TraditionalForm]]],
 " are equivalent. The duality principle implies that the duals calculated \
above are also equivalent. This can be verified by the ",
 StyleBox[ButtonBox["Equivalent",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equivalent"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["TautologyQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TautologyQ"], "FunctionLink"],
 " functions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TautologyQ", "[", 
  RowBox[{
   RowBox[{"Equivalent", "[", 
    RowBox[{"dual1", ",", "dual2"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["12.2 Representing Boolean Functions", "Section",
 CellTags->"Section2"],

Cell[TextData[{
 "In this section we will see how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to express Boolean functions in the disjunctive normal form (also called \
sum-of-products expansion). We will first look at the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function for turning an expression in Boolean algebra into the disjunctive \
normal form. Then we will see the function for finding an expression based on \
a table of values."
}], "Text"],

Cell[CellGroupData[{

Cell["Disjunctive Normal Form from an Expression", "Subsection",
 CellTags->"Section2a"],

Cell[TextData[{
 "Given an expression written using the logical connectives, the ",
 StyleBox[ButtonBox["BooleanConvert",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanConvert"], "FunctionLink"],
 " function can be used to transform the expression into disjunctive normal \
form."
}], "Text"],

Cell[TextData[{
 "Consider Example 3: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "+", "y"}], ")"}], 
    OverscriptBox["z", "_"]}], TraditionalForm]]],
 ". In logical form, this is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "\[Or]", "y"}], ")"}], "\[And]", 
    RowBox[{"\[Not]", "z"}]}], TraditionalForm]]],
 ". We assign this logical expression to a symbol."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"example3", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"x", "||", "y"}], ")"}], "&&", 
   RowBox[{"!", "z"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"x", "||", "y"}], ")"}], "&&", 
  RowBox[{"!", "z"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The most common way to apply the ",
 StyleBox[ButtonBox["BooleanConvert",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanConvert"], "FunctionLink"],
 " function involves two arguments: the expression to be converted and a \
string representing the desired form of the output. There are several \
possible forms, as detailed by the help page, but the two we will be using \
are ",
 StyleBox["\"DNF\"", "Input"],
 " for disjunctive normal form (or equivalently ",
 StyleBox["\"SOP\"", "Input"],
 " meaning sum of products) and ",
 StyleBox["\"CNF\"", "Input"],
 " for conjunctive normal form (or equivalently ",
 StyleBox["\"POS\"", "Input"],
 " meaning product of sums). Be sure to include the quotation marks."
}], "Text"],

Cell[TextData[{
 "Below, we convert ",
 StyleBox["example3", "Input"],
 " to disjunctive normal form using ",
 StyleBox["\"DNF\[CloseCurlyDoubleQuote]", "Input"],
 " in the second argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanConvert", "[", 
  RowBox[{"example3", ",", "\"\<DNF\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"x", "&&", 
    RowBox[{"!", "z"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"y", "&&", 
    RowBox[{"!", "z"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that this expression is different from the solution to Example 3 in \
the text. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " produces a reduced disjunctive normal form in which terms are not required \
to contain every variable."
}], "Text"],

Cell[TextData[{
 StyleBox[ButtonBox["BooleanConvert",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanConvert"], "FunctionLink"],
 " can also be applied with a single argument, in which case it defaults to \
disjunctive normal form."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanConvert", "[", "example3", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"x", "&&", 
    RowBox[{"!", "z"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"y", "&&", 
    RowBox[{"!", "z"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To produce conjunctive normal form, ",
 StyleBox["\"CNF\"", "Input"],
 " (or ",
 StyleBox["\"POS\"", "Input"],
 ") is required as the second argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanConvert", "[", 
  RowBox[{"example3", ",", "\"\<CNF\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"x", "||", "y"}], ")"}], "&&", 
  RowBox[{"!", "z"}]}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Disjunctive Normal Form from a Table", "Subsection",
 CellTags->"Section2b"],

Cell[TextData[{
 "Example 1 of Section 12.2 describes how to find an expression for a Boolean \
function represented by a table of values. We will illustrate the built-in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions that accomplish this task."
}], "Text"],

Cell[TextData[{
 "To define a Boolean function using truth values, we will use ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 ". This function has a variety of uses, including the ability to specify a \
Boolean function in terms of the number variables and an index into the list \
of all Boolean functions on that number of variables, and the ability to \
specify a Boolean function in terms of truth value assignments. We will focus \
on the latter."
}], "Text"],

Cell["\<\
For example, consider the function defined by the following table.\
\>", "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {"x", "y", "z", 
    RowBox[{"F", 
     RowBox[{"(", 
      RowBox[{"x", ",", "y", ",", "z"}], ")"}]}]},
   {"1", "1", "1", "0"},
   {"1", "1", "0", "1"},
   {"1", "0", "1", "0"},
   {"1", "0", "0", "1"},
   {"0", "1", "1", "0"},
   {"0", "1", "0", "1"},
   {"0", "0", "1", "1"},
   {"0", "0", "0", "0"}
  },
  BaseStyle->{FontFamily -> "Times"},
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]]]], "Text"],

Cell[TextData[{
 "There are two ways we can represent this function in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that will be usable as input to ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 ". The first is as rules that assign the truth value assignments to the \
value of the function. For example, the third row in the table would \
correspond to the rule ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "0", ",", "1"}], "}"}], "\[Rule]", "0"}], 
   TraditionalForm]]],
 ". Note that ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " allows for the use of 0-1 notation for true and false. It will also accept \
the symbols ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 ", as in ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", 
     RowBox[{"True", ",", "False", ",", "True"}], "}"}], "\[Rule]", "False"}],
    TraditionalForm]]],
 ". We will use the 0-1 notation as it makes for shorter input sequences."
}], "Text"],

Cell[TextData[{
 "Given a list of such rules, the ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " will output a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function object."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BFexample", "=", 
  RowBox[{"BooleanFunction", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "1", ",", "1"}], "}"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "1", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "1"}], "}"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "1"}], "}"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0"}], "}"}], "\[Rule]", "0"}]}], "}"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 FrameBox[
  RowBox[{"BooleanFunction", "[", 
   RowBox[{"<", "3", ">"}], "]"}],
  BoxFrame->False,
  FrameMargins->{{False, False}, {False, False}}]], "Output"]
}, Open  ]],

Cell["\<\
Applying the function to truth values yields the appropriate result.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BFexample", "[", 
  RowBox[{"True", ",", "False", ",", "True"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "With a list of variable names as a second argument, ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " will output an expression for the Boolean function instead of a ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " object."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanFunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "1", ",", "1"}], "}"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "1", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "1"}], "}"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "1"}], "}"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0"}], "}"}], "\[Rule]", "0"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"x", "&&", 
    RowBox[{"!", "z"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "x"}], "&&", 
    RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"y", "&&", 
    RowBox[{"!", "z"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "A third argument can be used to specify the form, e.g., ",
 StyleBox["\[OpenCurlyDoubleQuote]DNF\[CloseCurlyDoubleQuote]", "Input"],
 " or ",
 StyleBox["\[OpenCurlyDoubleQuote]CNF\[CloseCurlyDoubleQuote]", "Input"],
 ", for the output."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanFunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "1", ",", "1"}], "}"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "1", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "1"}], "}"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "1"}], "}"}], "\[Rule]", "0"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "0"}], "}"}], "\[Rule]", "0"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}], ",", "\"\<CNF\>\""}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "x"}], "||", 
    RowBox[{"!", "z"}]}], ")"}], "&&", 
  RowBox[{"(", 
   RowBox[{"x", "||", "y", "||", "z"}], ")"}], "&&", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "y"}], "||", 
    RowBox[{"!", "z"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can simplify the input to ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " slightly by entering only those rules corresponding to one of the possible \
function values, say true, and then use a ",
 StyleBox[ButtonBox["BlankSequence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BlankSequence"], "FunctionLink"],
 " (",
 StyleBox["__", "Input"],
 ") to assert that all others have the other value. This is illustrated below \
for the same function,"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanFunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "1", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "0", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "0", ",", "1"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", "__", "}"}], "\[Rule]", "0"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}], ",", "\"\<CNF\>\""}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "x"}], "||", 
    RowBox[{"!", "z"}]}], ")"}], "&&", 
  RowBox[{"(", 
   RowBox[{"x", "||", "y", "||", "z"}], ")"}], "&&", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "y"}], "||", 
    RowBox[{"!", "z"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell["Observe that this is identical to the output above.", "Text"],

Cell["\<\
You can simplify the input even further by entering only the output values of \
the function, provided you enter them in standard order. The order the values \
must be entered in is the same as displayed in the table above.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanFunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0", ",", "1", ",", "0", ",", "1", ",", "0", ",", "1", ",", "1", ",", 
     "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"x", "&&", 
    RowBox[{"!", "z"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "x"}], "&&", 
    RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"y", "&&", 
    RowBox[{"!", "z"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "If you are in doubt of the order in which to enter the function\
\[CloseCurlyQuote]s values, it is identical to the order used by ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 ", the function used to display truth tables. Below we use ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 " to display the canonical ordering of truth value assignments for two \
variables."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BooleanTable", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y"}], "}"}]}], "]"}], "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {"True", "True"},
    {"True", "False"},
    {"False", "True"},
    {"False", "False"}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["12.3 Logic Gates", "Section",
 CellTags->"Section3"],

Cell[TextData[{
 "In this section, we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to work with logic gates, particularly circuit diagrams. First, we will \
see how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to translate a circuit diagram into a Boolean expression. Then we will do \
the reverse and see how to transform a logical expression into a circuit \
diagram (modeled as a tree diagram)."
}], "Text"],

Cell[CellGroupData[{

Cell["Circuit Diagram to Logical Expression", "Subsection",
 CellTags->"Section3a"],

Cell["Consider the circuit diagram shown below.", "Text"],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzs3QdgFNXeNvAhoYiIgCB49XotV33fT28RGxJIAUJVQEoEAQGDIIoFkQ7y
Ik0ERMSKIkWKgBQhKKIgcKWJ0hIgASmBgBDSk82Waec7Myc5HmYDl5Jkdmef
n2GdnS2ZnZDdh/9pd8W/1rFfmCRJb1xHLzo+N6zJoEHPjehUkV5pOmJw3/hw
uvEKvT2CbhjbBAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAEKGm/7RFaLKCiG5hKikgOhH
8lM2JixY/ME7Cz+dvnj5Nxt+TXelmrcSuVAnbpn4dONR9EEavaQPVNhz0efR
jD30Szeein7JNr40AAAAAGegWcur60T2ELlASz/1St94KTw87PrqQ4aO7NTx
KYkKC3+u/8ATf3i9OvHoHprJaBjTFJ0FM69GfJr5RDSqCWnN/FJsfm0AAAAA
wc6jFBLiohtKFjlzpEndO2+oet8PB7KyCZFJASE5p37/9aG619WTpKoV/rJ+
H/EQ4qNfumykMc2ssJllNENxRNMJ0hoAAABAKfERj6b4aDDznZk98LVbpVqn
T5E8YqQyVfMYLaVaJjn1a2QNqYpUTboxMjWPFBpBTjWymkpUhaUys7hmhjdi
Vt7YF2H7AQAAAOCqGT3L3ISc1nOO/P2GujdV+vuBg3k+1Wwflc2qmZxD5BMb
Z42vZbSJ1pu1eneucaNGdJnFM03TimpoOk9rml707EhrAAAAANfGy/6cTtv9
/U1GHqsthf9lX2qOEdXcRtpSjCT2h3xq5/+GS7dJYS++syrTbAwluo/oGotn
F6YyRTe+TLr/9wMAAACAK6GyXmfZmbu/v12SKkpVpcp3L9hxzMhjiqKpci7x
+UgeyT/SqEbl+ySp5cDZ51laoxlP9RBdNTqqqUZaY9HMLKwpRtuojtIaAAAA
wLXSzcqaRjwk60jDetUqS5Wkmx446DabNpUsQtxeo93TTQpOPVqv+l8kqWbM
wFOEyMYMHW5zdIJC8xoLamZO04pbQpHWAAAAAEqBkdZ0WTEaPguJ7s3PKlBV
1u3MTTTX+dQTu3/Z9eP3CQvmflCvRsVqklStycA0mtaMHms0x+Vq9NJsCNV1
mtoUjd5idGPTZCPuacWDRQEguKmq8ctMf83tPhAAgJCkK0RXFSOc6T5Co5eL
FGbv2fTDkMGDJKlSZen666Sw+/9+e5Poh6qES9dJ0nXNB6fqRoc1s7aWpxNF
9hFWSFOMWXONmty58+mffvHJF198vmj2/DkAELRmz57922+/sahGzMyGwAYA
YAOfh9XAdKNF9Mz6he/dWe26cKmKFF4zrt/ATdv35ecWEL2AeP74Z73rb5Ok
Ks2HnjamxqV/3Kqe5yOabq5bYI4R9WmaMaPH6dOn3505ffr0adMnTwOA4DVl
ypSwsLDc3Fz2bsGiGgIbAEB5U4xhn0axTDn5wYjON0lSuFTzr//viT2n5BxC
8s2J1wjJI7kpj1wn/T9Jqhw77CTrkCbn0Qf7iqfG1Ymm8lnXiOZRvWy0qA4A
QYv+Ejdq1CgnJ0cXcpqmoUMqAEC5MqtkClFPfzNrYC0jqlUNv/6x1HTi0Y2B
n7lmYCMki+QmNald+QFJqtR0xB/GHjdRC3QzyynGPLpejchGbDNaSHVN9aps
DSosZQAQ5GhaS09P14vrabKMxX8BAMqbxyiU5ZG09XfdKElh9Ot/ftjp09nU
uJqx1LvHSG3nSG7iwzcbLaGVm449TR+mZ5w7uu+RR1umnjdu1olLJbKxcKgR
3YiueX1EMZKajtoaQBCjv8RRUVGsJdTjMQrtOppBAQDKXSH9I6dn7l1UTZKk
ijffHv3yOXM9KWMQgc9jTs3hIvop4kr5nzrVa0qVr2s28Rzdpx8/mbS9atV7
D6cZc+iqJEsmblUvKrQR4pWJWlRqA4CgpapqdHR0eno6u8qiGh90AAAA5cNt
1NayVn0ypIqR1m7vMWZhhjH5raIbS7prRvSSzxP95LH9666XpGrSDZWixptv
3EeP7dscHnbPsT9YWstUiZenNV3JV4jm07GWAUDQi4qKysrK4jN4oLYGAFD+
zPbK079+O71qRZrWbnskbozR5qG5iJplNHAW3enwuD6P3BhO71CxbovpxlpV
asqxpF+kao/sc5nVOTWbPoSFNbbslNEqajzyvF2vCwBKRWRkJE1rbBtRDQDA
FuYKUxk5J7YbLaEVqkgV/7JxT6qXpi6fm2iarhnDOlcum3O9JEXW/wu9lG5q
ffxMFpFPfTXv0781ePq4bg4apelO87Kl4AlLa0V1tRw7XxsAXDOkNQAA25lj
BBSiF6yaO+N6o3hWWapc+6vvdxrNHrLLdf7kkBEjpco1lySsO7Zv099oWrvu
fzt37/ufhOXhFcJGzfwynSU0VSa6ohTN5mF0VitOa24bXxoAXDukNQAA28lF
y0PJRMk+k7h1QO8eUoWqUqWaUniVykZHNene+//1w/4TmfQuav7hNR9JVW6U
wqpVlCqNHT85j83vYSxeZUQ2H0toZnc1lSU3vLUDBDmkNQAA+5lZzSe7iO42
xoEWFqqqeuhU6q6D+xMTE8+fyaC3F7A5cnWFFObmE3dGdqGrwMhmHmM9UJ85
x5rxND72hBpLaxrSGoADIK0BANjPaL9UzenJNa1obICiGJ3QNDZhmqqau1Rd
MyZPMxaoYsHMY9Tj6P+9NOrRcGb0dONrGehsaQOENYCgh7QGAGA/1UtkpXhZ
diN+yWZkU1VZkwvNjmeyuYAU8WjGSqJEV2RV8Rgpje6WjR5rKlu1QFOEtEbM
C8zKBBDskNYAAOyn05Bl1sIU443YZ86Xa6Qymrq0AmNqDtVrrmhQ3C3NWHZG
8xKPzxibYAYyI63JKlFU44mK+q2Z5TUsPAUQ9JDWAABsV9xeaa41pRvNl96i
tKYQtYAoLmLO0eFihTIa04wERu/m0s2aW/HgT5lN3GE+lVmn09hTIK8BBDd7
0xr/jg4OipbXyK9qmjm+Xil6F+VLSGAtCYAQ5OGTeBhpzZyNg0/EoSusROYr
GkEgm7PmskzmNbKYxts9Ff5YHWkNwEHsSmssq5TzNy0f9BVpxfhO8fXKssy3
S9wDAKGGpbXif6oVVdhYDGOz3RbdZIQxr7lmKGvo1Hhzp3krfYhWPBTUfLDx
xqMhrQEEOxtraxkZGc2bN+/YsWOXLl3i4uK6du0aF+SeffbZ+Pj4119/fdKk
SbNmzVqzZs3BgwcLCgr4S+ZnmC/1xVIcvcqTG2prACHIZ4wxKKqnmRU2N9GN
MZ40bim87KaZC4Ya4w/cRT3T1KLqWXHJvmjLfB4zrekaKUpuABDE7Epr9Hut
XLmS5rSUlJQDBw4cPnw4KSnpYJA4ZLJcpfbu3fvbb79t2LBh0aJFM2bMGDly
JI1wDz/8cOvWrUePHr1ixYozZ85YzgNNa+JpF8txABA6dOLV/0xrxCyn/Vk3
01nXNL0ofelGYFOKGjrNoQn6BbN28BodayLVULIHCHa2pDVWPlq9ejXNM2wp
edZ9Sw9O4kvz30OlpqYuX7581KhRUVFRTzzxxPTp02lGJUKnNX5OUFgDCFUK
T1bil0kr/hJv+jPLqSyt/Xn1z7UM+CQeABDUbGwJTUhIGDp0KHFQQUkXBhSI
jZtECGaHDh368MMPY2NjO3bsuGjRIpfLRe/pmDMAAFdLETuh/RnVjP8VRTV2
VSidKbqx1pRc9OZS1EVNE1eeInx8KAAEMxvT2qpVq0aMGEHMtMa72QeFS5TX
aCrj9TH6utg2D2Pi1V27dg0fPpyefxrecnJy2E56Hsr5pwAAgUHm/dB0Y860
4sGeOhtlYC4gZa4tRd8hjJGh5nwdKinwETdb7IB1afMRxStGPB1pDcAJbOy3
tnbtWlZbc54S20PFAhq/9fTp0+PGjXv88cenTZuWkZFRfocIAAAAQcLG2tqa
NWuGDx9e/t830NCXT38EU6ZMadSo0bJly1ibqaUhFQAAAEIW0prteCo7ceJE
t27dunbtumfPHiLU4liLKvq2AQAAhCakNRvxoQd0g5+EhISE2NjY999/n5g5
DSENAAAgxCGt2UsXpsPlU+a6XK6+ffv27NmT/mj8hyoAAABASEFasxHvnCYO
I2UbiqJ8+umnMTExO3fuJBcZtgAAAAChAGnNdv6rUJHic/LLL7/QwPbtt9+i
sAYAABCykNbsxRo6/RdE4JepqamxsbELFy4M5bMEAAAQypDW7MXbQPlAA0uR
jZhzskmSNHfuXJuOEQAAAOyEtBbgWHLLyMho3rz5559/zvdYloMHAAAAp0Ja
C0C8yCaembNnz7Zq1WrlypUEZwwAACCUIK0FIHFdUXZy2Dqqp06dioiI2Lx5
MxFOGgYgAAAAOBvSWmDivdcsYezIkSNNmjQ5fvw4WkIBAABCBNJaALKcEFZh
Y/GMbi9btqxDhw4+n6/EOwMAAIDDIK0FJlZSk2W5xIbON954Y+zYsXzQKAAA
ADgY0lpgYh3ViHly2OQefI0qepXe2rlz5/Xr1/OVRgEAAMCpkNYCEDshvKom
TpbLL3fv3t20aVOPx2PfYQIAAEB5KN20xitCvObD18H0X+kSae1KWcaBjh49
eurUqWyP5YQDAACAY5RKWmONdOzhlrY5PrbR//mR1q6U5URlZ2dHRUWlpKTw
Dmy8qZQISyEAAABAUCuttMa3aTZjaUFszqPbYkcstoG0dnVoDOMBePXq1b17
9yZmVdO/egkAAAAOUIotoV6vl20kJSWlp6fz/Rcr8iCtXQXessyu+ny+1q1b
HzhwwFLAxOgDAAAAxyittCbW06pXr16jRo3nnntu9+7dYlRj26itXQv/87l4
8eL+/fuLe3irNAAAADhA6baEsgrP3XffHR4eHhYWVrt27caNGy9ZsoQNXUS/
tWvEu6XxRk966fP5WrRocfDgQXJhZgYAAABnKK3ampgfaFqTTDSw0cvq1avf
dddd48aNS09P573XCNLaVRFPoGaiG1988cWYMWN4F0F2K0YZAAAAOEMppjUi
1NZoSAsPD2eZrUKFCvSySpUqderU6dmz588//8wegrR2pcSVQ3lso1fz8/Mb
NGiQk5NDhNgMAAAAznDrrbeOGzduwoQJEydOfNvEtq8IfQh94Pjx4yeYateu
zUIaw4tsbGe1atX+9re/ud3uhISEUaNG8e7xqAVdi6FDhy5dupQIBTdkNgAA
AGdISUlZsmTJ4sWLFyxYQC8XLVq0cOHCxVeIPuSrr76iz8Ceql69epKABTZW
aqOXNWvWbN68OU0UNK0NGTKEXDglBVwFevZ+/vnnHj16EL8VEAAAACDYle5M
qqx57t577+W1Nd4eWr169TvvvHPSpElpaWksUaxatWr06NEIFaWCnvmoqKj0
9HTLqlUAAADgALynE/2IFzuxXxHVxDbuuusuVk9jbaD16tWLiIig2czn84nz
gCUkJAwbNoxc2B0Lrtrrr7++Zs0ato1mZQAAAGegAcky2yq5qg96y1oGf//7
39mwgltvvbVnz5779u2jOY2FMfbkrG/V6tWr2SgDfhhwdVjeXrt2LQ1sxG9e
OwAAAAh2PJ6xD32Wna4IX7Sdlc5oSLvtttvee++9zMxMsWJmmQ3s22+/HTp0
qHgTXB12Ss+cORMTE8OvIq0BAAA4A49q15iXxFR24sQJVkwTxyfyKMjvvGLF
ihEjRhDM6VpK6I+yRYsWaWlpdh8IAAAAlDJLiLqK1CQ2vfFlj/wv+bdjG999
993QoUMR0q4d73b4yiuvbNy4EbU1AAAAKBWYHbcUsaQ9Y8aMjz76iDhoyAb/
FwR/RY55aQAAAIEPaa208BbnDRs2DBo0iLUsO68rIH+ZAAAAUD6Q1kqXz+dL
TEx85pln7D6QUsNb2BHSAAAAbIG0Vlr4jHlZWVnR0dHEWauF8heCiX8BAADK
GdJaaeHZTFXVf//73046n4qi8K5rzmvbBQAACHBIa6WLZbZGjRoVFhYSp9Sg
/OdqRmYDAAAoN0hrpcjr9RLzTMbExJw9e5ZP4hHU+KujCY1d9fl8dp1hAACA
EIS0Vlr40l30slWrVrt37543b97ChQu/DH5z586lL4S+nEWLFs2fP3/BggX7
9u2z+3wDAACECqS1UqQXD5ykaW3nzp3vvPPO5MmT3wl+7FVMmTKFbdMNSZIs
FTb8/QEAACgjSGulSJZlttG0adPTp0875pTyJlG9eBjFY489hrQGAABQPpDW
Sgvris9qa5GRkbm5uaSk/vnBiP/doHGUvcB//etfSGsAAADlA2mtdNHTSBPa
gw8+6JhRk/qFE6yx+uEjjzyCtAYAAFA+kNZKC++0lpOTExkZ6ZjzyV8Iy59s
5amHHnoIaQ0AAKB8IK2VFhbVVFVNTk7u3LkzcdAp5VPj8kSKfmsAAADlBmmt
dNEws3nz5pdeeolVopxxVvlMawzdRr81AACAcoO0Vlr4Cfzss89mzJhh78GU
It5jjU0ox14m+q0BAEAI4p+JbBTh1X320cfymRbYE4p93dkecS0kev+EhIRh
w4aJ3xquBT29NP3SsyoOEXUY+rcL/dYAACCkWEbbkeJxhVe3SBArgNCHZ2Vl
Xc43Xbt27ciRI/lVfOZeuzZt2hw9epQ492QirQEAQAgSKzCKolxdjYt3Amef
m/fdd1/9+vWXLl3qdrvZHfjHq1j2Wb16NUtrDq4FlQ922jMzMxs3bnwtBdLA
h7QGAAChhn2yWzqlX0Vqsnxc3nXXXZIk1a5d+5577hk3btypU6fEZ+aj/Fat
WsVaQi1LeMOVYidw8+bNffv2JY5OL0hrAAAQgsRlfci1FbhY9qOXNKRJxSpX
rnzHHXd07959+/btrL2VFdPod1y3bh0bZQDXjp7P0aNHL1q0iDi6Som0BgAA
IYjXtfhUCVdXW+NJjJi1tTATjWoss1WoUKF27doNGzacM2eOx+Nhj1q+fPnI
kSNJcX4rxRcVguiPoGnTpidOnCCOTi9IawAAEIJoSMvLy/vpp582bdq0cePG
zZs3s+2rQB/LLm+55Raa0GhaYzmNBbbw8HC6XbVqVXrrSy+9RD9wExIShg8f
zgc4OLgiVNboqdu3b1/nzp0dfw6R1gAAIDS1a9eud+/er7766ssvv0xz1Cuv
vPLyFRowYAB9OH0s3aYPr1GjBgtpPKqx5Mb2VKpU6YYbbqAfuN9884043xo+
c6/FlClTPv74Y3GCMnuPp4wgrQEAQKhhH3NPP/10UlJSqTwh+xi94447aDCj
qYyFNF5kq1evXoMGDVasWKEoCo0TfHZcuEyWGe2IsOZUw4YN09PTxZscmWGQ
1gAAINSUYlrjfc/o5X333RceHs4bQKtUqVK3bt34+Pi9e/ey8QXsEmntKljm
QmFWr149YMAAtug5EUbd2nKEZQppDQAAQk1ppTXLCkF33nkni2o33ngjTW7T
pk3LzMxUTeL3Zf3WCFpCLxuvpFmGdXTo0OHnn3/md3PwlGtIawAAEGpKsbbG
BwsQsyW0Vq1asbGxa9euZUNNS5xRzZLW4L+ynCiWyn766ae4uDhLYc2pkNYA
ACDUlG6/NdZpilq/fv2BAwcsfav4PCH8/mgJvTqWPEZ/fBs2bGDbjs8tSGsA
ABBqSiutsQKaJUXwUg+fxo3t4VU4pLWrw84naw/duXNn27ZtyYXrRDh48jqk
NQAACDWlPiZUXOGd7bH0eBcTnSWt4TP3cogtznS7devW27ZtY1d5MPZPzo6B
tAYAAKGmdMeEilfFtUdZFYh/pPINpLUrZalSfvbZZ6+99hq/SoScJrY4OwnS
GgAAhJpSr61dEZ7W8GlbIssQWovTp09HREScO3eufA/KZkhrAAAQapDWAhZv
0+Qnh5UoeSWtf//+X3zxBXF0LzV/SGsAABBqkNYCEzshXq+XXaV5zNLQ/Nln
n8XHx9P9ltzieEhrAAAQapDWAh9fIYJdZeNAmzZteu7cOacOJbgEpDUAAAg1
SGsBzr+VMysrq1WrVjSwOX74Z4mQ1gAAINQgrQUyFtXE8Z4ul6tz586ffPIJ
68NGhGVDQwTSGgAAhBqktcDkf0IURZFluWfPnlOmTOE7HT8Xrj+kNQAACDVI
awGLZzA+c1p4ePjQoUP5iIPQPGlIawAAEGqQ1uxlWeKhxKW7KI/H06dPnzff
fLOcDy8AIa0BAECoQVqzHa+V8WKauDYB3XC73b179x41ahSf0COUIa0BAECo
QVqznRjSxP1sXtyMjIy4uLhJkybZcWiBCGkNAABCDdKavVhCk2WZ1dP4ygXs
ctu2bY0bN549ezYxe6+F7FkSIa0BAECoQVqzl2U4J28Spdls5syZzZo127dv
H7vJqau0XymkNQAACDVIa4FAlmVx4/jx431N2dnZlj5sth1iwEBaAwCAUIO0
Zjtx0XZq6tSpERER8+fPJxcu6Q4M0hoAAIQapDV7sZZQlsrWr1/fpEmTESNG
ZGdnkwuXMLDzEAMM0ho4libTPyrRdEJ/9zXjQmVfdEtWiOYlxOgPoRN2OwCE
DnvT2urVq4cMGcK2ndHMJ86fZolblilt+a00eNDU2qpVq169eu3duzekFia4
Ckhr4FiaYqY1GtC0PyOZ8eWX1tgXAIQMG9MajSU0pYwaNYqY/bXokTijIz1r
0GTbfMl1dp7Z4p78Zebm5i5evDg2NjY+Pn7Pnj08gSCwXQLSGjiWztMauyqm
NYXu9LGbkNYAQo9daY0FEtYS6oyqGsNjA93gC69bbqX7N27cOHjw4IiIiJEj
R+7du7fEZ4ASIa2BY7Eh4WYBzSzHF0c1IcWZ8/sQNIUChBq70hr7vqtXrx4x
YgQbCEkzmzNqSmzaNMtO+hqPHj26bNmyl156qVGjRvHx8cuXL8/NzWUvWazF
KYripPha6pDWwLHMv8VCDU0zerLpRvunLka1orTmhJYIALhMNqY1KiEhYdiw
YeKRBDuauzwez7lz5/bv379x48ZFixaNHTu2V69eNKE98cQTY8aMWbduXUFB
ASmpudO/Fgf+kNbAscwkxtKa2T9NIZqb6C76r70/o5pWlNZ0Itt8tABQjuwd
ZZCcnFylSpX777//H//4xz9N/why9evXf+yxx5o3b96tW7dXXnll4sSJS5Ys
2bFjB01ofD1Q3m+NldF48yjbicx2aUhr4FiWtEYvNBrVXLoxxKC4ombmNt1o
GMWqwQAhxMa0xqeEdRLLmFD/EaBsT4mvncU5BI9LQ1oDx/JPazrSGgAY7B1l
IG44rLOWmB/4elLiTWynZVoP/8eCBdIaOBb9x5qq0b/KXo0NC9WI7qHXVKJc
kNaM6T00BS2hAKHE3pZQgCuFtAaOpSmql8azopEEqq6ZgwvYfLnFndaKamsK
0hpASEFag+CCtAaO5fMYIwt0cw4P3RhZ4CXETYrrbITPwGYMCFUwgwdAKEFa
g+CCtAaOpbo1rzEClKgFNK0phOQS4ipKa0YlrXhqXMVMawAQQpDWILggrYFj
qS6ieTyuLNf542lHkrbsTNx1Qs4pCmletjCV2XtNpoENaQ0gpCCtQXBBWgPH
8mV37fxERUmqJknXS1TN2x7tmkajmdEA6iZE8fLkhrQGEGKQ1iC4IK2BY6k5
yxZ83ve5uOqSRL+ksDr3xL5ylrC0ZrSQeopmzUVaAwg5SGsQXJDWwLHUHCLn
Ej0/98Svt1alca32bY1fPKWbaU0toGmtkKU1zU00pDWA0IK0BsEFaQ0cy2PM
i+slbpL9e4vqFW+SpLsjn/rd2G2ODtWIiw02UDSi+rw0uClmelOLlqNSipaD
18yBpMZ9FfNxZjmO3g0zfgAEMaQ1CC5Ia+BYPiOt+Yy0dqxJjetqSZX+HvHM
UTN4EVknulIUvVSa1lTjF0Ar/jL/+hev/K6Zmc64r1qc5pDWAIId0hoEF6Q1
cCxV9hkDP420FlPj+hrSDfc+3uc4Ifn0Jq2A6AVFK1JpGmGLHRiDQ4um0jXo
rLqmFf8ymOuk8OXgdbSdAgQxpDUILkhr4FiqQtOaUVvL/T265nU1pBv/3uDF
oyytkRxCjLTm04tmxjUnYXOzaT3MHKaZow+MFlO1qMjGV6vCPLoAQQ9pDYIL
0ho4llLcEpp3KPKmCjWkaveaaS3PuC2H6IXFtTWV6IrPWC20aAnR4iXgzbRm
rldV1ABKzJKaUVXTVNteFQCUAqQ1CC5Ia+BYmpHWVJrW8g80ri3dKFW577EX
jvO05svzKcXBzOiWZvzxGTFMKR5hYKxXZcYzohZFOJbWZFaCA4DghbQGwQVp
DRxL1czqmcusrUk1pCr3Nuh73GwEJSSL6C7WxGnMkqsaaS2fhTbjwlvcXU0z
splWVF4zG0MVvg4CAAQvpDUILkhr4Fiy7CGKixSQ3JTYG8NqSVXuaBR/uGgG
j4zfU35Z+/NP6zf+tHfTpszUVMWsuZlpTTaHjPKmT9kspvHGUI1fBYDghbQG
wQVpDRzLI7uIkkWyScHvbaqF1ZHC7mnx3BFCft6RdG/1auEVpH82bfiPB/9d
y1zpoFm7VnuPnvGZNTVV1Vlak2WzJdTIb0Z9zRgJqilEN1KbjKEGAMGva9eu
+/btI8Wfevjsg4CFtAaOZUyMqxaSPJJzsElVqZ4k1W/Xc9DsNZJ00/SR49UC
l4dGMl0j2ZlTxwyVKkphUuUFi1exdlCPUT+jgUzRdWM6NnPMqKIbLavCPB4A
ELR009NPP33w4EFxp42HBHAJSGvgWB7WdplHcpOi/iL9T+3rpfCaUtW/zVm1
yWjI9LGVDIiRznwZy5d+dqMkhUtVJn74VZ55I9F9RPPohA1VoH885hBRVTcL
a/gNAXAAmtb279+vaRoLb3YfDsBFIa2BY2lEUXVdziYFhx6tLdWUJKlC7Tc/
T8ghxc2bitHuqRkd2QqIlj+iW6vr6H0q1t1wMKPQeLxM45lPUwtU1ktNJloh
0eQCl7dHr+f7PN8/HgCC1nPPPde7d++bb7752LFjBC2hEPCQ1sCxvLo5JrSA
uI5G16pQQ5L+9lDbFHN5UKNtU/F5db1oCQPVrRNf7i/Lq9G0VqXOI2375BsL
TelE9+l/FuE0NqGH2+3+bff+X3bt2QUAQWv37t07duzgzaCqaq4ph88+CFRI
a+BYPq2QkFyST3JTmt9QoZZU6Z6mzx8hpMBY+dNNw1weG9opK0T2GgNFC/c0
vKeGFBYmVbn5x8RThcXrFxjTd+jmMFHdnO5DV3QW8wAgaLF4RszPO03DqCEI
dEhr4Fg+o4xWQApJfnKTG6XaUtU7GvU5TEMZvUmmSU12sSk7jJWnZA+NYp5f
x73YplIFSapSffSsr7PYXB/mSgeabi6LoBbNtyb7CjGDB0CwMwZ9CzSTXQcD
cGlIa+BYCluewEvyExsZaxlUvLvB878XzY5bQBTZXJeKmCM93QpRibx744Ix
1xtd1yoNmDb3D5brVJkoZpRjQw9UH9FcZj0OAIIY/5jjCQ1RDQIZ0ho4lk/2
6B6jwFaQFnl9lZukG/4a2WMvIWeJOQ+HbIz09BltoblEzzcHHqR9/8XYOjSt
Sbfe9dToQ2z9d7XAGG2gs4EGXuOBRUu8F9j62gAAIIQgrYFjqS4PUdw0c+Uf
iKou1ZGq/m9E/8PGolNmBvPJNHp5jQqcbM6rRnembpj7lpHWwuvcFN0vmZBM
YtbWZHOEgfGMstGNjXVg0732vjgAAAgdSGvgVBrx5BPNY4wJ3RdbR7pFqvKP
+i+e0M1V3fUMIheyyTu8ZrXMaAPxHN4y762bjLRW7eaoHsls/XdFK1p0yvid
0FS2QqhujioFAAAoF0hr4FT0r3GBEca8JD85+gaa1io++Fj/VKPhk5gTrBmr
uhfV1nSfkcfce3/8dLTRb61yzR7jZp1mLaG62WyqFQ8OZcNIdT6zLgAAQJlD
WgPH8pEClRTSHJafGlu7Yh0p7O5HehxnFTPdTf/vI7KX3qp5iOYlukYK/jO8
c1QFqap03Z2z1u/JNoOZrhblNOHLbBbFFB4AAFBekNbAsVxG3CowhoaeaFxD
qiFJ/2rx8lFCaFAjag7Rc1gLpzmDh2JUzNK/e6T29ZJ0p1T90d3nFONuRtlN
oXnOZzaAsunXimttaAkFAIBygrQGjuUx1jLwEJea9Vt0PXPlqRseOs6WkaJp
TXN5iTlkwAhtsleXTy9/u5YxIPQfzwxeYNTfjMZOoylVJbLP+FKK0poxelTR
kdYAAKC8IK2BYylGnzSZnCOFyQ1rVKhrJLG6r78912WGNNmovZnJTdGIN5O4
jzaqXbWKVE26/pED54hHM0d9+nKM9auIVzG+zPobW3+KJTcAAIBygbQGTqUY
CxDkEXKK5J15vMbtN0vSvXffLlWq+vma/emaMdYg3+iv5iGqi3hP/V+Ph6pJ
N0jSTeu2/+4x1w41R30qZqzzEiKb7aHErMx5zRUN7H55AAAQMpDWwKnMv9MF
hPxBsnL+X437G9/3MPGd/Xj++1KlmyfM+v5soS6TfC/J/mXX1n/eVuM2SapY
5b4dSWmFbBwBjWr0PzaigGYzc2CBl5XrjM5sGtIaAACUG6Q1cCpzRrR8QnKU
Qk/dWvcunrecyLmEZH6/ebVUMfw6SapCE1pFSQoPCwu/bt7Hc9LNSpyLeHU9
31xwiijqn6MKdHOpUGNUgtFvTcNvCAAAlBukNXAur/nlYY2YHh8b0ZlDyGlS
eOrk7t+O/LL3cFJSysnjLrMPm8vMdoWsrVP2mvN3ENmnsynXVN2MasVzrykY
ZQAAAOUFaQ0cSy1UNS+bGk3lTZd6IdELWOnMiFyqh8Y0H1EKzWly3caSVIqZ
ymg+KxpYUPQMxMxpRdPjsgVGAQAAygPSGjiWXnyhFxXENGNKD00z+50VKsay
U+bsa3lEyTE2NJdmzNTBlinQNI+HzYur8qlw2ZLuRlFNQ20NAADKDdIaOJXG
F/dUdVZb01hHNI01dhJjzlvNq9OApmlGbzRZM2/SZWORd+OesszWCHWbzaRy
cfxTdIJRBgAAUH6Q1sCpNLPxUteMqdK8rAGT7jEimU8nBbrRSU0mrMFTNYck
FLK1pmSdeI1+aroxMa45siCPkCxjsSqdPqWiGDOwaVh5CgAAyg3SGgAAAEAg
sz2tadqf/X/Y97U3K+om/23xDuIGveQvQdzm2J4Sn7PE5wcAAAAQ2ZvWeLY5
cuTIl19+uWDBgnnz5i2wz2LTkiVLli9fnpCQsHHjxt9++y0lJSU7O5ufIv94
yTYURRHvQK9acl2JDyR2p1MAAAAIcLbX1mTZGIH3+OOPDx48ePLkyVOmTKGX
b9uEfuuJEyeOGzfurbfeGj169Guvvda7d++4uLjGjRs/+uijUVFRzzzzzMCB
A+k9aZzbv3+/2+1mr0Isr5ELi2Y8xVloJkQ1AAAAuDR70xpLMqqqJiYmNm/e
nB3GxeJNudEFbA+NVfTY0tPT6XFu2LDhyy+/HDVqVNeuXR9++OGmTZv27dt3
5syZW7ZsycnJIUJyow/h4Y2+Rv8gx79dub48AAAACCq219ZojGHfceDAgfPn
z+cHYAvLsZW40+LcuXM0p33yySfx8fENGzZs27bthAkTvv/++8LCQv3CllDL
hn8PNwAAAAB/ut1pjYUWmtmOHDkSFRXlcrnK87v7889RfJiA2HDJrooVM2JW
BQ8dOrRw4cLXX3+dnlWa32bPnp2WlkaEghtrABW/nb2vFwAAAAKc7WmNCElm
xIgRM2bMKM9vfQk0ifE2Wf0ibZeW/fSFiGMNNm/ePGHChOjo6Pbt27///vvH
jx9nhUR+B+Q0AAAA+K/sTWv8G7EiVVZWVqNGjfLy8srnu18mXei9VuKZsTSk
itU5Vkk7cOAATWsxMTHPPvvs119/XVBQQIpfMrlwrCgAAACARSDU1tiwUPZ9
33zzzVmzZrGrYuYpz+MpO5s3bx4yZMhjjz02bNiwgwcPWjIev5tujkogJdX0
UI4DAAAINfamNd4yyDcOHTrUvHlzVncSa1ZiG2JQUxQlNzd3yZIlsbGx3bt3
/+mnn8SSHQ+u5MKfgqXLXHkeMAAAANjL9toajWGWGlp8fPyKFSssd3NMeY3P
mksvN2zY0LVr1zZt2nz11Ve8YZTxeDxsw7JQAqIaAABAqAmQfmu87Y8Gkl9/
/bVt27bEryXUAUFFzJy8jLZv374BAwY0adJkyZIlqkkcZ2pJcWgMBQAACDUB
0hIqxhi6s1u3blu2bGHbDHFKeU0cWSBeHjx4sH///q1bt/7+++9JST8CZ7x8
AAAAuFK2t4SK35oHkqVLlw4ePFicQMMx/dZE7EWxtmC2/csvv9Ck2qlTp+3b
t4t3E9tDUVsDAAAIKbanNUsHe3bV4/E88sgj+fn5YjhxRnHJ6/USIXSVWEPb
sGFDmzZtaF7NzMzk7aHOePkAAABwpexNa7xDmtiBjXWnp1ll2bJlzpvEg/Mf
7io2jNKdM2fObNy48ZIlS1iOtSwcDwAAACHC9tpaiWgy2bZtW5cuXYgTZ127
fKmpqd27d+/WrdvBgwfZHnE+YcseAAAAcKTATGtMs2bNjhw5QgLmeMofC6ir
V6+OiYmZPXs2X41UvJUT52oDAAAAxwjYtEazx4wZMz755JOL9e8KEWwGj4yM
jF69evXo0SM9PZ0IZ4MnNwAAAHCqwExrrGq0b9++p556KhCOJ0DMnz8/IiJi
3bp1RFiNlPdzc+SwWQAAAAjMtMbQ+NGwYcNz584RB608dUX4aqF8Y//+/e3a
tXv77bfZHbC6AQAAgOMFZlrjIWTMmDELFiyw92DsJU5gwrbz8/OHDx/evXv3
8+fPs5sURQnBIRgAAAAhIjDTGilu6du4cWO/fv1Cs7DGiLN8iOdh5syZzZo1
O3ToEFZ7BwAAcLbATGv8ANxud4MGDWhKCc0Bj6xiJv50xLnptmzZ0rBhw+++
+46E5PQmAAAAjlRiDKtfv74lrZUn3sBnyRs8lvTu3Xvbtm32HFyg4j/H5OTk
Nm3a8Mk9GM1EQrWzHwAAgAPwdMQ2HnzwQZbWLrEcUtlhRTMeF1kDKI8ZdOOT
Tz6ZPn26oii2V/wCCj8b+fn5bdu2fffdd4kwUJRjE4DYcHwAAABwDSw9nXhL
qH9a08sYmy5MLAGJ2yx77Ny5s2fPnuV1boKD5QeUk5PTrVu3MWPG8JW8ENIA
AACCF49kxC+tlb8ffvihVq1a8+bNy8vL44fEIgfvqJabmxsREeF2u205woDF
mjt5uqa5t2/fvkOGDOHz5VqWHwUAAIBgIRbQ2Mc977dmqa2VQ3Fm8uTJkiRV
q1btrrvuGjp06OHDhy13YJGjc+fOiYmJSB2cfx8/Gm7pD5GezJdeeon3W8NK
BwAAAEHKEsYeeughu0Zcvv322xUqVAgPD6cxo3LlyrfddtuTTz75ww8/sOPh
mWTq1KkLFixA0x7HWzxZdz5+Zmi4HTBgwODBg8U9th0lAAAAXC1L7PnXv/5l
V22NpjWa01ha426++WZ6SJ988kl+fj4LGytWrBgxYkRZH0wQsUy/xjbYHLle
rzc+Pn7s2LFsJ71q43ECAADAtaMf8ffcc8+BAweOHDlyuFhKSgq7LGtvvPFG
hQoVWEhjmY1fveGGG+rVq9ezZ8/jx4/v378/Li7O7lMVWFgrJy8/ijU0l8v1
zDPPTJ8+vdxSNwAAAJQuvXh6fPZZ//LLL3e0yf33389zWlhYmJjWmMqVKz/3
3HMej+fRRx+1+7QFB5biCgsL27dvP3fuXCIsNsrvg/wGAAAQLGz/1J48eTKL
Z2JjaKVKlWhI++tf/9q9e/e9e/eyyShiY2PT0tLsPdrApwvzDGdnZ0dHR2/Y
sIHdxItvvOBm+08fAAAASsSnyODDBm381J4yZYqlDbR27dp333033X/u3Dlx
ppH+/fv/5z//ses4g4hYOE1OTo6MjExJSbHcgQ9SsOcQAQAA4OIsH9O2z4nx
zjvvsLRGo1rdunVjYmJWrVrl8/nEI2RFoalTp37xxRe2HmwQ8P+Bfvfddy1b
tszLyxNHj6I/GwAAQCBjDYu8XUyWZRsz2+TJk6tUqXLbbbf1799/z549bKd4
eAy9unDhQhrY7DjGIMMDOf3Jsjw2Y8aMZ5991tJ7DTN7AAAABCz+ae7xeOw+
FnLq1Knx48dnZGSwq+IKoUQYB0GPdvXq1YMHD7bnKIOH/0pezKBBg2jcJcKC
7wAAABCY2Oc4/7wWO6XbQuxBx7C84b980s6dO+Pj48v9AIMVy728i1pBQUHT
pk03b95MSjrnAAAAEFD4iuokMHouWeYN4ywziaWlpbVq1aqcjy3oiP0SLZeJ
iYlRUVGZmZkkMH7uAAAA4AA8sLEp+jHl2lVj8eyDDz7o27cvuTDIsRIcr8IR
lN0AAADgsoklIEVRGjZsSDMb6kJXgbeAd+vWbd68eeQiowzEzAYAAADwX4kz
uNIUER0dnZOTI8YJJLfLx1qcz58/HxkZmZqaynby8ppl7lwAAACAS+NNdeKA
iNjY2LNnz1qKP0gXl481ei5cuLBXr16W2pp4GlFeAwAAgKtA48QTTzyRmpqK
eHYVxIk76EZcXFxCQgLbKS4YEThjTwAAACDo0AjRsWPHw4cP86sIFVeKt3se
OnQoJiYmLy+P7ZdlmQiTEuPEAgAAwOXw75/WrVu3/fv3W3YiWlwOXlvjZ3Xi
xIlvvfUWn9dOnHkPLaEAAABwOcQYxrafe+653377zbITae3ysRoa673m8/mi
o6OTkpLE5Q9wMgEAAODyiW2dNFrQyy5duhw4cADdq66COLSWbYwePXry5Mn2
HREAAAAEN7HrO9/ZoUMH3m/N/55waYqisKgmy/Lu3bubN2/ucrnsPigAAAAI
Vv4DFak2bdqcOHHiYveESxBra6qqduzYcf369fYeEgAAAAQ7MWCwolBsbOwf
f/xB0GPtarEz9sUXX/Tv359gNAEAAABcG//5WiMiIrKzs+07ouDGqmpnz55t
3LgxDb1er9fuIwIAAAAnEJtEH3744RIzBops/xWfoKNDhw4rV65kO0tcKhQA
AADgcvC2Tp7EcnJyGjduXGIwQ1q7TJMnT3755ZcxsTAAAACUFt6xSlXV1NTU
1q1bE2Szq0JP2o4dO5o1a5aXl8dHhtp9UAAAAFAyPg8q+9S2vTmMHUaJg0DF
Jd137drVq1cvOw4wiPGfcmZmZmxsLD2H4o8bowwAAAACFv/IZrPO2litSk5O
jo+P//3334kQLRhL8Wft2rXDhg0r7+MLQmxtAnYy+U/2lVdemTZtGk/CloQM
AAAAAYUvDWl7VY2iESI8PPyWW25p167dpk2bLnFsc+fOxdz7/xU7b3zOE3b1
7bff7tevH7+PJRUDAABAALJ8WNsY2yZNmiRJUsWKFellnTp16tevP2vWrNzc
XCJUhyh6hDTXLVy40K7jDBZiPY1ZtWrVE0884XK5fD6fZXwBz8YAAAAQOPwn
MbPX5MmTWVSjKlSoQC9vvPHGW2+99dVXXz127JjYbBcfH79582a7jzdosPLa
nj17Hn/88VOnTvH9lhZSAAAACDSWwQX8g1u/ULkdzzvvvMOiWpiJxzaqbt26
LVq0WLdunc/no0fbrFmzjIyMcjuw4MVD+NmzZ5s0abJt2zZSvEio+OMOhKwO
AAAAJaLJxzIa9KmnnooqFl2+7r77bpbQeIWNZbZKlSqxq1WrVu3SpUteXt5j
jz2GgHH5zp8//+STTy5dupRcWEQVMxvOJwAAQAAS58QgxSW1Bx54IMNPZrkY
M2YMT2j0kjWGMrVr177zzjtHjx6dnp6+d+/euLg4W89c0KAhvLCwsEOHDp9/
/rlYQ7NsIKoBAAAEC/qB/tBDD7GpPMSd5fPdJ0+ezHuscTfffHOTJk2WL1/O
j2rZsmX/93//h95W/vjwT1LcvinLcrdu3aZMmSLex6ajAwAAgFJgb1qjoYIl
tPDw8Ouvv75u3boDBgxITEzkrbSs2e7tt9+ePXt2+RxSUGA/ID4lHc9jiqLc
dNNNb775Jt+PiAsAABDs7E1rkyZNojmtTp0699xzz3vvvcfm7uAzS/AQ0rFj
x+TkZAQPETsbLNayS/pDjIuL+/TTT8mF7Z6orQEAAAQ1e9Pajh077rjjjnXr
1vGp1fwPgEa4Bg0aYHIwC35CWIXN5XJ17dp1zJgxxG/ALwAAAAQ1e9Oa/7KV
PITwHvLbt2+Pj48vn+MJFnyQCDuB58+f79ix49SpU+ke//UgUFsDAAAIavam
NYb1jSfC+pU8adCN999//9NPP0WZyIK3hObl5bVu3fqjjz4Sb+InEDVJAACA
YGdvWiuxVxX/7qwt7/nnn9+xYwdSh4iHsaSkpJiYmEWLFrGrltk5WAYOhAVh
AQAA4KrZXlvzbwwVS0Nut/uRRx7xer2orflbsWJFdHQ0W5CL9/rzn2ANAAAA
gprtae1iWNj48ccf+/bta/exlBM+OuBidxB/Lu+8807r1q2PHTtmWZkCAAAA
HCYw0xrvRT98+HC2fJLjl7bkk6cRv557/IWzndnZ2f379+/Tp4/b7Wb7sTwB
AACAgwVmWmPHoChKZGTk+fPnA+SQyoGY2YjwqumpYFFt9+7dzZo1mzp1KgmB
BAsAAAAkUNMaCyF79uzp3LlzKMwbZmkDpdmMvWp6Kc4VPGPGjJiYmB07drD7
WDqqYSAGAACAIwVmWiNmlendd9/98MMP+VV7j6es0dAlLkwgonvOnj3bpUuX
F198MTc3l15lQY5dYrUCAAAAZwvYtEY1btz4zJkzfKCo3YdTTtgr9Xq9pLit
c9GiRTExMYsXLyYX75xm+QkCAACAYwRsWtuyZUv37t1JyPTOYlNwWJpEk5OT
n3rqqfj4+NTUVPGevNIoFuIC4acGAAAApS4w0xo9gIEDB65cuVLsmuXgzCZO
CMxeptvtnjhxYnR09Jo1ayz34cSqo+0/MgAAACgj9qY1/2XcWVf57OzsiIgI
l8sVvJOJWbKl5ao4R4d4K6uwrVixonnz5mPHji0sLPQfO4BgBgAAEFJsr62J
MYYXi5YuXfrGG2+IVaPgiih8ZQFy4XhP/+glrr1Otzdt2tSuXbvevXsfOHCA
jzvgz1kehw4AAAABxva0Zvl2LJ+0b9/+l19+4bcGaVDRi/GrbEOspPHtvXv3
duvWrUOHDj///DOfXc3yVJbnAQAAgFAQIC2h4nT927Zt69SpE8snYtesIGoP
ZZNsXKzTnaVauGnTpp49e7Zu3Xr16tX8PpZZ5i62DQAAAI4XaGmN6tGjx7p1
6/wPI0hHGYgVNst6UsuXL2/Tps3TTz9NXy+/g1hGK3FmYKQ1AACAkGJ7S6g4
Vz+9TExMbNmyJZuoP3jb/vjUteyq5fjT09M/++yz6Ojo/v37swZfsVfbxQqJ
wdsiDAAAANfC9rTGIw1bZWnYsGHz5s3jhxG8Y0IZsR7odru//fbb+Pj4hg0b
Tpgw4ejRo0TIYDy1sp+FfyERaQ0AACA0BUJLKJ9k7I8//oiMjHS5XP5HElxZ
RTxsGsN++eWX8ePHN2rU6IUXXli/fr3YSY8IL83yAvnOIHrhANfIMpjashO/
CwAQmmyvrYl9tIYNGzZr1qxy+9YXc4ke/kToh2aJmuTC3FVYWLh27dqhQ4c2
bNiwW7dus2fPPn/+fDm+CIDgw/51Q/x6tIqz2VhmtgEACAW2pzWGvkUnJyc3
adLE4/HYOJqAdfJn27o5ge2li3uWPW63e9u2bTNnzuzbt++DDz74+uuvL1my
JDMzsxyOHMAxeB6z/H6J0xUG6ZgjAICrY3ta4++6/fv3X7hwIT8AW1zOyxfv
TM/bkSNHEhISJk6c+NRTTzVo0KB79+4ffPDB1q1b2UAJ9hDL6p8A4I/1XCVC
KvP/xdHNvqyX/g0FAHAe3da0xt94d+/eHRsbS9+c/Wf7L09isya58Dzk5OQc
Pnx4+/bta9asoXns5Zdfbt++PT11bdu2HTx48Pz58xMTEz0eD7kwzvHl1wHg
cvDfHbG8xneKg5LsO0YAgPJmb1pjaEJ75JFHYmJi3nzzzZEjR44aNWqkTcaM
GTNixAiavgYNGvTqq6/26dOnU6dOrVq1+sc//hEZGdmxY8fnn39++PDhH3/8
8YYNG2h4E+dP4zlTF+buYFANALgc7Jfo3LlzXbp0OX78ONupX7ioLt9Z3gcH
AGCfQEhrVGpq6sqVK1esWPHNN9+sXr16jU3ot05ISFi3bt0PP/ywefPmXbt2
paSkpKWl8fYX/zMjJjFLaY5Ngcuv4vMF4NLY78jixYtfe+01+s+3l19+OSkp
id3Ef7n04pEIAAChw960xt51eaoR15myixjMSuzJZrmJEOsABN5HmhcExD5s
AFAi/k+bJUuWvPXWW/RNica2Zs2a9e3bd/v27ewm/juFXygACCm219Ys/VIC
yrUcFf/oQR0A4IosXbqUpjVS/Au4cuXKJ5988tlnn926davl95G/e7CrlgZT
/z4JAABByva0BgAg4mmNo+9I3333XceOHePi4jZu3KgLs+uIQxKIUJ+3lLvx
ngYAQQ1pDQACin9aI8UdDDZv3tylS5f27dsnJCRYSmfsDpZqG8prAOAMSGsA
EFAsac2yBBW1devW3r17t2jRYvny5bIsX6y/q7gHyx8AQFBDWgOAwEHffPxr
a6qJb7ONPXv29OvXr1mzZgsXLvR6vXz4DwZiA4DzIK0BQODwT2uWnMZnzWVv
U8nJya+++mpERMTs2bNzcnLsOGQAgDKHtAYAAaXE2trF7swqaadOnRoxYkTj
xo1nzJiRl5fHEx2fPAfvaQAQ1JDWACCglDjKgAiLGlhmxeF57MyZMxMmTGjQ
oMG77757+vRp8VYAgKCGtAYAAeViae0yZWVlvfPOOw0bNhw/fvypU6dIScuM
8quWiT4I1koAgICEtAYAAeUa0xrLZrm5uTNmzIiIiBg9evTRo0eJUJqzhDRx
HVIe52RZvvoXAABQ2pDWACCgXGNaY9jgUI/H8/HHH0dHR7/xxhvJyclEiGFs
cjZ2VVzqVy9pEXkAAHshrQFAQLn2tMYTF3srKywsnD9/flRU1AsvvJCYmEgu
XOyAbfDWT8ymCwABCGkNAALKtac1Fr38u6V9/fXXsbGxvXv33rp1K9spVtIs
+Q0AIHAgrQFAQLnGtGZJX6xWxnayds+1a9c+9dRTXbt23bJli/+jUFsDgACE
tAYAAeUa0xqPWzSAWRYSZZes8vbjjz926tSpY8eO3377LbsD2y82oQIABAik
NQAIKKXSb028KlbbxIIbvdy2bduzzz7bqlWr5cuX80XhUV4DgECDtAYAAaVU
xoRekd9++40tObpgwQL/ydYs032wDZ7oLramPABAKUJaA4CAUv5pjZihiy05
GhUVNWvWLI/HI8Y2PmWuOHKBbvP5QPCeCQBlCmkNAAJK+ac1cda1Y8eOjRw5
smHDhp988glbJp4vNiqW0XizKcEbJgCUPaQ1AAgo5ZzWePpiVzVTWlra2LFj
H3/88ffffz8rK4vuZ2+S/3URBACAsoC0BgABpfzTGl8XXtygAezs2bMTJkyg
mW3cuHHnz5/n97e8YWJdUQAoa0hrABBQyr8l9GKTfjDZ2dlsydFRo0axZeKJ
2XjK3icta1cBAJQFpDUACCi2jDKwrA1KMxufeI29H7pcrpkzZ0ZFRQ0aNOjY
sWMEM7MBQDlCWgOAgFLOaY3ntBLnW+Nvhuxubrd7/vz5kZGRL730UnJyMlYX
BYDygbQGAAHFltraFZFledGiRS1btuzTp8+uXbvIhX3e+N38sxxCHQBcHaQ1
AAgogZ/WGK/X+80337Rt27Z79+4///wzufjcuf4sDa8AAJeGtAYAASXw05o4
CFSW5XXr1nXq1Onpp5+mG2Jgs0Q1Ns4U764AcBWQ1gAgoAR+WmMsc69t2LCh
V69eTzzxxJo1a/hNfLEDcTZdAIArhbQGAAElwNOaOCmuZaY1TdN27Njx3HPP
tWzZkr4Kj8cj3odP40bwHgsAVwhpDQACSuCnNctV1r4ptnImJSUNGDAgKirq
yy+/ZJmND0MAALgKSGsAEFACPK0xfEI2uiFW2FjpjM2d+/vvvw8aNKhx48Zz
5szJzMxkd2CPwvIHAHBFkNYAIKAERVqzEEtnlsVDU1NTR4wYERUV9e677+bm
5hIMCAWAK4e0Bk5VSP/Qv8j0r7ZKvIR+KYS4iSrTq0Qzv3T6n6YQTWX3JBq9
UMwvc+YsmehuQmT2HD6207hRNi+hrARjWrsEltnS0tLGjx8fERExZcqU9PR0
8Q7+S8Pzqh3bg1naAABpDZxKYZnKDGa6EdmMbGYGLc34P8tsRFGJbNxk/JVX
LkhrNJXpXpbWhJ0K0RWW66CMOCytcbIsZ2Vl0bTWoEED+gJpfuM30bdcPnqU
FPeFY9tsMCnekwFCHNIaOJdshjGawWQjYhmVNGJW0hS2RcwChkq8Zl2NmDHM
2FdcW1PM8tqfEY4U7TQ28RtSdpyX1hRF4UNBqYKCghkzZjRq1GjUqFFHjx61
NIzyd2M0mAIAh7QGjqXStCYr5hdhCc0IY0pRWjNra6q5R2fVt+KimVoUxjSe
8VQez4w9GmFBD8qG89Ka+I7Ka2g0s33++ecxMTGDBg3av3+/uHCVWExjG0hu
ACEOaQ0cSzWyltgAqgspS8hgitncKRNNZklM/zObkQuuFuU3pLWy5bC05j9T
Lp/rg77x0p3z589v2bLlCy+8sG/fPuK35AGaQQGAIK2Bg+nFX0bXNaNB08sa
NM2O2z6ieY2toqEEHqMzW1GDaYnPYf6vKK3pGGZQlhyW1ohZGRNHCljeYOmt
NLOtXLmySZMmPXv23LFjB9vPJwPxfwgAhBqkNXCqopTFBhSoRcNDzVYlY6eZ
1mRjlKgxlEAxJzCVi0pnQklNqLNpxZ3WkNbKlsPSGns7ZROsibN8lDjxWkJC
Qrt27eLi4jZt2sTvjzGhAIC0Bk6l88JaUXsozWZ5RHOz8QXmaALtwhGgSnFD
Z1F7qGrpsWYU32SktbLmsLQm4s2a4iUpzmM8m9Go1rlzZxrb1q9fjwVGAYAg
rYFz6UXDPBX6l9sonRlR7fT5w3uXLvpu2rTFH838YvU3K38/dd7FopfPYyQ7
Ft90RVV8ujg+1OixZk7ogbR22cSakv9N4hhJcmH5yMFp7XLwYQXbtm179tln
27Rps2LFCktpjvi9S/sPQ7AkQ3LhSeYzuVmGM/ClGSzd5wDAXkhr4FyyQpRC
oiqaTDS3ev74oD49KkqVK1a+7Y0hE+I6tKssSZWkCi88PywtTVM0UqixsQP0
fx4295pXM76KdupF068hrV0mS/d4/ukv5gr/+f9JyKc1IoxHoOdn165dffv2
bdas2aJFi+h+y1S6/oHq0vE4KSnp448/7tatW9u2bWNjY3v27Dlp0qStW7da
vi8meQMINEhr4FSKlpNPCP0iWgFJPxpx6x03VL1ny768QmNSDvoXPuvU0e31
61WuK0mVpbvX7SUuNgzBWNlAZqlM4bOyFY1H+DOtYUzo5eC96/0jhBgGLP2y
kNYor9crXj1w4MCrr74aFRU1d+7cwsJCfsZYWhNn1mV4fYxi2ydOnBg8eHDr
1q2nTZv222+/nTx58o8//ti7dy99wmeeeaZz5840FuoC9jyYOQQgQCCtgXNp
2cb6UzLxnZk7sP+tUq3UUyTbjGQqyVZJHiGZ5MzWRrVoWrtJqtb5aAEpahU1
ZvPwqaqsWtJa0ZpTmjCnB1yU5W2EXvV4PCXeZOmahbTG8DokbzU+cuTI0KFD
IyMjZ86cmZuba0m57P4lJuTff//9lltumTNnDnurF084m7l3w4YNLVq0WLx4
MX8I+ssBBBSkNXAslQWtTDX30H03VK8bdu/BJMWtFU2Q6zU6tbmJemzD58Nr
SNT/fJTwa6Y5btQcTWCU2WSdpzVSvKDBhROywcVZMoP/u4o4rwVaQkWWpGSZ
se3EiRMTJkx4/PHH33333fT0dHH6XP+TTPckJSX9+9//3rx5s3+/NbG8efLk
yQceeGDLli38Z4EPAoDAgbQGjuWmn2H0IyntcOJ3FaUKkvQXSap37OhJTXf7
aFxjQz/VjMIzW++4TqojSS9MW3qOjUfQVHNAgcJra0UDDYqjGlwmMYPRPDBu
3Lj//Oc/lpzg39YW4mmNZyoedLVi7CrLchkZGRMnToyMjKSXNLPxs2pZET4/
P7958+Y0g7E3eXEONyI0mLL70+eMiIhITk7m3eHQEgoQIJDWwLF85uJT2tlT
Sd9dZ1TPbpKkO7/dnFg08k0m5keWm7iOPFwn7HZJajnow7NGy6lxszE6QTPW
c/fpfFiowbLMAVwm9qFfq1atm2++uX79+vPnz8/Ozibmz4HdhNqaSOyKxkMa
X/5AvOf58+enT59OI9aoUaNOnDhhGYdL7zx79uw33nhD7OfGn82S61h3tS+/
/PKll16y3BkAbIe0Bo5lLlJAtFySuffxW8MqSFWlmtEH882eaXKWMUrUuJOL
5B+JqHv9XyWpTkzfk8Z1oqtaUW1NI8KEbOZTIq1dIbGFrk6dOmFhYTQ3V69e
/fbbbx86dOjx48f9322Q1sTymmUWDsayMy8vb9asWZGRkTSYpaSkkOL6G32e
li1bHjt2THyUf7nM0vQZFRV19uxZPua0TF4hAFwhpDVwLNls1zQWcTdGFmTk
Zek6zWK5hOTQz7eMkyn7d276/oflny/4sGatatWk8Koxw0+y1d/VAqKoPo2N
R5CNBKfLbCkEr24shKBrmMDjslhCRd26dWlUq1ChgmSqXLnyX/7ylyeffHLj
xo1ihW3x4sXjxo0rMaWEMv8htJap0jwez5w5c2JiYl599dXExERiztfRvn17
dn/L6lclYud/7NixCxYscMyZt9QbkT8hSCGtgXN5Nd2cI9dYkyCLaFnEl/ef
rVteHT5QCpfCKoZLUsW/33tndMxD1SsYraQ3Nhl6nI0yUFzGuqI6caks67mJ
ruh/9mEzJ/HAr8h/49/EWadOHRrSWHlNRPc/8MADH374YUFBAb3bkiVLaFoj
lxcwQgpfr4oILZhESMV0j9frpSewRYsWvXv3/vrrrwcPHiw+g/9cHxxvD6Vp
efLkyeQiU7cFI8sUf/iAg2CEtAZOVUgyC9kst4qb+H7fuGDaLdVrS1JtKfyG
rv36/fjLb+kFmjEZh/vcQzdX+psk1Wry+mlzaIKxAoIsaypbt8qYfk3TFNVs
ED2YfCCmWeM2bVrFNm0ZC/9NkyZN6CVNDs2bN6cbVapUCQ8PZwmNbrAiW8WK
FfkeeknMltCxY8cip12C2Hbp375p9MqU5VWrVtHz2bZtW7p9mW/p7G6bN28e
MGDA8OHDO3Xq1CH4dezYkb6Qdu3ase24uLj69esfOXKkDH4sAGUIaQ2cKp8U
GNGLfuh70z4Y8XR1IxHU+9v/tEs+4fKabaSFrFCWm/rg9RX+TmNc9MCTutH6
qRljE4oWGNWNwaOyMWWuucfrc/2RforKPJt9Fi7p/Pnz9DI9Pf2M6fTp0/Xq
1RNbQukGq7PRjbp16z788MOzZ8+mSWPJkiU0rREMSPRjmbqWCO16vNRGL+k5
nzx58mOPPTbYxB976fPJR3wsX778rbfeOn78+OHDhw8ePHg4yKWkpBw6dOj3
338/duxYcnIy3f7444+nTZtWVj8kgLKBtAZOlc8aMdVzqz4dYYwJDasqVX8k
Nd2Yn4Poiqzl+4xolkdyTzaqXft+Kaxm1KCzbHY13WjoVM0RBZqxCpVsDiNl
tyk6m0DXIW1EZci/qa527do0nvHyGlWlShUa4bp3775r1y5SnD2WLVs2YcIE
9HK3sNTTxNhWWGgMZabnKi0tjQZdNhVbRkbGtm3b+vXrxxo0LROnXMKnn376
3nvvXeadgwUfr0HRfxS8//779h4PwJVCWgOnMlbu0XK0U1tvr06jWkWp0t/W
/ZrpZatGqTINchrJ0UmGlnPq8Vp17pOkG6LHnDESWsHpY8n1H2uTep4lMnNB
Ktb/zSi1GXU2Y6iBesGHJvhjPwXxU7Ju3bq89bNmzZp33333O++8k5WVxftH
sY5qX331VYiPCb1M7HSx03vs2LEhQ4Y0atRo1qxZOTk57A5ut7tBgwa5ubnk
wj5vl3g2uvHkk0/u27ev7A+/nOgXjsWgl/QUTZ8+3daDArhiOtIaOJRRHPOe
zdyz/HqjU9Rf/xbTN103WzlVRfWxFQryiHaa5J74d81qt0pS1SZTM+g+5WRq
0q6wavcn/mHOvaZnEM1tLOpuRDXN/DKn79Au2lsbGEuHdhoG6tSpEx4eTjNb
48aNV6xYIRbf6AbPbEuXLh0/fjyfuLWcDzuQsVWiyIWLuaekpLz44ov0lM6f
P58N0xDP6htvvPHll19ezngB9oSHDh1q2bLlxZYfDVLiC6F/D+fOnYvaGgQd
pDVwKqN+ppz/7sPXjGbQyvfEvbU4i+7VXMQr65qRxFSSSdSU7F9X1KJpTpIq
Rk4x7qAfObH3P1KlB/ZlGJN9EHKeqC62iJW5ZJXmU1mXNu+lvzsQYQUldnnH
HXf069dv7969/A6WtxoWKtgMHsRBYxJLhWUtA7qxY8eO/v37R0dHL1u2TDGJ
92T3OXr0KL0DW1T0cpqVn3nmmZUrVxKhElU2r6b88JfAS4uzZs1iTb0AQQRp
DZzKWFtKyUpaPa0mTWuV6v2z24hzxGwGNXqikQKjQpZJlMRJvRvfWIHGuUo3
tZqeR8y0lrRDqvrPQwUk33iaDKK6xbRmdmxDWitDoTY7rlgo04snBxPfhC1v
yPTq1q1be/fu3bJlS5qsxHZn/2eml/PmzaO/AQUFBfwb8UtLs/Vrr702cOBA
8Ts6pt+geIpmz56NtAZBB2kNnMpY8VPOLTy27SZj2KEkVbnp28TUQsXsiSYb
s+DKJH/11x/UkKSI+ndKlSTpppZHzmYR/dRX8z6959FOaZpxR2MqXRrMNDbT
mqKzMQrGfoc0EgWgEElrujk9mniVvfFaVroXO5vRFPfTTz916dKlbdu23333
HSlOeiW+Y4ujEmbMmNG+fXu2BLx/AKN7CgsLn3/++aZNm7LPAhYXnfRBgLQG
wQ5pDZzKmGzN6HDmXj1nBo1kVSpK0nU3fLV+m/EZJnvzM9JeHzVSqlJz8dp1
hxM332y0lt7XqcdzG9ctDQsLG/vewmw2U64xAZuisvGg5jpUxdsOqTkEoBBJ
a4ylhqYXT54mZiq2nZCQ0KFDBxrVNmzYcBVRasmSJQ0aNKCxjS9NxfafOXNm
4cKFjRo1evPNN/0PzDGTqCCtQbBDWgOnchOvl9XBZN/Z/dsH9u4ghdFIFiZV
pP9JVSXp7vsfTUg6f4YY40CPrJkuhdeUwqvQ1PZ/Eya7imZjI6ri0f9cKlRh
s+0Wl9egTIRIWrNMsG+5VexttWLFiubNm/fq1WvLli2WO1+6ax/rXc8bQAsL
C6dOndqmTZtmzZrFxcX17NkzNja2YcOGo0aN4muJsoEMfMH3a3+ZAQJpDYId
0ho4lblolKx6C9la7JrXRbS8YycTdx/cmZj06/k/ztCbs811Q407FGR4FDUr
Oz+/UPaZrahF/dJ08mc80xXjy2wMdUjFISCFSFpjxJ5jrJmSpziatRYuXNi0
adN+/frxKTV4gvqvAzb9i2P8e505c+bIkSP79+8/ffr0pZ/EMYENaQ2CHdIa
OJnmIZqLfuK4dJa+ZKJmmzOtGVOmKZpszp8mF60lSm/Vi3qnedmqoKrOVjQg
RWsbKMa0a2Z+w2DFshNSaa1EeXl5c+bMadSo0auvvnro0CG2kwUnPokHc+n3
aj5mgfh1kxPXG2X3Ect0jmkA5ZDWINghrYGTqT5jYjTNCGAemsx0M3HJOv37
bjZu0vRVSHSvrpojPWlakz3GNG3mwu3eonUN6P80c1V3xUxrRR3WkNbKToik
NV5P4yNAaUbKz8//4IMPGjduPGzYsNTUVPFu4mN5x7b/GqvEB/I7Wx51sXEN
jplvjSCtQfBDWgOnMupm5v/YlB00cXmMWps5a65mzHIr6zyMmUtQ6bIR7RQf
GwHKymv0o8uYCkRTzLsrvFlUwSiDMhMiaY3hb7ZnzpyZPn36448/Tl97Wlqa
/0JRfPjn5b8/89wlLj5FhOEM4jFYlvoqcfRo8EJag2CHtAYAASV405qlSEUu
7PfF848lBdGcNn78+IYNG06aNCkzM7O8Dja0IK1BsENaA4CAEqRpzbJUlriW
liWe8Sx39OjRUaNGNWrUiIaHvLw8y8QdUIqQ1iDYIa0BQEAJ0rRG/FYisLRj
svDGLg8cODBw4MDo6OiPPvrI4/FcLNdBaUFag2CHtAYAASV40xoR4hYvtbFK
Gn9TTUpKeuGFF5o1azZ37tzCwkLi12EMygLSGgQ7pDUACChBmtbEZdP9N+ib
6s6dO+Pj41u2bLlkyRKv12t5LMEq9mUJaQ2CHdIaAASUIE1rInGSW7qxZcuW
7t27P/HEE9988w1/d2UDM8XVPAkqbGUGaQ2CHdIaAASUoE5rrGjG30J//PHH
uLi4p556av369WwPm8RMnMqMDS5AWitTSGsQ7JDWACCgBHVaI8UTo61evbpN
mzbdunXbsmWL/1RmpKQubVB2kNYg2CGtAUBACcy0xstflp3Eb72Ar776qkWL
Fn369NmxY0e5HiJcHNIaBDukNQAIKAGV1izz/6smtodt8AECHo9nzpw5TZo0
efHFF/fv34930YCCtAbBDmkNAAJK4KQ1vmCTZbSmLMt8ZU96mZOT88UXX7DF
PZOTk/n7p+V9FWyEtAbBDmkNAAJK4KQ1EVs3yrKSVG5u7owZMyIiIoYPH56a
mkrDG++TZpmjA+yFtAbBDmkNAAJK4KQ1lr7ExaRIcZPouXPnJk2a1LBhw3Hj
xp05c4bdwf+dE++lAQJpDYId0hoABJQASWvi2yBvCaX57fTp02PGjImJiZk6
dWpWVpY40EB8CN3GbLeBA2kNgh3SGgAElABJawxv0KSp7OTJk0OGDImKipox
Y0Z2djbxi2dsQxyJgPnTAgTSGgQ7pDUACCgBktb426AsyykpKa+99lpkZCT9
oC8oKCAXiWE8pPk/CdgLaQ2CHdIaAASUMkprlsZKce0AMWXxlk22sXv37n79
+rVs2fLLL7/0eDyolQUppDUIdkhrABBQyq62xme4FRfoZMRuZuz9cMeOHT16
9GjTps3y5csLCwvL4nig3CCtQbBDWgOAgFKmac2yRxzySYpXY9+0adMzzzzT
vn37b7/9VlzQk26jthakkNYg2CGtAUBAKbuWUHEgAN8vLuL5/ffft2vXrnv3
7uvXr+eTp5HiqTxK/ZCg3CCtQbBDWgOAgFJ2tbWLvbMpivLNN9+0aNGiR48e
27dvZ02i/tFOHOwJwQVpDYId0hoABJQySmus0xrvt8YWJvB4PPTbNW3atE+f
PomJiQRzbjgU0hoEO6Q1AAgoZZTWWFmMXrI8lpubu2DBgpiYmP/f3p3AR1Hl
eQAvbuUGQQ7HGRRnnd3Pzg6s40IuEg5BQVEEhqCjoozAcIlc4TCAERgRBQVl
AAcRuc8EAgwIeAICIjdBAoLcEEISkk4fVV319lW9zuOlurlCN+mq/n3Haaur
q9tOJen65R3/169fv8OHDxNhWU/+AcjnHbDxbASfjZaFtAZWh7QGAGElpLMM
qPz8/I8//jguLm7YsGHHjh0jwpA23rAmFvpAa5sNIK2B1SGtAUBYufW0Zvqk
4rHKlK/43ZycnI8++qh58+bJycmnTp0KxpsFa0BaA6tDWgOAsHKLaU1coDPg
o2I53HPnzr377rtNmzZNSUnJyspi3ZpoNIscSGtgdUhrABBWbr1tTWxM4wFM
rJBGnT17duzYsdHR0ZMnT3Y4HKRoNBq6OCMK0hpYHdIaAISV22pbI0bR2oAH
HD16dPjw4VFRUdOmTSsoKBDb4m7cLgf2g7QGVoe0BgBhpQSzDExzBA4fPty/
f//WrVvPnj1bURQx17Ejedk0fNZFCKQ1sDqkNQAIK7eY1tjHFO/WZPbs2dO7
d++EhIQlS5a43W6xsK1pPXeCcWuRBGkNrA5pDQDCyu2OW2ORbNu2bd27d2/f
vv3KlSt5UTUi1EwT6+KaXgFsD2kNrA5pDQDCym31hLJF2Dt16vTMM89s2rTJ
tGiU+FEmNqmJ00UhEiCtgdUhrQFAWFmyZMn48eNv2gImy/L69etpSOvWrdv3
33+PFTzhBpDWwOqQ1gAgTLDRZcuXLx8zZowpffEOTdbLuWzZsieffPLll1/e
vn07KT6GDcAf0hpYHdIaAIQD3kG5ePHicePGsZ3ijE6a0woKChYtWpSQkNCn
T5/9+/ez/WwoGkt3CGwQENIaWB3SGgCEA/axQz+LaB6bOHEiKV781ul0zpo1
q1WrVoMGDcrIyCDC4p6Y4wk3hbQGVoe0BgBhZdmyZcnJyWybBrC8vLxp06ZF
RUWNGjUqMzOTBJpEwJIb0hpcD9IaWB3SGgCECbfbTW8XLlyYkpJCN7KysiZP
ntysWbO33377woULpKi3VDWQojFspfuewRKQ1sDqkNYAIEywT56lS5cOGDCA
Le45adKky5cvE2MGqCmYif2kplcAMEFaA6tDWgOAcMA/drZv3y5J0rRp03Jz
c/ngNH4An1BA/CrfAlwP0hpYHdIaAIQP+uFDwxibCooPIgiW8ElrrCk4MzOz
UqVKTZs2feyxx5o0aUKvwk0t7r//+7/p19KqVauuXbv26tVrzJgxCxYs2LZt
G/2bi3/V/LvAavX4N4+Lf52xneJthENaA4Cwwj7G8SkEQRQ+aY1JT08fOnQo
Kb56mtXJspyVlZWRkfH9998vW7Zs/Pjxr732WlxcXPv27ZOTk1evXp2Xl2d6
Cl/A1/8MiKV77sKbD39IawAAYG/hk9ZopCFGWktKShL3WJ04YsHUVvbLL78s
WbJk8ODBsbGxr7zyysKFC1mDGyn+fWGDHHh+I8JQB7StEaQ1AAgP+NiB0Amf
tMasXbt22LBhARe0tS42hoFti32aHA1d33333ejRo5s1a0bDGytwzUeiisfz
BGvqQo1kSGsAECa0IqX9RsBuwiqt0SiyZs0aGlqIUQ6a2OKaa/oSxLusxYwI
JXfy8/NXrFjRpk2b119/fevWrfx4GvZ4TmNPwaAIDmkNAMIHPnwgFMIqrVHr
168fNmwYa4myWTef2GAoDjkzDVFjd1euXNmxY8cXX3xxx44dpkkH4uxvfCwQ
pDUACCf48IFQCJ+0xqLIqlWrRo4cyfbYoz3Z1DAuDl1joVQckGb6ejdt2tSm
TRt6QrKyssRpF6al5SIc0hoAhAnTZQsfRBAs4ZPWmDVr1owaNYrYqGHNtAwc
2+b9mLyVjO/ht2zD7XZPmTKlRYsW8+bNE1/Q1EAXyZDWAADA3sIwrY0YMYJE
9tXW1PVJU9mZM2def/31nj17Zmdnmw7zb2eLtFOHtAYAAPaGtBaG/Ct1sI3p
06fHx8fv3r07YDE6epfNRBC7ViMB0hoAANgb0loYCriEAbNr166EhIRZs2bx
I9nBPNcRG3Ui3yKkNQAAsDektTAk1tElQiRjD+Xk5HTr1u0f//iH2+1mO8XB
b0xEnT2kNQAAsDektXBjmh8acGJsYWFhjx496IkyrSsamfMOkNYAAMDekNbC
jdiMJu433aXh5IEHHkhMTORF2NigNUVRTE1ztoe0BgAA9oa0Fob829PE1jMe
z6g+ffoMGjSICPNDIyqnMUhrAABgb0hrYShgiTbi10lKjJa0v/3tbykpKeL+
SAtsSGsAAGBvSGtW53K5EhMTP/7449J+I6UGaQ0AAOwNac3qZFnOz89v1arV
tm3bIq1VjUFaAwAAe0Nas4fdu3fHxsZevnyZ3Y2o2Ia0BgAA9oa0ZnWsuhq9
nTdv3ksvvRSBRTyQ1gAAwN6Q1mxDUZSBAwfOnDkzohrWCNIaAADYHdKa1Ykn
6ty5c3FxcWfOnCnF93P3Ia0BAIC9Ia3ZAyuNS82cOZNVYIscSGsAAGBvSGs2
IC7p7vV6n3766a+++qoU389dhrQGtqW56e+3pv88E00liur2EtlFVP0Xnu5V
9XGr+q1RZ1EjjtJ+uwAQKmINfJrWPvroo+C+vljf1X8FTPE9sD00rY0aNYrd
FUMI3IA4s4Cew+3btz/77LP+o9fsej6R1sC2VLeqKjSi6T/OemCjP+SyQrzG
B6gvpBVtUO7SfKsAEBr8Es8v659++mno2tZWrVr1448/su2AEY5tp6WljRw5
ku+PtAHzJUZPl5jZunXrtmnTJv4Quc7q8PaAtAb2pdDfavqT7aH/kmlco0nN
JSQ0RTMO8IU3gk9LAHsyXdGC2xPK4gFvz2nUqFHdunVjYmKWLl3KVh4nRWGM
xQx2u3bt2qFDh9INt9vt/w7BHz9F7ASy0/7111936dIlYOtlqbzJkEJaA9vS
ZJloBaqm/27TX2KvrHd9sqhGkxuRPUShP/fGbz5vYQMAuxEv9KwndNq0aVqQ
iK9P49kjjzwiGapXr/7QQw+lpKScOnVKfBvsNi0tbcSIERFYNOwOsTPGsjHL
Zh07dty6dSt7FGkNwJI0t0ZU+oerR9X0tKboUc2rGslNT2tumtbcPK2haQ3A
puhFnDfI0Nu5c+dOmjQpWC9uulz+9re/pVGtQoUK5cuXpxuVK1du2LBh165d
v/32W34wfTNr1qwZPXo0EQbUBev92Jup45jGto0bN7700kskUJe3zSCtgW3J
+USjP9mqV+/91KOam41O07OZL62hbQ0gQvDAxtrWgvWyLBuoBvrivG3N5De/
+c3jjz8+f/78vLw8YvSEJiUl8RfBNfemNGOGiLiH5Rb6bW3ZsuXp06eJ3U8j
0hrYlvsKUR3ZV7OyLp87cTRj2479O45l59A/x+hDXhreZK8e5HzQIQFgS6zX
zOVysbv06rZu3bpatWr1DJJevXq9+uqr7LZHjx5Vq1YtX7582bJly5QpQ0Na
uXLl2AZToUKF6tWrO53O1NTU0aNH06SBq+1tYd9NXnWN7Zk0adKMGTNI8Umj
9oO0Brblyn7hubZSOalcGakS/aAsW/3+/23/KyGFxEhratHvu6Z3jnpu9EIA
YGHixEy2vWvXrh+CZPv27fR269atOw3169fn2YznNBre6G2NGjXoo8OHD6dh
Iz09fcSIEXbtswsF0+xa1pjJ9hw4cODZZ5/FnFAAq3JnL5wz7eUeXe+RpOr6
R2bV/3zy9WOE5BOjJ9QrCzMOFFdpv1kAsChx+sDDDz/MchqPauXLl69Tp050
dPSqVavYDFCC6rjB1q5du5MnT7KWN7ueUqQ1sC05lxCnohXk/bq/cQWpglS1
QbPuJwm5SozCuapbSGtqYWm/WQCwOlVVH3roIZ7TKleu3KBBg1deeWX//v2s
k443ByGtBQsbzzZu3LgFCxaQ4p2kNoO0BrYlF3j0RjQ3yT4eU7NSNanSw9Ev
H/OlNQfRnEVlcvU/x5DWAKAETO05bJZB7dq1H3744enTp2dlZfF5oOIoNaS1
YGEBeOPGjX369GEnE2sZAFiN3mLmITLJ+6VNzXK1pQp/iH31KOsJ1SObU2GT
C4wibOgJBYCS4YPbZVl+9NFH4+Li0tPTiREbrncxRVoLrry8vOjoaD6YzZYD
ApHWwLY0kkNoDJPJ1czWVaS6Uvn/iv3bcWIsCKpdJZo+gsT4q1hhq1IBANwu
cdAajW35+fnsrhjVWFVeccYi0lqw8LPaoUOHY8eO2TKnMUhrYFuK3r/poHkt
70jbGlI9qdJDf37Fl9bUAqLJLmNdKn1yqCrbeeI3AISSmBD8y4IFzA9Ia8HC
T+DQoUPT09NtfD6R1sC2PKwcrpsUHGpZQ6ollX84us/PLK15nURTitKaU78L
AHD7xG5QcdwUi22miylmGYQCO9WfffbZ+++/z/bYsoUNaQ1sq1BfsaCQFOhp
rZZUXarwu5h+vrSmuIlKCn1pzUG8Diw9BQAlw1MZ7w/lD/HyX6ZCuEhrwcJP
4KZNmwYOHEhsGtUI0hrYmKKHMYWms6uHYmpKNaUaj/7fmyd8swwuEHJV0Yje
A0r/UfRPV7rfoxhD2WT6y+7UU5ym6HdV9vEr0zsOY/aofrTsLt0vDgDCgbi8
O7V///7z58/zR3mDGyl+YU1LS2NrGdzNt2pvR48eff7550v7XQQZS558lmvT
pk15iRLkNLAPtz7TM5/+7+rhdjWk2lKlRrF/O8KWCnVfysz4cf3WHzZu3rZv
y9c5Z056iOpikw70Nd/pL4KTaIU0pbG7xiqiqqwX0dX03lW3m9hzjjgAlASf
jXjPPffUq1fvxRdf3LlzJ2/k8U9lq1evHjZsGHvIxiXC7qa8vLy4uDiaYUyt
T9YldqyzWSqPP/44/+qQ1sA+XHo7WA65SvIz2teU7pekR9q9eJSQLd/v/cN9
tctJ0v8kJPyx6f/WkKQqkhT/TMLPmUedqh7wnHpko5+f+mIHqqLpn7Qa8cqK
psc51aMZZXXRtAYAxmVULPDF1jKg6tat26xZswULFrDLqziGjW6vX79+6NCh
du2zu/tYHv7Tn/5ks2Jr4s8MvW3SpAn9cTLlNMQ2sDxZj1Qe4iD5h2OqSbUk
qenTnYfNTZXK13oveby3wGn0k8py/rl/jBskldPXEv1i2Yoc+jcaC2peD/Ea
jWp6+5pek00z2qT1oEY/EPDXMAAUYRdTmhkaN27MVgVl67nXqFHjwQcfHDNm
zJkzZ9gx7Nq6Zs2akSNHBuwkhdsixrO4uLicHPoRbocVDfzHQNKvlKY1t9st
/rTYeF1UiByal3j1gFVAcg9F1Zf+58FqUvl7pXvrz1y1SQ9hTpUmL2PkWq5G
rqxZ8dl9+vJ+0oTPl18ymtdYB6iiBzbVaFTT73oVj/4g/Q2y1R9wAFBCfJFx
tkHTGl/GnS/sXqFChTp16rzwwgtff/01e1ZqampSUpL4ClBiPJslJCSIgwYt
Tfyp4P3srCcUaQ3sRjOmA+gj145E1zcWdq9Ya9Snq3KMh4is6c1lRNH0aFZA
1PyRf2leUT+m9peZF/LpHzUqYW1rmr7aAT2aqMbvRGH+lZQxySOGjUkGgIg3
atSoMWPGjBs37q233ho5cmTNmjWlIiyz8cBGb6tVq9awYUOn05menk4PJoGG
tEEJsFrE7du3/+GHH+j3gn5H3ra4CRMm0C+E/oCNHz8+xUC/qHLlyvl39SKt
geXJpFBTPDSJOU80v0eqI5Wp+8cnMtmcUI9M9IdYCFOI5vYSuXDnghp6B0bN
/+vaO5umMq/R/6kQr6Z49QmhMpsX6nA40tasXpW6egUARLzU1FR6u2TJEhrA
Vq5cWa9ePZbQWEijl1feyEbDW40aNWiioAmNPovGPD6fFAPYSkzsX27duvXe
vXsXLFhAvxFLLW6xgf5o0e1FixYtX76cbhw+fNg/myGtgeUZc0IdNJ3l/dy2
xj21paq/b9nnGI1h+mwBvUDHVTYJ1KNX7tDHAefvbPZgValcValig81HL+Wz
Kh3GonMKcXuI21ftg76CoqArFACIUN2LtZKxnlAW1Xh/6H333Uf3v/feexcu
XGDH02jHe0LhDvH2yZYtW166dInYIsCYanewrzHg12WDLxYinaynNZee1zJb
1rinllSzcfMBx4yCaUTNJaTAV7JD1v+lb7j2/6P383pnaIW6SZ+uzqK/IJqv
thpNa4rRDuclrIYb0TApFAAE7KLZqFEjntaoBg0atGjRIi0tzTTuHdVxg4gF
G3qGY2JiHA4HTimAxXhVt6867tEWNcvVkKr/R7MBRdVxr9C05uEF1lRVD2WO
A1u/+EcV+glbqW7v6Wln2KNe+hnrNLpBWXu7ntaMP+SQ1gDgWtsaCwmPPPII
/QipWrXqb37zm169eu3fvz9gqwjSWrCIcyebNm2KUfcA1qPIRjFbJ8k/kqCv
E3pv4yi9J1RfeUrLpRnMzfOYKhuNbMe/m5eiT0YoX/33nYcd19vljHWpSIFX
n4zgK5yrsFq5XsvPEAeAYOFjzx544IFGjRpNnTo1OzvbdIw4oQBpLVh4PLt8
+XJCQgKfolva7wsAbpnsdulhy0muHnmimlRbKtso9tWjLIN5CwhxO3xpzUlU
t95SJh/bMHtELb1trXqNuFd/8dXxuErIVa9Rb01vU2PzTPWRb/g0AADzpM4z
Z86wsUas65M9ysMDVnUPOn4C9+3bl5iYaNoJABZgpLVCPa0dbV1dr47bKO7F
n1laU2g6k81pzZu54dPRNY22tfqte51kRxKa6wp8DWtGs5qxeDPBJAMA4FgM
c7vdYr8nX8mdjxXnG0hrQbd27dqhQ4fifAJYj0IzmVdfyyDvaIvqUg2p7CMx
L/p6Qr0K0WRfT6gq021jlsG+LZ+9XU2fylX75bfnnWVpTdOXovIlNJXXcCME
RZIAwCAmNJ4WWAUwcQ96QkOEndjp06fPmDED9esArMdLZH3VgQKSdyy2ZqWa
0j1/iH75BCFXif6XME1orByHUQDXKHzr2T2sc3R1qYxUpt6stXsv+yYS6G1q
RcvD+BrVvL79ABDpeF+nWPjLlBlM6x0QpLVgo+e2f//+mzdvJig4DGA5ek00
1UvT2dXjsbWq1JKq/Vfsa6cCpjVjVBq5srnZfVI1qWKZex45coXkEuIyPkc1
IZ4JFTyQ1gBAJy73eezYMZfLmJ9khDcxwomQ1oKuRYsWFy9eJDilAJbjPZtL
lPP0345f2lSTGkhly1RtddJjxC73r8R7UU9c+vrsDuL1XCZkT+r0SmXvlcr+
T5eh83M1Y36B9zzx5mvEV7WDxbai5FZQ2l8eANyc2DVJQrOuojgP8f7772/Y
sOGECRNOnTolHsCXfWd7aFpj1XERLYLi9OnTLVu2JDesIgsAYcqluvQodoUU
nnyiepUHpAqS1KjXPxbl+nKXXEiMtQpcTqJccRdkNG9Yo4xUSarxp31XWAXd
fCJfIaqiFVXt0PSWuqI5oRrqrQFYg9jMFfRuMjH+0Rdn9dbKlSv3wAMPdOnS
ZefOnfy/yDfo8enp6aNGjTLVq4cSYCd/9erVfG0IUyFiAAh3LmKs2H6c5ByP
q/ZgA+neRo3+IFWp/eH6ny4Rclmjj7v15jWXTArPpfT435pS+TJSlU27j+f4
Kq259MkIsr6igfFZrJfJ9RLFl9bQEQpgTR6PJ7gvKPaENmrUiC3gTlWsWLFu
3bpNmzZdunSpw+HgU0SJsPIUotodYol30KBBq1at4jvRtgZgIZpCf2dpWjtD
8s81ue/RFg9HaW75/c8/ke6t8vY/v3DrEzw9Hrdzz+79j9a/735Jqln+NwcP
/GosVqW4NYceyLxEU431p3T0pRyasVqor6AHAFiEaTXPIF7NxTXZ6TZbJ1Rc
fKps2bJVq1b97W9/O2bMmJMnT7J4lpaWxtuCUH7/TtBTR7+nsbGxvBwxTiaA
tdBfWU11Ey3X48x94L5GC2euoOnLS3LXf7+0bHmpiiRVk6RyUlmpfEWpUq1/
zV6Sn288RW9D02vnsoXb3VrRQDW961Ov0uYlqq+gBwCEPd54xQf8U/Pnz58T
JHPnzv3ss8/+ZaB3GzZsSONZGYOY2dieGjVqxMfHezwePsuAIF3csc2bN/fo
0YMEqkIMABageYi+FLusshmcXmOHPiX0PHGePbN3//FdGYf3HTl66iwNZ1dV
vffTozecFRKN5jZVMcKZi03/1BTiVfWGNj2qqRrCGoB1mFYjovnq97///fvB
895773344YcffPDB1KlTq1atKrat8bTGukerVavWuHHjwsLCtWvX0rTG2oVK
9dzYQVJS0qJFi8TqdkhrAFaieryqS180SiVsPoGmzxko1Ps0vR7Wy8nLcyjG
gqBG4SQ3cTnowW6jjq7HaGDT05rqq45rlF9T8QkLYBW8eY1Fo08//ZQmqyC+
PntZr+HRRx+lCa1s2bK8G5TepVGtXr16PXv23L17N3tKWlrayJEjSVFHHprX
Siw3NzcqKsrhcPBydjiZABajD1wrmsap6K1lXtaPaVRXcxsD0YxClvRRmssu
6wtR0aO9MpFlltZc+vNkI62pvqimsWK5KkYGA1iCacQavTtnzpwPP/wwuK/P
o8Lvfvc73p5Wrly5WrVqNW7ceOrUqVlZWeLx6enpw4cPD9Z7iGTz588fOnQo
20bDGoA1yawD1JgRoGr6AlJGyDJ+oRVfkVv2qD7f08We5DU6PY0459YPUFjq
8/V90m29dxXLhAJYidje8q9//YvGp+C+Mi8P0qhRIxrSaFSrV69eu3bt0tLS
xPohWCe0xExJjDVI0vMZHx9//PhxMaEhrQFYjcM3XE0PV06FGNM8VZa4nEZz
m28xKa1okQKdqsczL1E919alKlrMXd8hG09HWgOwktClNSLU3aW3tWvXrlOn
Tr9+/Q4dOsQKf/GCIWKKQForAT7Gj1dU++KLLwYNGsTu0tNLDxCzMQBYhIMN
VjPSmtvLxq6pQgEOIa1pGi97qxgTPxUPP9D4l9foAPV1iSKtAVhKSNOauAAo
DWmslIR/DGNxgm0jrZWMOC+DhrS2bdvSE84XaeWQ1gAsxVdzo1i20oqFNN8e
wiZ78rSmt61d6yfVx6rJXra2u0qK0hrKZQNYRujSmhgMeJDg1UJYkMM6oXfO
VEaYnti0tDRWuMO0WgROKYDV0ADGqqb5Rq9prENT401rKmFTPo2WN+M33CjW
YSQ3XxVc34RS35xSoakNK08BWEZIx63xtrUbJDdSNNSKbSOtlYDYhpadnR0f
H3/s2DFTPT2xnRMArMLLx54pxgA2X6lbt17q1khlxFcBV1bYklIsrRmzRr2+
wKb4Gui0aw1rRghEWgOwjFDPMhBfP+BANVPfHNJaCfCmM5rQxo0b9+6777L9
rPUSpXEBLOpa+xgLZcZfYIoezJyanrX4/FB2hKywDVYIV/UlPcU3s0D1zS3V
W+dYgVx8IABYRqjTGhGGvrM9/kPi0bZ2J8RUvHfv3ri4OHpi+VRc9hDWcwew
Iq+vr9Otj0Gjv8R6XQ6nmzgcRC4sKt+haUUNcIrD9/ewJutHGjNJPUZ1XI3N
HlV4JRCZLWcAAFYR0lkGpmB2KwEMaa1k6Kl2u91PP/305s2b2R52Anl7pjg/
FwAAACwkpGmtBJDWrsc0AtB/NNqgQYPGjx9fWm8PAAAAQgRpzSr4CVEUhU+t
5Q8tWbIkMTHR6XSyPVhlFQAAwDaQ1iyE5jRxNCAvgZKRkREdHf3LL7+U6rsD
AACAkEBas4qAzWX0LP3666/x8fE7duwggaqjAAAAgNUhrVkIC2NidbUzZ860
adNm9erVbBIBnwSKtAYAAGAbSGtWYZpfIMtyTk6OJEnz588nwnQDPgkUAAAA
7AFpzVp4X+elS5eefPLJJUuW8MFs/CHTWlQAAABgaUhrFsIL4bKxaosXLxZL
4wZcdxUAAACsDmmttIj1N9iGaUEufpe3lbEjd+3aFRcX9+WXX5IIOEsAAACA
tFa6tCL8LhH6NE1zPOndGTNmtGrVas+ePZhKAAAAECGQ1kqRmNNYA5ppaid7
iK3bdeXKlT59+vTo0ePSpUsENToAAAAiBtJamLhxx2hqamqrVq1mzJghPoQF
CwAAACIB0lqp48PSxBod9MtnYSwjI+MFw6FDh4hxWsSQFjlnCQAAIGIhrZUW
0xfIY5hYo2PSpEmxsbErVqwQh7exDVNvKQAAANgV0lrpMs33ZHd/+eWXlJSU
pk2b0m9Hbm6ueLw41A1D1wAAACIB0lppEceesS+W7tm5c+fw4cPj4uKmTZvm
dDpJ8XjGDxNfAQAAwod48RI7RGx/UYOQCsO0RuMK27blz3bAX+SDBw9OmTKl
VatWXbt2XbFiRX5+Pn+Urw1aSu8XAABuScBBxXwnL7skVjUHuEXhk9ZYJyBN
a0lJScT4CbdiRNGKI0LQEk81/WIzMjKWLl06atSo5s2bd+7c+aOPPjp58iQR
iniIT8fvNQCA5fA+FFmW/f9UB7h14ZPWiBFU1q9f37dv3wMHDuzfv3/v3r37
9u3bb3F79uzZvn07/boWLFhAT++QIUOee+65P//5zzSh0aiWlpZ28eJFU6IT
q+PilxoAwCr4pze9ivk3OHgNd/1NgR2EVVqjP8ZZWVk0zCQmJnbv3p1Vruhu
ES9cx0svvdS7d+/hw4dPnDhx1qxZ69aty8zMdDqd7BeZVb4VG9B4oyKr40Gw
ACgAgKWsXr1akiT64f/999+zPfSj3lSsCZ/qcFvCKq2R6/T72wnPYKT4F8ti
m+lI0wYAAIQt/pG+bNmy8ePHb9iw4emnn05MTNy8eTPbzz783W53qb1FsKzw
SWti95+dggr7csRV2knxRdvFDYr//cWfftfeKgAA3An2d/eCBQuSk5PZni1b
ttDA9swzz6Snp9OPd9N4ZoBbFD5pjeEjtezXsHa9Dk0WzwIuUhCwFQ4AAMIQ
nyO2YsWKlJQUvp9+kv/www+vvvpq27Ztly9fjtFrUAJhldb8p0/a4w8Q8XfT
v34aMX7Hr/cHlynFAQBAmFu6dOnbb78t7mFdJwcOHOjdu3ebNm2++OILl8vF
HjL9IS9e/sSq6fZrwYDbElZpDQAAwOoCpjVSlLt+/vnnAQMGxMXFzZ49Oy8v
jx9wvcpsAARpDQAAIKhMaU1cA5q3pJ04cWLEiBGtWrWil93Lly/zg005jUc4
05BmiDRIawAAAEHk37bGx7rwAk1s/6+//jp+/Pj4+PiJEyfSzCaOWOYdoPYY
EQR3CGkNAAAgiExpTUxcphlkHo+H3p4/f37KlCnNmjWjz7pw4QIR4hxffBC9
ohEOaQ0AACCI/NMa2+C9mbxXlBRNQ6Pbubm5H374YXR09IgRI06ePHm9NW4g
MiGtAQAABFHAtGZa/ZlcZxKBw+GYMWNGfHz84MGDjxw5wgq4oWENkNYAAACC
yH/cmjjfM+CGaXya0+mcP39+y5Yt+/Xrt3//fjSsAdIaAABAEPmntdvCr8tu
t3v58uXt27d/6aWXdu7cSYov+kP8JpDyhji0xdkP0hoAAEAQ3WFaI8XrddDt
devWdejQoWvXrlu3biVFkw5uvKw2ZpLaDNIaAABAEN1hWhNXYBQj2ZYtW557
7rmOHTtu3LiR7eHNaDyb8dWvSvxfh/CEtAYAABBEd962Roo3jolNbVu3bu3R
o8eTTz65fPlysSc04IRTsA2kNQAAgCAKSlpjTIPQZFlma1Tt2rWrV69eTzzx
xOLFi2lOY3XbWA8pxq3ZEtIaAABAEN1hWjO1kpGiRRD8M9iBAwf69esXGxs7
b948+ix+DNKa/SCtAQAABNGdt62JBXXFWh/+S1NRx48fHzZsWPPmzT/++OPC
wkI+7A3sBGkNAAAgiO68god/3DLtMS1woKrquXPnxo4d26xZsw8++CAnJwdz
Qm0GaQ0AACCIgjhu7bYoinLx4sUJEyZERUXRW9OSo0S44iPLWQ7SGgAAQBDd
5bTGohcrjcvk5OR88MEHNLMlJyefPn1avNBfbxv5LcwhrQEAAARRqbStsf5T
caJBYWHh9OnTY2Njhw0b9vPPP/NyH2zOgulZd/ndwu1CWgMAAAiiUmlbY1hB
XXE8m8fjmTNnTqtWrfr373/w4EH/zlBMSbAEpDUAAIAgKpW2NdP6BawsGym6
yjudzsWLF9PM1qdPnx9//JE9xG7F0rsQtpDWAAAAgujupzU+aI1V/OCzSsVU
RozMlpqa+swzz3Tv3n379u3iAWhhC3NIawAAAEFUKm1r/mPPTEtQ8XFrVHp6
+rPPPvuXv/xl8+bNWFTUEpDWAAAAgujupzX/VeBN8z1ZVDONcPvmm2+6d+/+
zDPPpKamYqJBmENaAwAIV6rxT3CPhJArrXprt4uNWNuxY8drr73Wtm1btuQo
f9Q/+PH+VjTH3X1IawAA4QppzZLCPK2Jg9lYBlBVdd++fQMHDoyLi5s/f77L
5eJ5zDTyTZxwCncT0hoAQLhCWrOkME9rpmVGxT7QjIyMwYMHx8TE0DyQl5cn
PgshrXQhrQEAhKvbCmBIa+Ei/NOaaQKCx+MhRbGN3p46dSopKSk2Nnb69Om5
ubn0MFmWxQXlkdzuPqQ1AACAIArztMaZZiWwVMYfvXDhwpgxY6Kjo6dMmXLp
0iVSfOEDVPy4y5DWAAAAgijM0xofq8bvim1lpskF2dnZ77zzTlxcHE1ubJl4
KBVIawAAAEEU5mmNCFFNHL0WcAYB25OTk/PRRx/FxsaOGDHixIkTYhMc3B1I
awAAAEEU/mmNCOsXiLcM3zaFt/z8/FmzZsXFxQ0ZMuTw4cN38c0C0hoAgPWY
luRGadOwYom0dlv4UlaUw+FYuHBhy5Yt+/btu3fvXvEY4pf9TD+Z4uvcyZth
t+Jvge3H0SGtAQBYi2mgEcGQ7zBjs7Qm5iLeB0o3li1b9uSTT77wwgs//PAD
2ylmM7ot/lia1o6/k3xlWqYhQn74kdYAACzK1MIGYcJmaY0E+gFje2hm+/e/
/92xY8cuXbp89913/p2q7Bj/tt87bA3jT+cvYkqD9oO0BgBgLfxzW+xvQmAL
HzZLawH/KBDLfdDtr776qlOnTs8///z69et5W5ypqS3gT2kJfm4jod/TH9Ia
AICl0EtesRpZEXjlCnM2S2uMmLXEHzneKU8P2LZt2wsvvNChQ4eVK1d6PB7W
ZX/jhBaUn95IqNaLtAYAYB2q8I9OqFZaqu8LBLZMawwLYOxHTlw8VGxG++mn
n3r16tWmTZtFixax9jdx5Bsp3jhWgrTGciOrNxI5f60grQEAWIcvqimKx1TF
lF64SvWNwTU2S2sBkxWLTKakxAaPsf0HDhx44403YmNjP/30U7fbTYq3gN1h
xDJlPxIBzWtIawAA1qGyntCDB/cvXLiQfmJPmTIlNTX1/PmLpf3G4BqbpTVS
fISkf1Lyx4/JzMwcOXJkTEzM9OnTCwsL/ecClCC2Xbhw4csvv5wzZ85nn322
evXq48ePR0LzGtIaAECY8upNBkTzuAlxET2mESXr5J606Nj/kCqUl8pWf+HF
V7o890RFSbpHqvjmy2+dyyUuerj+we7Qn6Fv6U/zEtVDCP1HYTvovzWFeFGi
7ZaIA+b9G3DEtQBIUZcfTWspKSl3922GEbEGILs9ffr0mDFjaGZ77733srOz
xXPl/0RGXI2Ub2/fvj0xMTEqKiopKWnSpEn0T5WxY8cmJCQ8/fTTy5cv572u
xPi+2Gw9U6Q1AIDwpMp6W5qqebxEUWnaUp0rPpteuawklZMGjH4r66rLGLwm
0/1bN21+sFp9qcKfzmYRt0ojGY0Qiqro0UIlXk0PbMYuX1pT9bSGgro3w/MG
r1bBH/J4POKRpj4+1rZmm5xQAmJJQLZBT+O5c+dovmrevPnEiRMvXrwYsLDz
9QIbfZEhQ4Y8++yz3377rTgdlR1z8ODBnj17dunShb6s+CKmb5OlIa0BAIQp
zdc8pnlziXZ21bwPK0llJKn62A9mu1juov+XNSN9OTzZxxs9GNuy3Uv5hLh9
z1f1nKZ52JQEL09r+item6cAN0UjAW9DE/vy2AAt/949ntYiM7DxAMaDrjhv
9NKlS9OmTaOZLTk5+dSpU2y/fxOlGIBp6GrZsuWjjz7qn774nFO6MXPmzCZN
mrhcLvE/bZtlPpDWAADClSorrDUs6/KxjdUkmtVqNmnR87Kid2t6vQ4ve1g/
IvvQrkXVK9SXKtTd82tOIb38Ge0ZXiLL9FiazYxLVtElk+VAm1zFQoplBn6h
PH78OCk+pZE3HBFh4L3YExqZgY2ft4BNZ/S2sLDw448/jo6OTkpKyszMFA8z
Ff2gd4cPH/7mm2/y/aZibmI5OBrY2rVrRwOb/SYdIK0BAIQro5FMU4hy9o2/
xlaSaFz7beqW7EIjd6nEpbAApsrnj39fs7xUWSovla+58ruD+cZ4N73FgrgV
vavUaFYz+kN1Xj2teZDWbsaUNOjdKlWqPPbYYwsXLqR5gN4VO/vEiZNLlixB
WmMb/CyZOkbZafF4PHPmzImPjx8wYMDBgweJ0N3MD9izZ89TTz0lvqB4AN/m
EXrkyJGzZs3iQdE2axwgrQEAhCdFzdMDmzs3O+PrmjSqSRWk6o+fcxBZJW53
Ho1h+cbcAaI5Lh7fWkWiaU2SylZetGlPga/8muLUCrw0qHn10W009Xl5E5tG
Xxdp7eZ40mAX/QYNGpQpU6Zq1aqNGzdOTk4+efIkCVRan6Y1m80JLQFWD03c
43+i2B6a0OgZa9269WuvvbZ3715TSO7evfuWLVvYS7GeUPq94JHPf1mrrKys
6OjowsJCm+VkpDUAgPCkRypvAZEvbvjXeBrGpIp1ug+ZmaOxJOZUiFpIWBNZ
LnFfGDukd0VJatexy9kCfQKpSjOaRq9tbr0n1DdkTdF7PzVfhyjS2k2ZmnHo
7X333Ue/D+XKldO/GxUr1q9fv3Pnzt9++y07jIcT+1XwuC2mAWP0tLAWM/9m
MbFPk96uWbOmffv2iYmJ3333HdtJo1dUVJRprCDfFl9QfKlXXnmFLVpqm0Fr
BGkNACBcGb2XBcRzYsUHA6vqKeG+/lNWZxntZETVe+KMohz05grRCvSyHEQp
VPXeU9mYTuAlqpt49ITGekLZzIKitIae0NvCrpX3339/2bJlJQG9W6dOnSZN
msydO5fXE1u2bFkkV/DgxOJsvGdTvMsfZeeNDfz78ssvO3Xq1LVrV7qxefPm
N954gx0vZjbT2DZTsY7PP/988uTJxF7d0EhrAADhSb+wyfnEk5n0lz/qbWsV
ar3+/vKzhBQSNpiNTRSg6SyPaHr4chll2TRjkqjetqbfKCrx+hZA0Mi5c+c2
rN+4+d+b1qxZ8+/NG/4NN7Rx48a1a9du2rRp3bp1bE/NmjVZWmPNa2XKlGF3
6W3lypVr1apFYwa9qi5evHj8+PHERoOmgsJ/4QP/sWekKHp99dVXiYmJjz32
2JAhQ4jfzIKA+KOrV68eOXLkt99+m5aWRn/UV1sc/SrS09NTU1PZLd3z7rvv
Iq0BAIQJPa15HcSTMeIvf6hWRpLKVx88a8NpQq4SoxabLBstZjINbJoxraDQ
1xznG5mmF8XVZP0K6Js3Sn76ae+I4SMHD3ozKWnYwEH9hsANvfkmPVFJw4YN
Gzx4ML1LN6pWrcqjGm9kq1SpEtu499576S0xxq2988477Jtop+adOyeWSguI
DxQ8ceLE8OHD6fns1q2bKdTd4OkssG3YsGHQoEGzZs0aPXo0jW2jLC45OZme
irfeeovd0q/ojTfeyMnJCdL3BAAA7ogevbx5xLHnrZf+dA+9bpWv3XvSqgus
bY3IxGukNX15UIXVXXMYG8ZOYyQPmzlq0CuvqazZzRiCpbJauXATYoEOekrr
1KlTpkwZ3g3K2tYqVqzYsGHDxMTEbdu2sb68xYsXv/322zarpX/nxEkHfMIm
n4zA2iHpdkZGBs1aLVq0mDt3blpa2hCjbc1USiUg3v42b968Dz/8kAgDDi1N
/NpNK38BAECpM1oZcoiSuf6zERX1tFb/zff/neOLWQV6YNOI5i2q1qHvdLM0
5mZl2OhjmhE2NCJe5PREoXeU2qfMe0jxjEEvlw0aNGAhrXz58nSjSpUqDz30
0Lhx406fPi0O0Fq6dOmECRP43dJ532GJZQ/tOuP/d+/e3atXr4SEhOXLl7Ma
KRcuXIiNjTWNWLvei/PX7Nev35o1a8jNek4thC2k5V9iDgAASp2mOYl2lbhP
rp33TmWaDsre17TdoHzfdIE8mejrh+pZQK/MwQocXFbdDgchDjYTQdNXmNLr
8Pta1PTl4D2K7FvOQHHf+L8ODB8AT2/r1avHWtXuv//+qKgomspY5XyGt4Es
WbKERjiCq2pxPHSJAZjltx9++OGVV1556qmnVq1axRrceMrt3Lnz/v37iTC3
9AbosxwOR3R0dE5Ojp1ysn9GtdOiWgAA1qYVKLKTEGfB2b3VaFgrU0m65z+P
XmRzOp0eovgGqumLfnr0K1PBiQ2pi37f4onzhBTqbW56A1rRHFAa6Fwakb1G
cpPdxsLucDNiTyhVoUKF+vXr9+jRY9euXZrfmkqkqLjrsmXLxowZQ5DWAhEz
Gz2HX3/9ddeuXTt16pSens4eYhGOFDVLrl27tnv37nznDbCzPXHiRDbFQys+
7dS6+F8B7IzZb5kGAABrU2moMuYauC9+NPQvNfQB7g1Gf5ye5+veJA61qAPU
69T/fXnvHx+q+8LYd08ak0P1pQwUegj7rNcPU4nLrXmLlgpFWrsJfqFnGYxF
i0uXLpHiC1mKixqwhLBgwYIJEybYICcEnThcbf369R07dqRRbcuWLTyJ+ddk
oyefxuPevXuzPTduXqPRLiEhITs7mw/6CuEXc3eZqp3Y6UsDALA2j9tldGsS
NZec29T4XkkqW12q0STjoubVezeJRzWynF7pVj8qb/HEKpK08siZ4+xZ9FPd
W6ixBaf0G6dXb5FTNTbaDWntZgI2zohjrsQ8IC4jzlaeut7orEjGejNTU1Of
eOIJmsG2bdsmPsS3TRM0XC5X5cqVb1qz4ptvvvnd73535MgRIrTg2SDV8B8k
+0VQAAAbuEIKXEbRW+J0Ek/ukc1La0nSPdI9le9tevQ0yfaSPL0NjV6VzhJy
Mjdzk1TnD60T/55vNKy5i8YjF/3Dy3gYO2lU024yBAhKLKLWMuC5KOAC7uJd
mtPmz5/funXrXr16HThwwP8YfzwD5+fnJyYmDhgw4NdffzW9Pt2gcY7G41at
Wl28eFHcf6dfGwAAwM249HWjNN8qn7KbyNmZu7+uVaWKJNUuU/E/FqX9mO0h
HuLOvvLLnE/eo3s79UrKMSaLuorGqulTCfS5nwHTGtrWQiWi0hoTsFwJb2zM
y8ubO3duXFxc//79WduXaan3gMTKHlRhYeHixYtbtGjRrVu3f/7zn/Qkp6Wl
zZ49u2/fvo8//vj7779/5cqVW3xlAACA4FF8DWNeo56aXqCjoKDgwvsfvGtU
+5KksvpthQoVypepunLJV/lGB2hhUSU1r+IpSmXX0lqxwAahESFpjQ8hM3UN
i5mN5rRPPvmkZcuWgwcPPn78OBES3a3HKrH5jm7v2rVr5syZKSkpo0ePnj59
+saNG+l/hR3gdrv5OzE9FwAAICRYCVuZsEK3RhubU09kWq4qXz517KcjB3cc
2vfToYPHXG69jEehkeeMNUKLoprX3LZ2La1hndCQiZC0xvinLxbecnJyJk6c
GBMTM27cONZ9yYMcLx12085Kp9PJNkyBkNUfM70BE9PKVgAAACHh8WUvzZfa
jPimuYw5BTSzOQmNZIpecs3t9R3L2hEU2a0PS2P/qDKbQKr5FqQSAxuEROSk
NbFuCSkKTllZWSkpKax38vz588RvIqf/1IwbYGUrTK/gX+uDx7+b1mQDAAAI
JmMpdtnjMFYCVbzqtaClet1G8Vv9GLrfrXq8RL72NDaJQHEZgc24rhFf6xzS
2l0QIWnNf6X1U6dOjRo1qnnz5lOnTuW9k7xqh3iwqU5dQKZOUlNxXTGV+Re1
MMU5AACAENE7QPULk1vTHJrqYdlMMy5uiiqz3EUvSsYeem1yql4jm6k8p8ma
7OLdoDytFUFaC5UISWukKCDRn9KjR48OGTIkPj5+xowZDn09jWsHmEKXqbTa
DdIUf3GxCU48nnWJ8m3/F8SgNQAACDU9YulRjOYuY+wZS2t6x6hidH7KHtVX
6pY1crBWNdXj1DOb32xQv4Y1pLVQiZy0RgPSoUOH+vfvT3Pa559/LuY0/17O
22pYY0zZLGClO1NOM/WNYnIoAADcFUXJ6lrYkglxakb3KJ87oF07UhXDmFb8
n2sviDmhIWPdtGZKQaR4C5hpouVPP/3EFvekX29hYaHp6QAAABHBF7BUth6B
t1gqY1MKijWd+T81UFQjSGuhZt20xhuvxFYv07oA9Hbbtm3dunVr3759enq6
x+MRV4m6u+8XAACgtBlRrSitqdfSmjmBqfochOLtbHygmjdgVENaCyWLpjX/
oV9sQU9S1K1JffXVV88//3znzp2//PJL/8U9CXoeAQAgQpk7N/0ay3xNbbcR
1TBuLZQsmtZ4OQ4Ww3iTGitctmrVqg4dOnTv3p0GNjHOBcxsAAAAEeJa0ArQ
mMb+8T/0Zh2gyGmhZ9G0Rvwqp9EARvcsX768Xbt2PXv23L17tynIkaIWuVup
nAYAAGA/5rSm8r5R2ZhiQG/VYo/6TS4QBJh9ACFi3bTG1yOg0cvj8SxYsCA+
Pr5v37579+71n+DJlyQggeZ7AgAARISiljTfhIJrCYyvbCC2oKnCqqBi61ng
iaK4poaORdMaL2DrdDpnz54dFRU1aNCgzMxM8RhW4kycIsrW7uSP3vV3DQAA
UKp8jWm+bOabX0B4NhMb3BR9jVC2csG1wCZMKCi+vLsXaS2ULJrWqKtXr06d
OpXmtJEjR549e1Zc2ZP4hTHTQDWU7wAAAACrCKu0xoOWaaEoU7K6dOnSpEmT
mjdvPm7cuAsXLrAOTQQwAAAAsKXwSWumrOV/lzp9+vTYsWNjYmImT56clZXF
H/V4PHfpXQIAAADcXWGS1vgET0VRWJMaC2B8mFlmZuaoUaNiY2OnTZuWn5/P
1+UUZwqgbQ0AAADsJ0zSGsMmeJp2Hjp0aMCAAQkJCXPmzHG5XDyeictOcXf1
7QIAAACEXrilNXbLmtf27NnTt2/f1q1bL1y4sLCwUBzPJk4oMD0dAAAAwE7C
JK3xemjsdteuXa+++mrbtm1XrFjBFmEnRWGM9Y3yYMZrdIhVcAEAAABsI0zS
GkOj2jfffNOtW7cOHTqsXr2a7+c9pOItudnEBAAAAAAbCJO0RoNWenr6c889
16VLly1btogPsdY2XqaDCA1xRAhy6AYFAAAAW7oLaS1gZVpx4NnKlSufeuqp
v/71rzt27CDCOgUhfVcAAAAAlhCitCbO0GSzBkzzAtgiUPS/HhMT07t37z17
9hCjjgc7jBX0QGADAAAACGnbmqkiB992uVxz586Ni4sbOHDgoUOH/J+FnAYA
AADAhC6t8fUFWOUNFtXy8vL++c9/Nm/efNSoUceOHeMH+08c4E8BAAAAiGQh
bVsTx/9fuXJl6tSpMTExo0ePPnv2LJ8UIPaTinsIekIBAAAAQj/LgCYumtNS
UlJoTqO3Fy9eJMI0TzGPmSZ1on4aAAAAAAlxT+jp06eTk5OjoqKmTJmSm5tL
ind08g0e2/hqBQx6QgEAAABClNZ+/vnnESNGxMbGfvLJJ/n5+URoOmMb/sPS
TP2e6AYFAAAAINdJa6YF000P3WAiAL17+PDhgQMHxsfHf/755yynAQAAAECJ
+ac1/4TGmLKZ2PbFHtq3b99rr73WsmXLRYsWud1u9hBfxxNtZQAAAAAlcFtp
jW2w0hxi8dtt27b99a9/feKJJ9LT06+3AhTSGgAAAEAJ3HTcGg9v9NY/iW3Y
sKFbt26dOnVat24dmyMgHmN6CmYNAAAAANyuG6Q10xA1vkHJsrx27dr27dt3
7979m2++4ZGMT+00LWGA5QkAAAAASsaU1m7Q/MU6QGnoWrZsWdu2bXv27Pnj
jz+KwYwt7kmEWZ/XW3kKAAAAAG7RrfSEsvRFw9j8+fPj4uL+/ve/79u3T1EU
vuKAOI+AHRwwpKHgLQAAAMDtul4FDzFiuVyu2bNnx8bGDh48OCMj4wavZpo0
yge8BftdAwAAANgcT1AsrV2vUG1eXt60adOio6Pfeusttgi7GMAwFA0AAAAg
pDwez+LFi1NSUvgePvYsKytr8uTJzZs3f+edd86cOcP6N/lsAvRpAgAAAIQa
ax9bsWIFS2tsEgF1+vTpCRMmNGvW7N1332WLsDN8XiefO4DMBgAAABBqCxYs
4G1rp06dGj58eGxs7NSpU/Pz8wOOOrte7VwAAAAACCI+8GzZsmXjx4/PyMgY
MmRITEzMzJkzPR4PXy6K1+JgUwbYBnsFNKwBAAAAhBQborZw4cJy5crFx8fP
nTuXdYbywWm8b1ScTUAzG89yAAAAABA6PHEtXryYt6GR4gXTxJXcaUjjQQ5p
DQAAAOAu4BMHTLU4/MOYuPLUDQ4DAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAgDD0//qpLlU=
    "], {{0, 621}, {826, 
    0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->{298.5199999999992, Automatic},
  ImageSizeRaw->{826, 621},
  PlotRange->{{0, 826}, {0, 621}}]], "Text"],

Cell[TextData[{
 "Our goal in this subsection is to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to produce a logical expression for the output of this diagram. "
}], "Text"],

Cell[TextData[{
 "To do this, we use the fact that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has functional forms for each of the logical expressions. That is, ",
 StyleBox[ButtonBox["Not",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Not"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 ", and ",
 StyleBox[ButtonBox["Or",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Or"], "FunctionLink"],
 " can all be used as functions applied to expressions. For example, the \
following forms the disjunction of ",
 StyleBox["x", "Input"],
 ", ",
 StyleBox["y", "Input"],
 ", and ",
 StyleBox["z", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Or", "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"x", "||", "y", "||", "z"}]], "Output"]
}, Open  ]],

Cell["\<\
We give each gate in the diagram a label. The specific names are not \
important. We chose to label the gates using the capital letter G with \
subscripts numbered from the right to the left.\
\>", "Text"],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzs3QtclHXe//8Z91D3Ye97t91Ou4/2f+/u/bh/j8dv17HfHnIVFAbM0hKw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    "], {{0, 623}, {814, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->{296.1199999999993, Automatic},
  ImageSizeRaw->{814, 623},
  PlotRange->{{0, 814}, {0, 623}}]], "Text"],

Cell[TextData[{
 "Interpret the labels as names for both the gates themselves and for their \
outputs. Note ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 " is the name for both the output of the final gate and also names the \
output of the circuit. The input of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 " is the outputs from gates ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "2"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "3"], TraditionalForm]]],
 ". That is to say, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "1"], "=", 
    RowBox[{
     SubscriptBox["G", "2"], "or", " ", 
     SubscriptBox["G", "3"]}]}], TraditionalForm]]],
 ".We can write that in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " as shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"G1", "=", 
  RowBox[{"Or", "[", 
   RowBox[{"G2", ",", "G3"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"G2", "||", "G3"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "For each gate, do the same. Note that the order in which the gates are \
specified is irrelevant. The gate ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "2"], TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "and", " ", "b"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"G2", "=", 
  RowBox[{"And", "[", 
   RowBox[{"a", ",", "b"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"a", "&&", "b"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The output of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "3"], TraditionalForm]]],
 " is the conjunction of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "4"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "5"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"G3", "=", 
  RowBox[{"And", "[", 
   RowBox[{"G4", ",", "b", ",", "G5"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"G4", "&&", "b", "&&", "G5"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "And ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "4"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "5"], TraditionalForm]]],
 " are inversions on ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", respectively."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"G4", "=", 
  RowBox[{"Not", "[", "a", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"!", "a"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"G5", "=", 
  RowBox[{"Not", "[", "c", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"!", "c"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Once all of the gates have been specified, inspect the value for the final \
gate, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["G1"], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"a", "&&", "b"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "a"}], "&&", "b", "&&", 
    RowBox[{"!", "c"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This tells us that the circuit's result is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "\[And]", "b"}], ")"}], "\[Or]", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[Not]", "a"}], "\[And]", "b", "\[And]", 
      RowBox[{"\[Not]", "c"}]}], ")"}]}], TraditionalForm]]],
 ". In 0-1 form, this is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"ab", "+", 
    RowBox[{
     OverscriptBox["a", "_"], "b", 
     OverscriptBox["c", "_"]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "The reason this works is that when we define the output of a gate in terms \
of unassigned names, such as ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "2"], TraditionalForm]]],
 ", ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " accepts the definition. When ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "2"], TraditionalForm]]],
 " is later assigned its own value and then the expression for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 " is evaluated, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " resolves all assigned names into their definitions so that the expression \
for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 " is in terms of unassigned names (a, b, and c) only."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Logical Expression to Circuit Diagram", "Subsection",
 CellTags->"Section3b"],

Cell[TextData[{
 "We have just seen how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to transform a circuit diagram into a logical expression for the result of \
the circuit. Now we consider the reverse. Given a logical expression, such as \
that for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ", we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to transform the expression into a circuit diagram."
}], "Text"],

Cell["\<\
We will model a circuit diagram as a binary tree. While circuit diagrams are \
generally not necessarily binary, this will serve for our purposes.\
\>", "Text"],

Cell["\<\
Recall that a binary tree has a number of vertices and directed edges. \
Vertices in the tree will correspond to gates in the circuit. One of the \
vertices is distinguished as the root, which will correspond to the output of \
the circuit. Each vertex has at most two children vertices. The edges between \
the vertex and its child correspond to the inputs to the gate. Each vertex \
other than the root has a parent, and the edge from the vertex to the parent \
corresponds to the output from the gate.\
\>", "Text"],

Cell["\<\
The assumption that a circuit can be modeled as a binary tree requires that \
the circuit satisfy the following properties. First, the circuit has only one \
output. Second, each gate has only one output. Third, each gate has at most \
two inputs.\
\>", "Text"],

Cell[TextData[{
 "Recall that in Chapter 11, we wrote the function ",
 StyleBox["expressionTree", "Input"],
 " for converting an algebraic expression in terms of the binary arithmetic \
operators into a tree representation. We will make use of the functions from \
Chapter 11 here, so we load them. If you place the file Chapter11.mx from the \
website in the same directory as this notebook is stored, the executing the \
following expression will load the functions from Chapter 11."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"<<", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"NotebookDirectory", "[", "]"}], "<>", "\"\<Chapter11.mx\>\""}], 
   ")"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Get", "::", "noopen"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Cannot open \
\[NoBreak]\\!\\(\\\"/Users/Dan/Dropbox/RosenMathematica/Chapter12/Chapter11.\
mx\\\"\\)\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/noopen\\\", ButtonNote -> \
\\\"Get::noopen\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData["$Failed"], "Output"]
}, Open  ]],

Cell["\<\
Recall that the functions written in each chapter of this manual are \
collected in packages. Please refer to the Introduction if you need \
instructions on how to use the packages on your system.\
\>", "Text"],

Cell[TextData[{
 "Observe what happens if we apply the ",
 StyleBox["expressionTree", "Input"],
 " function to the logical expression ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[And]", 
    RowBox[{"(", 
     RowBox[{"b", "\[Or]", "c"}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"expressionTree", "[", 
  RowBox[{
   RowBox[{"a", "&&", 
    RowBox[{"(", 
     RowBox[{"b", "||", "c"}], ")"}]}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "5"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"expressionTree", "[", 
  RowBox[{
   RowBox[{"a", "&&", 
    RowBox[{"(", 
     RowBox[{"b", "||", "c"}], ")"}]}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "5"}]}], "]"}]], "Output"]
}, Open  ]],

Cell["Compare the tree above to the circuit diagram below.", "Text"],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3QtcVHX+//FB2/qvaz93s8R0H7W/euyjHpXUZqnJiMIqmreB3fXSOqT+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    "], {{0, 301}, {652, 
    0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->{305.7199999999997, Automatic},
  ImageSizeRaw->{652, 301},
  PlotRange->{{0, 652}, {0, 301}}]], "Text"],

Cell["\<\
Observe that the diagram and the tree have the same structure. After \
reversing the arrows, rotating by 90\[Degree], and exchanging the symbols \
with the functions labeling the internal nodes, the two are identical.\
\>", "Text"],

Cell[TextData[{
 "As you can see, we nearly have a function for turning logical expressions \
into trees that correspond to circuit diagrams. The only problem is that the ",
 StyleBox["expressionTree", "Input"],
 " function does not allow for unary operators such as ",
 StyleBox[ButtonBox["Not",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Not"], "FunctionLink"],
 "."
}], "Text"],

Cell["\<\
In order to create a function like this that will work with expressions using \
the Boolean operators, we need to allow for the possibility that there is \
only one operand.\
\>", "Text"],

Cell[TextData[{
 "The inner ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement below tests the number of operands of the expression using ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 ". This will function similarly to the original function, except using the \
left hand side for the sole operand. We also need to create a new function, \
",
 StyleBox["addLeftBranch", "Input"],
 ", to take the place of ",
 StyleBox["joinTrees", "Input"],
 " for unary operators."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"logicTree", ",", "HoldFirst"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"logicTree", "[", 
   RowBox[{"expr_", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "e", ",", "lhs", ",", "rhs", ",", "operator", ",", "lhsTree", ",", 
      "rhsTree", ",", "result"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"expr", "[", 
         RowBox[{"[", "0", "]"}], "]"}], "===", "Hold"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"e", "=", "expr"}], ",", "\[IndentingNewLine]", 
       RowBox[{"e", "=", 
        RowBox[{"Hold", "[", "expr", "]"}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"e", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "0"}], "]"}], "]"}], "===", "Integer"}], "||", 
        RowBox[{
         RowBox[{"e", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "0"}], "]"}], "]"}], "===", "Symbol"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"result", "=", 
        RowBox[{"newExpressionTree", "[", 
         RowBox[{
          RowBox[{"ReleaseHold", "[", "e", "]"}], ",", "opts"}], "]"}]}], ",",
        "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"operator", "=", 
         RowBox[{"Extract", "[", 
          RowBox[{"e", ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "0"}], "}"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", 
            RowBox[{"e", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "]"}], "\[Equal]", "1"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"lhs", "=", 
            RowBox[{"Extract", "[", 
             RowBox[{"e", ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "1"}], "}"}], ",", "Hold"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"lhsTree", "=", 
            RowBox[{"logicTree", "[", "lhs", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"result", "=", 
            RowBox[{"addLeftBranch", "[", 
             RowBox[{"operator", ",", "lhsTree", ",", "opts"}], "]"}]}]}], 
          ",", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{
           "else", " ", "there", " ", "are", " ", "2", " ", "operands"}], " ",
            "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"lhs", "=", 
            RowBox[{"Extract", "[", 
             RowBox[{"e", ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "1"}], "}"}], ",", "Hold"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"rhs", "=", 
            RowBox[{"Extract", "[", 
             RowBox[{"e", ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "2"}], "}"}], ",", "Hold"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"lhsTree", "=", 
            RowBox[{"logicTree", "[", "lhs", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"rhsTree", "=", 
            RowBox[{"logicTree", "[", "rhs", "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"result", "=", 
            RowBox[{"joinTrees", "[", 
             RowBox[{
             "operator", ",", "lhsTree", ",", "rhsTree", ",", "opts"}], 
             "]"}]}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"drawBinaryTree", "[", "result", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "To implement ",
 StyleBox["addLeftBranch", "Input"],
 ", we essentially repeat the definition of ",
 StyleBox["joinTrees", "Input"],
 " with the elements related to the right child removed."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"addLeftBranch", "[", 
   RowBox[{"newR_", ",", 
    RowBox[{"A_", "?", "binaryTreeQ"}], ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "newRI", ",", "newT", ",", "newVerts", ",", "Aroot", ",", "newEdges", 
      ",", "v", ",", "e", ",", "p", ",", "w"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"newRI", "=", 
      RowBox[{"ToString", "[", "newRI", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newVerts", "=", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", "newRI", "}"}], ",", 
        RowBox[{"VertexList", "[", "A", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Aroot", "=", 
      RowBox[{"findRoot", "[", "A", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newEdges", "=", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"EdgeList", "[", "A", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"DirectedEdge", "[", 
          RowBox[{"newRI", ",", "Aroot"}], "]"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"newT", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{"newVerts", ",", "newEdges", ",", "opts"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"newT", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}], 
        "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"A", ",", "v"}], "}"}], ",", "\"\<order\>\""}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "A", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "Aroot"}], "}"}], ",", "\"\<order\>\""}], "]"}],
       "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "newRI"}], "}"}], ",", "\"\<order\>\""}], "]"}],
       "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"newT", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}], 
        "=", 
        RowBox[{"PropertyValue", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"A", ",", "v"}], "}"}], ",", "VertexLabels"}], "]"}]}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", 
         RowBox[{"VertexList", "[", "A", "]"}]}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"newT", ",", "newRI"}], "}"}], ",", "VertexLabels"}], "]"}], 
      "=", "newR"}], ";", "\[IndentingNewLine]", 
     RowBox[{"drawBinaryTree", "[", "newT", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "With these in place, let's look at the result for an example. Consider ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "+", "y"}], ")"}], 
    OverscriptBox["x", "_"]}], TraditionalForm]]],
 ", the subject of Example 1(a) from the text."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"logicTree", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", "||", "y"}], ")"}], "&&", 
    RowBox[{"!", "x"}]}], ",", 
   RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"drawBinaryTree", "[", 
  RowBox[{"joinTrees", "[", 
   RowBox[{"And", ",", 
    RowBox[{"drawBinaryTree", "[", 
     RowBox[{"joinTrees", "[", 
      RowBox[{"Or", ",", 
       RowBox[{"drawBinaryTree", "[", 
        RowBox[{"newExpressionTree", "[", "x", "]"}], "]"}], ",", 
       RowBox[{"drawBinaryTree", "[", 
        RowBox[{"newExpressionTree", "[", "y", "]"}], "]"}]}], "]"}], "]"}], 
    ",", 
    RowBox[{"drawBinaryTree", "[", 
     RowBox[{"addLeftBranch", "[", 
      RowBox[{"Not", ",", 
       RowBox[{"drawBinaryTree", "[", 
        RowBox[{"newExpressionTree", "[", "x", "]"}], "]"}]}], "]"}], "]"}], 
    ",", 
    RowBox[{"ImagePadding", "\[Rule]", "10"}]}], "]"}], "]"}]], "Output"]
}, Open  ]],

Cell["Compare this diagram to Figure 4(a) in the text.", "Text"],

Cell[TextData[{
 "Note that the function above will not work correctly on expressions, such \
as the expression ",
 StyleBox["G1", "Input"],
 ", that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " interprets as applications of ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["Or",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Or"], "FunctionLink"],
 " with more than two arguments. Observe that the ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 " of ",
 StyleBox["G1", "Input"],
 " reveals that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " considers it to include an ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 " applied to three arguments."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullForm", "[", "G1", "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Or", "[", 
    RowBox[{
     RowBox[{"And", "[", 
      RowBox[{"a", ",", "b"}], "]"}], ",", 
     RowBox[{"And", "[", 
      RowBox[{
       RowBox[{"Not", "[", "a", "]"}], ",", "b", ",", 
       RowBox[{"Not", "[", "c", "]"}]}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[TextData[{
 "It is left as an exercise for the reader to alter the ",
 StyleBox["logicTree", "Input"],
 " function to handle this case. "
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["12.4 Minimization of Circuits", "Section",
 CellTags->"Section4"],

Cell[TextData[{
 "In this section we will discuss the use of the ",
 StyleBox[ButtonBox["BooleanMinimize",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanMinimize"], "FunctionLink"],
 " function for minimizing circuits, and we will provide an implementation of \
the Quine-McCluskey method."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "The ",
 StyleBox["BooleanMinimize", "Input"],
 " Function"
}], "Subsection",
 CellTags->"Section4a"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["BooleanMinimize",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanMinimize"], "FunctionLink"],
 " function, applied to a Boolean expression, finds a minimal representation \
of the expression in disjunctive normal form."
}], "Text"],

Cell[TextData[{
 "For example, we apply ",
 StyleBox[ButtonBox["BooleanMinimize",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanMinimize"], "FunctionLink"],
 " to the expression we obtained for the output of the circuit diagram at the \
beginning of Section 12.3."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanMinimize", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "&&", "b"}], ")"}], "||", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"!", "a"}], "&&", "b", "&&", 
     RowBox[{"!", "c"}]}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"a", "&&", "b"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"b", "&&", 
    RowBox[{"!", "c"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The result indicates that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Not]", "a"}], TraditionalForm]]],
 " can be removed as an input to the second AND gate."
}], "Text"],

Cell[TextData[{
 "The result of ",
 StyleBox[ButtonBox["BooleanMinimize",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanMinimize"], "FunctionLink"],
 " is guaranteed to be of minimal length among all possible disjunctive \
normal form representations of the input, however, such a minimal expression \
is not unique. Note that if you don\[CloseCurlyQuote]t care that the \
expression be in disjunctive normal form, the ",
 StyleBox[ButtonBox["Simplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Simplify"], "FunctionLink"],
 " function will produce a shorter expression."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "&&", "b"}], ")"}], "||", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"!", "a"}], "&&", "b", "&&", 
     RowBox[{"!", "c"}]}], ")"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"a", "||", 
    RowBox[{"!", "c"}]}], ")"}], "&&", "b"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["BooleanMinimize",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanMinimize"], "FunctionLink"],
 " function can also accept, as a second argument, all the same forms as ",
 StyleBox[ButtonBox["BooleanConvert",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanConvert"], "FunctionLink"],
 " in order to produce minimal expressions of different forms. For example, \
to find a minimal conjunctive normal form expression for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "1"], TraditionalForm]]],
 ", you enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanMinimize", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "&&", "b"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "a"}], "&&", "b", "&&", 
      RowBox[{"!", "c"}]}], ")"}]}], ",", "\"\<CNF\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"a", "||", 
    RowBox[{"!", "c"}]}], ")"}], "&&", "b"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Don\[CloseCurlyQuote]t Care conditions", "Subsection",
 CellTags->"Section4b"],

Cell[TextData[{
 "Informally, a set of don't care conditions for a Boolean function ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " is a set of points in the domain of ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " whose images do not concern us."
}], "Text"],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " is a function on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " variables, then its domain is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"{", 
     RowBox[{"true", ",", "false"}], "}"}], "n"], TraditionalForm]]],
 ". Let ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 " be the subset of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"{", 
     RowBox[{"true", ",", "false"}], "}"}], "n"], TraditionalForm]]],
 " for which the value of ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " is specified. If we think of ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " as fully defined on this subset ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ", then we are interested in the family of all extensions of ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " to all of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"{", 
     RowBox[{"true", ",", "false"}], "}"}], "n"], TraditionalForm]]],
 ". In other words, the set of all ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " defined on ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"{", 
     RowBox[{"true", ",", "false"}], "}"}], "n"], TraditionalForm]]],
 " that agree with ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " on ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]]],
 ". The goal is to choose the particular ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " that is simplest. That is, the ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 " that has the smallest sum of products expansion."
}], "Text"],

Cell[TextData[{
 "We should pause to consider the size of this problem. If there are ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " don't care points, then there are ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", "d"], TraditionalForm]]],
 " possible extensions ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]]],
 ". Considering every possible extension can become rather time consuming."
}], "Text"],

Cell[TextData[{
 "Consider the Boolean function ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " defined by the following table of values, in which \
\[OpenCurlyDoubleQuote]d\[CloseCurlyDoubleQuote] in the final column \
indicates a don't care condition."
}], "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {Cell["x"], Cell["y"], Cell["z"], Cell["F(x,y,z)"]},
   {"true", "true", "true", "true"},
   {"true", "true", "false", "false"},
   {"true", "false", "true", "false"},
   {"true", "false", "false", "true"},
   {"false", "true", "true", "d"},
   {"false", "true", "false", "d"},
   {"false", "false", "true", "false"},
   {"false", "false", "false", "true"}
  },
  BaseStyle->{FontFamily -> "Times"},
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
    "RowsIndexed" -> {}}]]]], "Text"],

Cell[TextData[{
 "The points that must evaluate to true are: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"false", ",", "false", ",", "false"}], ")"}], ",", 
     RowBox[{"(", 
      RowBox[{"true", ",", "false", ",", "false"}], ")"}], ",", 
     RowBox[{"(", 
      RowBox[{"true", ",", "true", ",", "true"}], ")"}]}], "}"}], 
   TraditionalForm]]],
 " and the don\[CloseCurlyQuote]t care conditions are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"false", ",", "true", ",", "false"}], ")"}], ",", 
     RowBox[{"(", 
      RowBox[{"false", ",", "true", ",", "true"}], ")"}]}], "}"}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "In Section 12.2, we showed how to use ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " to define a Boolean function in terms of a table. We also saw above that \
you can specify a default output by using a ",
 StyleBox[ButtonBox["BlankSequence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BlankSequence"], "FunctionLink"],
 " (",
 StyleBox["__", "Input"],
 "). For example, the following returns the Boolean expression that is true \
on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"true", ",", "false", ",", "true", ",", "false"}], ")"}], 
   TraditionalForm]]],
 " and false otherwise."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanFunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"True", ",", "False", ",", "True", ",", "False"}], "}"}], 
      "\[Rule]", "True"}], ",", 
     RowBox[{
      RowBox[{"{", "__", "}"}], "\[Rule]", "False"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"!", "w"}], "&&", "x", "&&", 
  RowBox[{"!", "y"}], "&&", "z"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can specify a don\[CloseCurlyQuote]t care condition within a call to ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " by identifying a don\[CloseCurlyQuote]t care condition, e.g., ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"false", ",", "true", ",", "false"}], ")"}], TraditionalForm]]],
 ", with a blank. "
}], "Text"],

Cell[TextData[{
 "So we can determine a Boolean function defined by the table above as \
follows. (Note that we could use a ",
 StyleBox[ButtonBox["BlankSequence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BlankSequence"], "FunctionLink"],
 " (",
 StyleBox["__", "Input"],
 ") to simplify the input, but in this example we list all the elements of \
the domain for clarity.)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanFunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"True", ",", "True", ",", "True"}], "}"}], "\[Rule]", "True"}],
      ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"True", ",", "True", ",", "False"}], "}"}], "\[Rule]", 
      "False"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"True", ",", "False", ",", "True"}], "}"}], "\[Rule]", 
      "False"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"True", ",", "False", ",", "False"}], "}"}], "\[Rule]", 
      "True"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "True", ",", "True"}], "}"}], "\[Rule]", "_"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "True", ",", "False"}], "}"}], "\[Rule]", "_"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "False", ",", "True"}], "}"}], "\[Rule]", 
      "False"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"False", ",", "False", ",", "False"}], "}"}], "\[Rule]", 
      "True"}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"x", "&&", "y", "&&", "z"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "x"}], "&&", 
    RowBox[{"!", "z"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "y"}], "&&", 
    RowBox[{"!", "z"}]}], ")"}]}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Quine-McCluskey", "Subsection",
 CellTags->"Section4c"],

Cell["\<\
We conclude with an implementation of the Quine-McCluskey method. This method \
is fairly involved and it will take considerable effort to implement it \
correctly, but understanding this algorithm is worthwhile.\
\>", "Text"],

Cell["\<\
It will be helpful to have an example that we can use to illustrate the \
method as we build the function. The expression we use for the example is \
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"w", " ", "x", " ", 
    OverscriptBox["y", "_"], " ", 
    OverscriptBox["z", "_"]}], "+", 
   RowBox[{"w", " ", 
    OverscriptBox["x", "_"], " ", "y", " ", "z"}], "+", 
   RowBox[{"w", " ", 
    OverscriptBox["x", "_"], " ", "y", " ", 
    OverscriptBox["z", "_"]}], "+", 
   RowBox[{"w", " ", 
    OverscriptBox["x", "_"], " ", 
    OverscriptBox["y", "_"], " ", 
    OverscriptBox["z", "_"]}], "+", 
   RowBox[{
    OverscriptBox["w", "_"], " ", "x", " ", "y", " ", "z"}], "+", 
   RowBox[{
    OverscriptBox["w", "_"], " ", "x", " ", 
    OverscriptBox["y", "_"], " ", "z"}], "+", 
   RowBox[{
    OverscriptBox["w", "_"], " ", "x", " ", 
    OverscriptBox["y", "_"], " ", 
    OverscriptBox["z", "_"]}], "+", 
   RowBox[{
    OverscriptBox["w", "_"], " ", 
    OverscriptBox["x", "_"], " ", "y", " ", "z"}], "+", 
   RowBox[{
    OverscriptBox["w", "_"], " ", 
    OverscriptBox["x", "_"], " ", 
    OverscriptBox["y", "_"], " ", "z"}], "+", 
   RowBox[{
    OverscriptBox["w", "_"], " ", 
    OverscriptBox["x", "_"], " ", 
    OverscriptBox["y", "_"], " ", 
    OverscriptBox["z", "_"]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "We assign this to the symbol ",
 StyleBox["F", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"F", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"w", "&&", "x", "&&", 
      RowBox[{"!", "y"}], "&&", 
      RowBox[{"!", "z"}]}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{"w", "&&", 
      RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{"w", "&&", 
      RowBox[{"!", "x"}], "&&", "y", "&&", 
      RowBox[{"!", "z"}]}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{"w", "&&", 
      RowBox[{"!", "x"}], "&&", 
      RowBox[{"!", "y"}], "&&", 
      RowBox[{"!", "z"}]}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "w"}], "&&", "x", "&&", "y", "&&", "z"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "w"}], "&&", "x", "&&", 
      RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "w"}], "&&", "x", "&&", 
      RowBox[{"!", "y"}], "&&", 
      RowBox[{"!", "z"}]}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "w"}], "&&", 
      RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "w"}], "&&", 
      RowBox[{"!", "x"}], "&&", 
      RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"!", "w"}], "&&", 
      RowBox[{"!", "x"}], "&&", 
      RowBox[{"!", "y"}], "&&", 
      RowBox[{"!", "z"}]}], ")"}]}]}], ";"}]], "Input",
 InitializationCell->True],

Cell["\<\
Let us begin by (very) briefly outlining the approach. More details will be \
given as we proceed.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Transform the minterms into bit strings.", "Item1Numbered"],

Cell["Group the bit strings by the number of 1s.", "Item1Numbered"],

Cell["Combine bit strings that differ in exactly one location.", \
"Item1Numbered"],

Cell["\<\
Repeat steps 2 and 3 until no additional combinations are possible.\
\>", "Item1Numbered"],

Cell["\<\
Identify the prime implicants (those bit strings not used in a \
simplification) and form the coverage table.\
\>", "Item1Numbered"],

Cell["\<\
 Identify the essential prime implicants and update the table.\
\>", "Item1Numbered"],

Cell["\<\
Process the remaining prime implicants using a heuristic approach in order to \
achieve complete coverage.\
\>", "Item1Numbered"]
}, Open  ]],

Cell[TextData[{
 "Implementing this will require several different functions that will come \
together to achieve the goal of minimizing the expression for ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell["Modifying Arguments", "Subsubsection"],

Cell[TextData[{
 "Before we begin implementing the method, we take a moment to explain the ",
 StyleBox[ButtonBox["HoldRest",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldRest"], "FunctionLink"],
 " attribute. Earlier in this manual, we have seen how to use a held argument \
to allow modification of an argument to a function. We will need to do this \
again here in order to avoid the need to copy data structures that must be \
modified by a function."
}], "Text"],

Cell["\<\
Holding parameters to a function means that, when you call the function on a \
symbol, instead of applying the function to the object stored in that symbol, \
the function is given the name of the symbol itself. This allows the symbol \
name to be reassigned and otherwise modified within the function. \
\>", "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["HoldRest",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldRest"], "FunctionLink"],
 " attribute causes all but the first argument to a function to be held. For \
example, the following function updates the symbol given as the second \
argument to be the sum of what it previously stored and the first argument, \
and appends the result to the list associated to the symbol given as the \
third argument."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"exampleHold1", "=", "5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"exampleHold2", "=", "12"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"exampleHold3", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"exampleHoldFunction", ",", 
    RowBox[{"{", "HoldRest", "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"exampleHoldFunction", "[", 
   RowBox[{"a_", ",", "b_", ",", "c_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"b", "=", 
      RowBox[{"a", "+", "b"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"c", ",", "b"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleHoldFunction", "[", 
  RowBox[{"exampleHold1", ",", "exampleHold2", ",", "exampleHold3"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "17"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Observe that the values stored in the last two arguments have changed. \
Without the ",
 StyleBox[ButtonBox["HoldRest",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldRest"], "FunctionLink"],
 " attribute, both expressions in the body of the ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 " would have produced errors."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["exampleHold2"], "Input"],

Cell[BoxData["17"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["exampleHold3"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "17"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Other attributes that can be used to hold arguments are ",
 StyleBox[ButtonBox["HoldFirst",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldFirst"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["HoldAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldAll"], "FunctionLink"],
 "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Transforming Minterms Into Bit Strings", "Subsubsection"],

Cell[TextData[{
 "The first task is to process the input. That is, ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 " must be transformed into a list of bit strings. This is not strictly \
necessary, but it makes working with the minterms more convenient. We \
represent bit strings as lists of 0s and 1s."
}], "Text"],

Cell["\<\
We begin by creating a function to transform a single minterm into a bit \
string. We assume that the input to this function will be a properly formed \
minterm, that is, a conjunction of variables and negations of variables. We \
require that a list of variables be provided to the function, so that the bit \
string can be formed in the proper order.\
\>", "Text"],

Cell[TextData[{
 "Consider the following minterm, which is the fourth minterm in our example ",
 Cell[BoxData[
  FormBox["F", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"minterm", "=", 
  RowBox[{"w", "&&", 
   RowBox[{"!", "x"}], "&&", "y", "&&", 
   RowBox[{"!", "z"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"w", "&&", 
  RowBox[{"!", "x"}], "&&", "y", "&&", 
  RowBox[{"!", "z"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Fortunately, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " automatically transforms a series of conjunctions into a single \
application of the ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 " (",
 StyleBox["&&", "Input"],
 ") function with multiple arguments. Applying ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 " illustrates."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullForm", "[", "minterm", "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"And", "[", 
    RowBox[{"w", ",", 
     RowBox[{"Not", "[", "x", "]"}], ",", "y", ",", 
     RowBox[{"Not", "[", "z", "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 "\[CloseCurlyQuote]s first argument can have any head, not just a ",
 StyleBox[ButtonBox["List",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/List"], "FunctionLink"],
 ". So, we can determine that ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 " is part of the conjunction but that ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is not by applying ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " to ",
 StyleBox["minterm", "Input"],
 " and the variables."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MemberQ", "[", 
  RowBox[{"minterm", ",", "w"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MemberQ", "[", 
  RowBox[{"minterm", ",", "x"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "But of course, the negation of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is part of ",
 StyleBox["minterm", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MemberQ", "[", 
  RowBox[{"minterm", ",", 
   RowBox[{"Not", "[", "x", "]"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell["\<\
To transform the minterm into a bit string, we only need to check, for each \
variable, whether the variable or its negation is in the list. Recall that we \
will insist that the function be given the list of variables as an argument \
to maintain the proper order of the variables.\
\>", "Text"],

Cell["We first assign the list of variables to a symbol.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableList", "=", 
  RowBox[{"{", 
   RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Now create a list, initialized to the proper length, for the bit string.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"bitstring", "=", 
  RowBox[{"ConstantArray", "[", 
   RowBox[{"Null", ",", "4"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Null", ",", "Null", ",", "Null", ",", "Null"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Finally, we use a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop to check, for each variable in the variable list, whether the \
variable is in the minterm. If the variable is a member of ",
 StyleBox["minterm", "Input"],
 ", then we change the bit to 1. If the negation is in ",
 StyleBox["minterm", "Input"],
 ", we set the value in the bit string to 0. Otherwise, we place the \
character \[OpenCurlyDoubleQuote]-\[OpenCurlyDoubleQuote] in the list, to \
indicate the absence of the variable in the string."
}], "Text"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"i", "=", "1"}], ",", 
   RowBox[{"i", "\[LessEqual]", 
    RowBox[{"Length", "[", "variableList", "]"}]}], ",", 
   RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
   RowBox[{"Which", "[", 
    RowBox[{
     RowBox[{"MemberQ", "[", 
      RowBox[{"minterm", ",", 
       RowBox[{"variableList", "[", 
        RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
     RowBox[{
      RowBox[{"bitstring", "[", 
       RowBox[{"[", "i", "]"}], "]"}], "=", "1"}], ",", "\[IndentingNewLine]", 
     RowBox[{"MemberQ", "[", 
      RowBox[{"minterm", ",", 
       RowBox[{"Not", "[", 
        RowBox[{"variableList", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "]"}], ",", 
     RowBox[{
      RowBox[{"bitstring", "[", 
       RowBox[{"[", "i", "]"}], "]"}], "=", "0"}], ",", "\[IndentingNewLine]",
      "True", ",", 
     RowBox[{
      RowBox[{"bitstring", "[", 
       RowBox[{"[", "i", "]"}], "]"}], "=", "\"\<-\>\""}]}], "]"}]}], 
  "\[IndentingNewLine]", "]"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "This has created the bit string associated to ",
 StyleBox["minterm", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["bitstring"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}]], "Output"]
}, Open  ]],

Cell["We condense this process into a single function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"mtToBitstring", "[", 
   RowBox[{"minterm_", ",", "variableList_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "bitstring"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"bitstring", "=", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"Null", ",", 
        RowBox[{"Length", "[", "variableList", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "variableList", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Which", "[", 
        RowBox[{
         RowBox[{"MemberQ", "[", 
          RowBox[{"minterm", ",", 
           RowBox[{"variableList", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"bitstring", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "=", "1"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"MemberQ", "[", 
          RowBox[{"minterm", ",", 
           RowBox[{"Not", "[", 
            RowBox[{"variableList", "[", 
             RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"bitstring", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "=", "0"}], ",", 
         "\[IndentingNewLine]", "True", ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"bitstring", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "=", "\"\<-\>\""}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "bitstring"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mtToBitstring", "[", 
  RowBox[{"minterm", ",", 
   RowBox[{"{", 
    RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Transforming the Original Expression Into Bit Strings", "Subsubsection"],

Cell["\<\
Now that we have the means for transforming a single minterm into a bit \
string, we are ready to transform an expression in disjunctive normal form \
into a list of bit strings.\
\>", "Text"],

Cell[TextData[{
 "Observe that an expression in disjunctive normal form is the ",
 StyleBox[ButtonBox["Or",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Or"], "FunctionLink"],
 " (||) function applied to minterms. Again, ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 " reveals this convenient structure."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullForm", "[", "F", "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Or", "[", 
    RowBox[{
     RowBox[{"And", "[", 
      RowBox[{"w", ",", "x", ",", 
       RowBox[{"Not", "[", "y", "]"}], ",", 
       RowBox[{"Not", "[", "z", "]"}]}], "]"}], ",", 
     RowBox[{"And", "[", 
      RowBox[{"w", ",", 
       RowBox[{"Not", "[", "x", "]"}], ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"And", "[", 
      RowBox[{"w", ",", 
       RowBox[{"Not", "[", "x", "]"}], ",", "y", ",", 
       RowBox[{"Not", "[", "z", "]"}]}], "]"}], ",", 
     RowBox[{"And", "[", 
      RowBox[{"w", ",", 
       RowBox[{"Not", "[", "x", "]"}], ",", 
       RowBox[{"Not", "[", "y", "]"}], ",", 
       RowBox[{"Not", "[", "z", "]"}]}], "]"}], ",", 
     RowBox[{"And", "[", 
      RowBox[{
       RowBox[{"Not", "[", "w", "]"}], ",", "x", ",", "y", ",", "z"}], "]"}], 
     ",", 
     RowBox[{"And", "[", 
      RowBox[{
       RowBox[{"Not", "[", "w", "]"}], ",", "x", ",", 
       RowBox[{"Not", "[", "y", "]"}], ",", "z"}], "]"}], ",", 
     RowBox[{"And", "[", 
      RowBox[{
       RowBox[{"Not", "[", "w", "]"}], ",", "x", ",", 
       RowBox[{"Not", "[", "y", "]"}], ",", 
       RowBox[{"Not", "[", "z", "]"}]}], "]"}], ",", 
     RowBox[{"And", "[", 
      RowBox[{
       RowBox[{"Not", "[", "w", "]"}], ",", 
       RowBox[{"Not", "[", "x", "]"}], ",", "y", ",", "z"}], "]"}], ",", 
     RowBox[{"And", "[", 
      RowBox[{
       RowBox[{"Not", "[", "w", "]"}], ",", 
       RowBox[{"Not", "[", "x", "]"}], ",", 
       RowBox[{"Not", "[", "y", "]"}], ",", "z"}], "]"}], ",", 
     RowBox[{"And", "[", 
      RowBox[{
       RowBox[{"Not", "[", "w", "]"}], ",", 
       RowBox[{"Not", "[", "x", "]"}], ",", 
       RowBox[{"Not", "[", "y", "]"}], ",", 
       RowBox[{"Not", "[", "z", "]"}]}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Our goal is to produce a list of the bit strings obtained from the \
minterms. We can transform the disjunction into a list by using the ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ") function to replace the ",
 StyleBox[ButtonBox["Or",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Or"], "FunctionLink"],
 " (",
 StyleBox["||", "Input"],
 ") head with a ",
 StyleBox[ButtonBox["List",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/List"], "FunctionLink"],
 " head."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"List", "@@", "F"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"w", "&&", "x", "&&", 
    RowBox[{"!", "y"}], "&&", 
    RowBox[{"!", "z"}]}], ",", 
   RowBox[{"w", "&&", 
    RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ",", 
   RowBox[{"w", "&&", 
    RowBox[{"!", "x"}], "&&", "y", "&&", 
    RowBox[{"!", "z"}]}], ",", 
   RowBox[{"w", "&&", 
    RowBox[{"!", "x"}], "&&", 
    RowBox[{"!", "y"}], "&&", 
    RowBox[{"!", "z"}]}], ",", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", "x", "&&", "y", "&&", "z"}], ",", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", "x", "&&", 
    RowBox[{"!", "y"}], "&&", "z"}], ",", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", "x", "&&", 
    RowBox[{"!", "y"}], "&&", 
    RowBox[{"!", "z"}]}], ",", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", 
    RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ",", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", 
    RowBox[{"!", "x"}], "&&", 
    RowBox[{"!", "y"}], "&&", "z"}], ",", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", 
    RowBox[{"!", "x"}], "&&", 
    RowBox[{"!", "y"}], "&&", 
    RowBox[{"!", "z"}]}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Then we just need to apply the ",
 StyleBox["mtToBitstring", "Input"],
 " function to each member of the list. We can do this by using the ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " (",
 StyleBox["/@", "Input"],
 ") function, which applies a function (given as the first argument) to a \
list (given as the second argument) and returns the list obtained by applying \
the function to each element of the list. Since the ",
 StyleBox["mtToBitstring", "Input"],
 " requires two arguments, not just one, the first argument to ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " (",
 StyleBox["/@", "Input"],
 ") will be a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") obtained by calling ",
 StyleBox["mtToBitstring", "Input"],
 " on a ",
 StyleBox[ButtonBox["Slot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Slot"], "FunctionLink"],
 " (",
 StyleBox["#", "Input"],
 ") and the list of variables."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"mtToBitstring", "[", 
     RowBox[{"#", ",", 
      RowBox[{"{", 
       RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}]}], "]"}], "&"}], 
   ",", 
   RowBox[{"List", "@@", "F"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["We define a function based on this model.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dnfToBitList", "[", 
   RowBox[{"dnfExpr_", ",", "variableList_"}], "]"}], ":=", 
  RowBox[{"Map", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"mtToBitstring", "[", 
      RowBox[{"#", ",", "variableList"}], "]"}], "&"}], ",", 
    RowBox[{"List", "@@", "dnfExpr"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We now apply this function to the example expression and store the result \
as the symbol ",
 StyleBox["Fbits", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Fbits", "=", 
  RowBox[{"dnfToBitList", "[", 
   RowBox[{"F", ",", 
    RowBox[{"{", 
     RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Transforming Bit Strings Into Minterms", "Subsubsection"],

Cell["\<\
At the conclusion of the Quine-McCluskey process, we will want to display the \
result in disjunctive normal form. This will require that we turn bit strings \
back into minterms.\
\>", "Text"],

Cell["\<\
Note that since this function will be applied at the end of the process, it \
may be that some of the variables have been removed. We will be using the \
string \[OpenCurlyDoubleQuote]-\[OpenCurlyDoubleQuote] in a bit string to \
indicate the elimination of a variable.\
\>", "Text"],

Cell["\<\
This function will require the bit string and a list of variable names as its \
input. It operates in two stages. First, it processes the variable list based \
on the content of the bit string. It initializes an empty list and, for each \
variable, appends the variable or its negation or does nothing, depending on \
the content of the bit string.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"bitstr", "=", 
  RowBox[{"{", 
   RowBox[{"0", ",", "1", ",", "\"\<-\>\"", ",", "0"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1", ",", "\<\"-\"\>", ",", "0"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"outList", "=", 
  RowBox[{"{", "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"i", "=", "1"}], ",", 
   RowBox[{"i", "\[LessEqual]", 
    RowBox[{"Length", "[", "variableList", "]"}]}], ",", 
   RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
   RowBox[{"Switch", "[", 
    RowBox[{
     RowBox[{"bitstr", "[", 
      RowBox[{"[", "i", "]"}], "]"}], ",", "\[IndentingNewLine]", "1", ",", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"outList", ",", 
       RowBox[{"variableList", "[", 
        RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
     "0", ",", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"outList", ",", 
       RowBox[{"Not", "[", 
        RowBox[{"variableList", "[", 
         RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "]"}], ",", 
     "\[IndentingNewLine]", "\"\<-\>\"", ",", "Null"}], "]"}]}], 
  "\[IndentingNewLine]", "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData["outList"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"!", "w"}], ",", "x", ",", 
   RowBox[{"!", "z"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note the use of the ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 " function. Recall that ",
 StyleBox[ButtonBox["Switch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Switch"], "FunctionLink"],
 " evaluates its first argument, and then compares that result to the \
arguments with even index, executing the argument following the first of the \
even-indexed arguments that matches the result of the first argument."
}], "Text"],

Cell[TextData[{
 "Once this list is formed, we form the conjunction of the elements by using ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ") to change the ",
 StyleBox[ButtonBox["List",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/List"], "FunctionLink"],
 " head into ",
 StyleBox[ButtonBox["And",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/And"], "FunctionLink"],
 " (",
 StyleBox["&&", "Input"],
 ")."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"And", "@@", "outList"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"!", "w"}], "&&", "x", "&&", 
  RowBox[{"!", "z"}]}]], "Output"]
}, Open  ]],

Cell["Here is the function based on this process.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"bitStringToMT", "[", 
   RowBox[{"bitstring_", ",", "variableList_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"outList", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"outList", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "variableList", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Switch", "[", 
        RowBox[{
         RowBox[{"bitstring", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", "\[IndentingNewLine]", "1", 
         ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"outList", ",", 
           RowBox[{"variableList", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", "0", ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"outList", ",", 
           RowBox[{"Not", "[", 
            RowBox[{"variableList", "[", 
             RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", "\"\<-\>\"", ",", "Null"}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"And", "@@", "outList"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Applied to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}], TraditionalForm]]],
 ", we see that ",
 StyleBox["bitStringToMT", "Input"],
 " reproduces the original minterm example."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"bitStringToMT", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"!", "w"}], "&&", "x", "&&", 
  RowBox[{"!", "y"}], "&&", "z"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "And applied to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "\"\<-\>\"", ",", "1"}], "}"}], 
   TraditionalForm]]],
 ", it removes the ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " and negates ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"bitStringToMT", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "\"\<-\>\"", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"!", "w"}], "&&", "x", "&&", "z"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The final result of our Quine-McCluskey process will be a list of bit \
strings. To produce the associated disjunctive normal form expression, we \
only need to apply ",
 StyleBox["bitStringToMT", "Input"],
 " to each element of the list and then join the elements of the list in a \
disjunction."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"bitListToDNF", "[", 
   RowBox[{"bitList_", ",", "variableList_"}], "]"}], ":=", 
  RowBox[{"Map", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"bitStringToMT", "[", 
      RowBox[{"#", ",", "variableList"}], "]"}], "&"}], ",", 
    RowBox[{"Or", "@@", "bitList"}]}], "]"}]}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initializing the Source Table", "Subsubsection"],

Cell["\<\
In order to form the coverage table in the second part of the method, we need \
to know which of the original minterms are covered by which of the prime \
implicants. Refer to Tables 3 and 6 in the text. Notice that each bit string \
in those tables is associated with either a single number, in the case of the \
original minterms, or lists of numbers, for the derived products.\
\>", "Text"],

Cell[TextData[{
 "We will store this information as an indexed symbol whose indices are the \
bit strings and whose values are lists of integers. Given the ",
 StyleBox["Fbits", "Input"],
 " list, we initialize this indexed symbol with the elements of ",
 StyleBox["Fbits", "Input"],
 " as the indices. The corresponding entries will be the whose sole element \
is the bit string's position in ",
 StyleBox["Fbits", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "We will refer to this as the \[OpenCurlyDoubleQuote]coverage dictionary,\
\[CloseCurlyDoubleQuote] since it allows us to look up any bit string and \
determine all of the original minterms covered by it. The following function \
accepts the name of a symbol and the ",
 StyleBox["Fbits", "Input"],
 " list as arguments and assigns the coverage dictionary to the symbol. Note \
that we must set the ",
 StyleBox[ButtonBox["HoldFirst",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HoldFirst"], "FunctionLink"],
 " attribute in order to pass the symbol name as an argument. We apply ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 " so as to remove anything already associated to the symbol."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"initCoverDict", ",", 
    RowBox[{"{", "HoldFirst", "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"initCoverDict", "[", 
   RowBox[{"symbol_", ",", "L_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "i", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Clear", "[", "symbol", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "L", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"symbol", "[", 
         RowBox[{"L", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}], "=", 
        RowBox[{"{", "i", "}"}]}]}], "\[IndentingNewLine]", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Applying this function to a symbol and ",
 StyleBox["Fbits", "Input"],
 " produces the initial coverage dictionary. "
}], "Text"],

Cell[BoxData[
 RowBox[{"initCoverDict", "[", 
  RowBox[{"coverageDict", ",", "Fbits"}], "]"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We can check what is stored in the indexed symbol ",
 StyleBox["coverageDict", "Input"],
 " as shown below. Remember that the indices are the elements of the list of \
bit strings ",
 StyleBox["Fbits", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"coverageDict", "[", 
  RowBox[{"{", 
   RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "6", "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The value, after initialization, is the location within ",
 StyleBox["Fbits", "Input"],
 " where the bit string is stored."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Fbits", "[", 
  RowBox[{"[", "6", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Grouping by the Number of 1s", "Subsubsection"],

Cell["\<\
Step 2 in our outline is to group the bit strings by the number of 1s.\
\>", "Text"],

Cell[TextData[{
 "The reason for this step is to improve the efficiency of finding \
simplifications to make. Since two bit strings can be combined only when they \
are identical except for one location, the only possible combinations are \
when one bit string has ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " 1s and the other has ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "After step 1 is concluded, we have a list of bit strings. That will be the \
starting point for the function we create for this step. The result of this \
step will be to turn the list of bit strings into a list of lists of bit \
strings, which we'll call ",
 StyleBox["groups", "Input"],
 ". In location ",
 StyleBox["i", "Input"],
 " of ",
 StyleBox["groups", "Input"],
 " will be the set of all bit strings with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "-", "1"}], TraditionalForm]]],
 " 1s. So location 1 will have the bit strings with no 1s, location 2 will \
contain the bit strings with a single 1, etc."
}], "Text"],

Cell[TextData[{
 "We know that the number of 1s in any bit string must be between 0 and the \
length of the bit string. We initialize ",
 StyleBox["groups", "Input"],
 " to be the list of empty lists. The maximum number of 1s in equal to the \
length of a bit string, which we can obtain from the size of the first \
element of ",
 StyleBox["Fbits", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"groups", "=", 
  RowBox[{"ConstantArray", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{
     RowBox[{"Length", "[", 
      RowBox[{"First", "[", "Fbits", "]"}], "]"}], "+", "1"}]}], 
   "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Since the bit strings had four entries, ",
 StyleBox["groups", "Input"],
 " now consists of five copies of the empty list."
}], "Text"],

Cell[TextData[{
 "For each member of ",
 StyleBox["Fbits", "Input"],
 ", we need to count the number of 1s. We will create a small function to do \
this."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"count1s", "[", "bitstring_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"c", "=", "0"}], ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    
    RowBox[{
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"i", "\[Equal]", "1"}], ",", 
         RowBox[{"c", "++"}]}], "]"}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "bitstring"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "c"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell["We test this function on a small example.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"count1s", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "0", ",", "1"}], 
   "}"}], "]"}]], "Input"],

Cell[BoxData["4"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We use ",
 StyleBox["count1s", "Input"],
 " to sort the members of ",
 StyleBox["Fbits", "Input"],
 " into ",
 StyleBox["groups", "Input"],
 ". Using a loop to step through the ",
 StyleBox["Fbits", "Input"],
 " list, we apply ",
 StyleBox["count1s", "Input"],
 " and add 1 to the result (since the bit strings with no 1s are in the first \
position) to obtain the correct location for that bit string in ",
 StyleBox["groups", "Input"],
 ". We then add that bit string to the correct sublist in ",
 StyleBox["groups", "Input"],
 "."
}], "Text"],

Cell["Here is the function implementing this.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sortGroups", "[", "bitstringList_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"groups", ",", "bitstring", ",", "c"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"groups", "=", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{
        RowBox[{"{", "}"}], ",", 
        RowBox[{
         RowBox[{"Length", "[", 
          RowBox[{"First", "[", "bitstringList", "]"}], "]"}], "+", "1"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"c", "=", 
         RowBox[{"count1s", "[", "bitstring", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"groups", "[", 
          RowBox[{"[", 
           RowBox[{"c", "+", "1"}], "]"}], "]"}], "=", 
         RowBox[{"Append", "[", 
          RowBox[{
           RowBox[{"groups", "[", 
            RowBox[{"[", 
             RowBox[{"c", "+", "1"}], "]"}], "]"}], ",", "bitstring"}], 
          "]"}]}]}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"bitstring", ",", "bitstringList"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "groups"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Here is the result of sorting ",
 StyleBox["Fbits", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"groups", "=", 
  RowBox[{"sortGroups", "[", "Fbits", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Applying ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 " will make the output easier to read. The ",
 StyleBox[ButtonBox["TableDepth",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableDepth"], "FunctionLink"],
 " option prevents ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 " from splitting the sublists into subtables."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"groups", ",", 
   RowBox[{"TableDepth", "\[Rule]", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", "}"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$, TableDepth -> 1]]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Combining Bit Strings", "Subsubsection"],

Cell[TextData[{
 "Step 3 is to combine all of the bit strings that differ in exactly one \
location. We first write a function that takes as input two bit strings and \
either combines them if, in fact, they do differ in exactly one location, or \
returns ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " if they do not."
}], "Text"],

Cell["\<\
This function needs to do two tasks. First, it has to check to see whether or \
not the two bit strings differ in more than one location. Second, it needs to \
combine them if they are allowed to be combined.\
\>", "Text"],

Cell["\<\
Combining two bit strings is easy, provided we know the one location in which \
they differ. For example,\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"bit1", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "\"\<-\>\"", ",", "0", ",", "1", ",", "1"}], 
   "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "\<\"-\"\>", ",", "0", ",", "1", ",", "1"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"bit2", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "\"\<-\>\"", ",", "0", ",", "0", ",", "1"}], 
   "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "\<\"-\"\>", ",", "0", ",", "0", ",", "1"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["You can see that these are identical except in position 4.", "Text"],

Cell["\<\
To merge them, we take either one and replace position 4 with \
\[OpenCurlyDoubleQuote]-\[OpenCurlyDoubleQuote].\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"bit1", "[", 
    RowBox[{"[", "4", "]"}], "]"}], "=", "\"\<-\>\""}], 
  ";"}], "\[IndentingNewLine]", "bit1"}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "\<\"-\"\>", ",", "0", ",", "\<\"-\"\>", ",", "1"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We determine that they differ only in position 4 using a ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " with a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop. Inside a ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 " block, initialize a symbol ",
 StyleBox["pos", "Input"],
 ", for position, to 0. Now begin a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop to compare each pair of entries in the two bit strings. If we find a \
difference, check the value of ",
 StyleBox["pos", "Input"],
 ". If it is still 0, then this is the first difference that has been \
encountered, so set ",
 StyleBox["pos", "Input"],
 " to the position of this difference and continue the loop. If ",
 StyleBox["pos", "Input"],
 " is not 0, however, then we know that this is the second time a difference \
was found. In this case, we immediately ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 ", terminating the loop. Once the loop is complete, if ",
 StyleBox["pos", "Input"],
 " is still 0, then there was no difference, so again we ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 ". Otherwise, ",
 StyleBox["pos", "Input"],
 " stores the location of the sole difference, and",
 " ",
 "we modify one of the bit strings and return it."
}], "Text"],

Cell["Here is the function", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"mergeBitstrings", "[", 
   RowBox[{"bit1_", ",", "bit2_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "pos", ",", "result"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"pos", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "bit1", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"bit1", "[", 
            RowBox[{"[", "i", "]"}], "]"}], "\[NotEqual]", 
           RowBox[{"bit2", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"pos", "\[Equal]", "0"}], ",", 
            RowBox[{"pos", "=", "i"}], ",", 
            RowBox[{"Throw", "[", "False", "]"}]}], "]"}]}], 
         "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"pos", "\[Equal]", "0"}], ",", 
        RowBox[{"Throw", "[", "False", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"result", "=", "bit1"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"result", "[", 
        RowBox[{"[", "pos", "]"}], "]"}], "=", "\"\<-\>\""}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Throw", "[", "result", "]"}]}], "\[IndentingNewLine]", "]"}]}],
    "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We see that it works correctly on our two example bit strings.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"mergeBitstrings", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "\"\<-\>\"", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "\"\<-\>\"", ",", "0", ",", "0", ",", "1"}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "\<\"-\"\>", ",", "0", ",", "\<\"-\"\>", ",", "1"}], 
  "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Searching for Combinations to Make", "Subsubsection"],

Cell[TextData[{
 "The ",
 StyleBox["mergeBitstrings", "Input"],
 " function will do the work of checking to see if bit strings can be merged \
and returning the result if they can. However, we need to give ",
 StyleBox["mergeBitstrings", "Input"],
 " the bit strings to test."
}], "Text"],

Cell["\<\
Recall that, in our example, we have successfully grouped the minterms by the \
number of 1s they contain.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"groups", ",", 
   RowBox[{"TableDepth", "\[Rule]", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{"{", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", "}"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$, TableDepth -> 1]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Next we will produce a list containing all the bit strings formed by \
merging two members of ",
 StyleBox["groups", "Input"],
 ". Note there may be multiple ways to obtain the same bit string, so we will \
think of this collection as a set. We initialize to the empty set."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Fbits1", "=", 
  RowBox[{"{", "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Also recall that it is only possible to merge bit strings that are in \
successive locations in ",
 StyleBox["groups", "Input"],
 ". In other words, we only need to check bit strings when one has ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " 1s and one has ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " 1s. This suggests a loop with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " ranging from 1 to one less than the size of ",
 StyleBox["groups", "Input"],
 ". Within the body of the loop, we will consider the lists with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " 1s (index ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ") and with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " 1s (index ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 "). (Remember that ",
 StyleBox["groups[[1]]", "Input"],
 " is the set of bit strings with 0 1s.)"
}], "Text"],

Cell["The loop is structured as follows.", "Text"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"n", "=", "1"}], ",", 
   RowBox[{"n", "\[LessEqual]", 
    RowBox[{
     RowBox[{"Length", "[", "groups", "]"}], "-", "1"}]}], ",", 
   RowBox[{"n", "++"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"A", "=", 
     RowBox[{"groups", "[", 
      RowBox[{"[", "n", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"B", "=", 
     RowBox[{"groups", "[", 
      RowBox[{"[", 
       RowBox[{"n", "+", "1"}], "]"}], "]"}]}], ";"}]}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "search", " ", "for", " ", "bit", " ", "strings", " ", "from", " ", "A", 
    " ", "and", " ", "B", " ", "to", " ", "merge"}], " ", "*)"}], 
  "\[IndentingNewLine]", "]"}]], "myCode"],

Cell[TextData[{
 "After ",
 StyleBox["A", "Input"],
 " and ",
 StyleBox["B", "Input"],
 " have been defined, we need to compare every possible pair. We use a ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop with indices for the members of ",
 StyleBox["A", "Input"],
 " and another for members of ",
 StyleBox["B", "Input"],
 ". Within the ",
 StyleBox[ButtonBox["Do",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Do"], "FunctionLink"],
 " loop, we use ",
 StyleBox["mergeBitstrings", "Input"],
 " and store the result. If it is not false, we add it to the new list of bit \
strings, ",
 StyleBox["Fbits1", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"n", "=", "1"}], ",", 
   RowBox[{"n", "\[LessEqual]", 
    RowBox[{
     RowBox[{"Length", "[", "groups", "]"}], "-", "1"}]}], ",", 
   RowBox[{"n", "++"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"A", "=", 
     RowBox[{"groups", "[", 
      RowBox[{"[", "n", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"B", "=", 
     RowBox[{"groups", "[", 
      RowBox[{"[", 
       RowBox[{"n", "+", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"Do", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"m", "=", 
        RowBox[{"mergeBitstrings", "[", 
         RowBox[{"a", ",", "b"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"m", "=!=", "False"}], ",", 
         RowBox[{"Fbits1", "=", 
          RowBox[{"Union", "[", 
           RowBox[{"Fbits1", ",", 
            RowBox[{"{", "m", "}"}]}], "]"}]}]}], "]"}]}], 
      "\[IndentingNewLine]", ",", 
      RowBox[{"{", 
       RowBox[{"a", ",", "A"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"b", ",", "B"}], "}"}]}], "\[IndentingNewLine]", "]"}]}]}], 
  "\[IndentingNewLine]", "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData["Fbits1"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "\<\"-\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "\<\"-\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "\<\"-\"\>", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "\<\"-\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "\<\"-\"\>", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"-\"\>", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"-\"\>", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"-\"\>", ",", "1", ",", "0", ",", "0"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["\<\
This is close to the function we want, but we need to think ahead a bit. \
Recall from the description of the Quine-McCluskey process in the text that, \
in order to proceed with the second half of the method, we need to know which \
of the bit strings are prime implicants. That is, which bit strings are never \
used in a simplification.\
\>", "Text"],

Cell[TextData[{
 "We will track which bit strings are used as follows. Before the first loop, \
we create a set consisting of all of the bit strings in ",
 StyleBox["groups", "Input"],
 ". We can do this by applying ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " to ",
 StyleBox["groups", "Input"],
 ", removing the sublist structure. To obtain the elements of the sublists of \
groups rather than just all the 0s and 1s, we need to use the option second \
argument of ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " to specify that we wish to flatten only the first level."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Flatten", "[", 
  RowBox[{"groups", ",", "1"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Then each time ",
 StyleBox["mergeBitstrings", "Input"],
 " is successful, we remove the pair of bit strings from this set, using the ",
 StyleBox[ButtonBox["Complement",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Complement"], "FunctionLink"],
 " set operator. For example, to remove ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], TraditionalForm]]],
 ", we would execute the following expression."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Complement", "[", 
  RowBox[{
   RowBox[{"Flatten", "[", 
    RowBox[{"groups", ",", "1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}]}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
The function will return the list consisting of the next level of bit strings \
and the prime implicants from this stage. Here is our next attempt at the \
function.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nextBitList1", "[", "lastgroups_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"nextL", "=", 
       RowBox[{"{", "}"}]}], ",", "primeImps", ",", "n", ",", "A", ",", "B", 
      ",", "a", ",", "b", ",", "m"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"primeImps", "=", 
      RowBox[{"Flatten", "[", 
       RowBox[{"lastgroups", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"n", "=", "1"}], ",", 
       RowBox[{"n", "\[LessEqual]", 
        RowBox[{
         RowBox[{"Length", "[", "lastgroups", "]"}], "-", "1"}]}], ",", 
       RowBox[{"n", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"A", "=", 
         RowBox[{"lastgroups", "[", 
          RowBox[{"[", "n", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"B", "=", 
         RowBox[{"lastgroups", "[", 
          RowBox[{"[", 
           RowBox[{"n", "+", "1"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"m", "=", 
            RowBox[{"mergeBitstrings", "[", 
             RowBox[{"a", ",", "b"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"m", "=!=", "False"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"nextL", "=", 
               RowBox[{"Union", "[", 
                RowBox[{"nextL", ",", 
                 RowBox[{"{", "m", "}"}]}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"primeImps", "=", 
               RowBox[{"Complement", "[", 
                RowBox[{"primeImps", ",", 
                 RowBox[{"{", 
                  RowBox[{"a", ",", "b"}], "}"}]}], "]"}]}]}]}], 
            "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"a", ",", "A"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"b", ",", "B"}], "}"}]}], "\[IndentingNewLine]", "]"}]}]}],
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"nextL", ",", "primeImps"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell["\<\
This still isn't sufficient, however, because we also need to update the \
coverage dictionary as we create new bit strings. Recall that \
\[OpenCurlyDoubleQuote]coverage dictionary\[CloseCurlyDoubleQuote] is the \
name we gave to the table that records, for each bit string, which of the \
original minterms are covered by that bit string. The coverage dictionary was \
initialized with the bit strings formed from the minterms.\
\>", "Text"],

Cell[TextData[{
 "We can inspect the values stored with the ",
 StyleBox[ButtonBox["Definition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Definition"], "FunctionLink"],
 " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Definition", "[", "coverageDict", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "10", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "9", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "8", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "7", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "6", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "5", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "4", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "3", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "2", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "1", "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`coverageDict],
  Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
Within our function, we need to update the coverage dictionary. To do this, \
we will pass it as an argument to the function. The function will be able to \
update the indexed variable as if it were global without it needing to be \
held. Consider the following, for example.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"indexedExample", "[", "\"\<a\>\"", "]"}], "=", "1"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"indexedExample", "[", "\"\<b\>\"", "]"}], "=", "2"}], 
  ";"}]}], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Definition", "[", "indexedExample", "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"indexedExample", "[", "\<\"a\"\>", "]"}], "=", "1"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"indexedExample", "[", "\<\"b\"\>", "]"}], "=", "2"}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`indexedExample],
  Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"functionToChangeIndexed", "[", 
   RowBox[{"indexedVar_Symbol", ",", "i_", ",", "v_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"indexedVar", "[", "i", "]"}], "=", "v"}], ";"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The intent of this function is to add the index ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " with value ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]]],
 " to the indexed variable passed as the first argument. If the first \
argument were a list, for example, this function would not have the desired \
effect, as the list would be passed to the function, modified within the ",
 StyleBox[ButtonBox["Module",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Module"], "FunctionLink"],
 ", but remain unchanged outside the function. For an indexed symbol, \
however, when we call this function with first argument ",
 StyleBox["indexedExample", "Input"],
 ", the argument variable ",
 StyleBox["indexedVar", "Input"],
 " is assigned to the symbol ",
 StyleBox["indexedExample", "Input"],
 ". It is like assigning ",
 StyleBox["indexedVar", "Input"],
 " to the name of the indexed variable. Inside the function, the variable ",
 StyleBox["indexedVar", "Input"],
 " is resolved to the symbol ",
 StyleBox["indexedExample", "Input"],
 ", making the assignment within the function a \[OpenCurlyDoubleQuote]global\
\[CloseCurlyDoubleQuote] assignment."
}], "Text"],

Cell[BoxData[
 RowBox[{"functionToChangeIndexed", "[", 
  RowBox[{"indexedExample", ",", "\"\<c\>\"", ",", "3"}], "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Definition", "[", "indexedExample", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"indexedExample", "[", "\<\"a\"\>", "]"}], "=", "1"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"indexedExample", "[", "\<\"b\"\>", "]"}], "=", "2"}]},
       {" "},
       {
        RowBox[{
         RowBox[{"indexedExample", "[", "\<\"c\"\>", "]"}], "=", "3"}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`indexedExample],
  Editable->False]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We update the dictionary within the ",
 StyleBox["m=!=False", "Input"],
 " ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement. When we form a new bit string ",
 StyleBox["m", "Input"],
 ", we obtain the set of minterms it covers by taking the union of the sets \
of minterms covered by the two bit strings that were merged. That is, ",
 StyleBox["coverDict[m]", "Input"],
 " is the union of ",
 StyleBox["coverDict[a]", "Input"],
 " and ",
 StyleBox["coverDict[b]", "Input"],
 ". ",
 "Note that bit strings formed beyond the first step are typically generated \
multiple times. However, each time they are generated they always cover the \
same set of original minterms."
}], "Text"],

Cell[TextData[{
 "Here is the final version of ",
 StyleBox["nextBitList", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nextBitList", "[", 
   RowBox[{"lastgroups_List", ",", "coverDict_Symbol"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"nextL", "=", 
       RowBox[{"{", "}"}]}], ",", "primeImps", ",", "n", ",", "A", ",", "B", 
      ",", "a", ",", "b", ",", "m"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"primeImps", "=", 
      RowBox[{"Flatten", "[", 
       RowBox[{"lastgroups", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"n", "=", "1"}], ",", 
       RowBox[{"n", "\[LessEqual]", 
        RowBox[{
         RowBox[{"Length", "[", "lastgroups", "]"}], "-", "1"}]}], ",", 
       RowBox[{"n", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"A", "=", 
         RowBox[{"lastgroups", "[", 
          RowBox[{"[", "n", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"B", "=", 
         RowBox[{"lastgroups", "[", 
          RowBox[{"[", 
           RowBox[{"n", "+", "1"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"m", "=", 
            RowBox[{"mergeBitstrings", "[", 
             RowBox[{"a", ",", "b"}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"m", "=!=", "False"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"nextL", "=", 
               RowBox[{"Union", "[", 
                RowBox[{"nextL", ",", 
                 RowBox[{"{", "m", "}"}]}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"primeImps", "=", 
               RowBox[{"Complement", "[", 
                RowBox[{"primeImps", ",", 
                 RowBox[{"{", 
                  RowBox[{"a", ",", "b"}], "}"}]}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"coverDict", "[", "m", "]"}], "=", 
               RowBox[{"Union", "[", 
                RowBox[{
                 RowBox[{"coverDict", "[", "a", "]"}], ",", 
                 RowBox[{"coverDict", "[", "b", "]"}]}], "]"}]}], ";"}]}], 
            "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"a", ",", "A"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"b", ",", "B"}], "}"}]}], "\[IndentingNewLine]", "]"}]}]}],
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"nextL", ",", "primeImps"}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We apply it to ",
 StyleBox["groups", "Input"],
 " to obtain ",
 StyleBox["Fbits1", "Input"],
 " and ",
 StyleBox["primes1", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"Fbits1", ",", "primes1"}], "}"}], "=", 
  RowBox[{"nextBitList", "[", 
   RowBox[{"groups", ",", "coverageDict"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "\<\"-\"\>", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "\<\"-\"\>", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "1", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "1", ",", "\<\"-\"\>"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "\<\"-\"\>", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\<\"-\"\>", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\<\"-\"\>", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\<\"-\"\>", ",", "1", ",", "0", ",", "0"}], "}"}]}], "}"}], 
   ",", 
   RowBox[{"{", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["We see that there are", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "Fbits1", "]"}]], "Input"],

Cell[BoxData["13"], "Output"]
}, Open  ]],

Cell["\<\
bit strings in the second level, but no prime implicants coming from the \
first pass.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "primes1", "]"}]], "Input"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Also, almost as a side effect, the function has updated ",
 StyleBox["coverageDict", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Definition", "[", "coverageDict", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "10", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "9", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "\<\"-\"\>"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"9", ",", "10"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "8", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "\<\"-\"\>", ",", "1"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"8", ",", "9"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "7", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "6", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "0", ",", "\<\"-\"\>"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"6", ",", "7"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "5", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "\<\"-\"\>", ",", "1"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"5", ",", "6"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"7", ",", "10"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "1"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"6", ",", "9"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "\<\"-\"\>", ",", "1", ",", "1"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"5", ",", "8"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "4", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "3", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "2", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "1", ",", "\<\"-\"\>"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "3"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "\<\"-\"\>", ",", "0"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "4"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "1", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "4"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"\<\"-\"\>", ",", "0", ",", "0", ",", "0"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"4", ",", "10"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"\<\"-\"\>", ",", "0", ",", "1", ",", "1"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "8"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"\<\"-\"\>", ",", "1", ",", "0", ",", "0"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "7"}], "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`coverageDict],
  Editable->False]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Repeating", "Subsubsection"],

Cell["Step 4 is to repeat steps 2 and 3.", "Text"],

Cell[TextData[{
 "The ",
 StyleBox["Fbits1", "Input"],
 " list takes the place of ",
 StyleBox["Fbits", "Input"],
 ". We apply ",
 StyleBox["sortGroups", "Input"],
 " to produce ",
 StyleBox["groups1", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"groups1", "=", 
  RowBox[{"sortGroups", "[", "Fbits1", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\<\"-\"\>", ",", "0", ",", "0", ",", "0"}], "}"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "\<\"-\"\>", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "\<\"-\"\>", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\<\"-\"\>", ",", "1", ",", "0", ",", "0"}], "}"}]}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "\<\"-\"\>", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "1", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "1", ",", "\<\"-\"\>"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\<\"-\"\>", ",", "0", ",", "1", ",", "1"}], "}"}]}], "}"}], 
   ",", 
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Then applying ",
 StyleBox["nextBitList", "Input"],
 " to ",
 StyleBox["groups1", "Input"],
 " produces ",
 StyleBox["Fbits2", "Input"],
 " and ",
 StyleBox["primes2", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"Fbits2", ",", "primes2"}], "}"}], "=", 
  RowBox[{"nextBitList", "[", 
   RowBox[{"groups1", ",", "coverageDict"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "\<\"-\"\>", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\<\"-\"\>", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}]}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "1", ",", "\<\"-\"\>"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "\<\"-\"\>", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\<\"-\"\>", ",", "0", ",", "1", ",", "1"}], "}"}]}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["\<\
We see that we have found three prime implicants. The coverage dictionary was \
further expanded to include the new bit strings.\
\>", "Text"],

Cell[TextData[{
 "Do the same thing again with ",
 StyleBox["Fbits2", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"groups2", "=", 
  RowBox[{"sortGroups", "[", "Fbits2", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\<\"-\"\>", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}]}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{"0", ",", "\<\"-\"\>", ",", "\<\"-\"\>", ",", "1"}], "}"}], 
    "}"}], ",", 
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"Fbits3", ",", "primes3"}], "}"}], "=", 
  RowBox[{"nextBitList", "[", 
   RowBox[{"groups2", ",", "coverageDict"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\<\"-\"\>", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "\<\"-\"\>", ",", "\<\"-\"\>", ",", "1"}], "}"}]}], 
    "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This time, ",
 StyleBox["Fbits3", "Input"],
 " was empty, which indicates that no more merging is possible and all prime \
implicants have been found."
}], "Text"],

Cell["\<\
This part of the process concludes by forming the list of all the prime \
implicants.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"allprimeImps", "=", 
  RowBox[{"Union", "[", 
   RowBox[{"primes1", ",", "primes2", ",", "primes3"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "\<\"-\"\>", ",", "\<\"-\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "\<\"-\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "\<\"-\"\>", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"-\"\>", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"-\"\>", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Forming the Coverage Table", "Subsubsection"],

Cell["\<\
Now that we have identified all of the prime implicants, we will use the \
coverage dictionary to create the coverage table.\
\>", "Text"],

Cell["Take a look at the final state of the coverage dictionary.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Definition", "[", "coverageDict", "]"}]], "Input"],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "10", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "9", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "\<\"-\"\>"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"9", ",", "10"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "8", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "\<\"-\"\>", ",", "1"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"8", ",", "9"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "7", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "6", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "0", ",", "\<\"-\"\>"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"6", ",", "7"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "5", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "1", ",", "\<\"-\"\>", ",", "1"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"5", ",", "6"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"7", ",", "10"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "1"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"6", ",", "9"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "\<\"-\"\>"}], "}"}],
           "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"6", ",", "7", ",", "9", ",", "10"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "\<\"-\"\>", ",", "1", ",", "1"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"5", ",", "8"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "\<\"-\"\>", ",", "\<\"-\"\>", ",", "1"}], "}"}],
           "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"5", ",", "6", ",", "8", ",", "9"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "4", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "3", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], "]"}], "=", 
         RowBox[{"{", "2", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "1", ",", "\<\"-\"\>"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "3"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "0", ",", "\<\"-\"\>", ",", "0"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"3", ",", "4"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}], "]"}], "=", 
         RowBox[{"{", "1", "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"1", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "4"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"\<\"-\"\>", ",", "0", ",", "0", ",", "0"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"4", ",", "10"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"\<\"-\"\>", ",", "0", ",", "1", ",", "1"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"2", ",", "8"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"\<\"-\"\>", ",", "1", ",", "0", ",", "0"}], "}"}], "]"}], 
         "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "7"}], "}"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"coverageDict", "[", 
          RowBox[{"{", 
           RowBox[{"\<\"-\"\>", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}],
           "]"}], "=", 
         RowBox[{"{", 
          RowBox[{"1", ",", "4", ",", "7", ",", "10"}], "}"}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`coverageDict],
  Editable->False]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Each bit string, and in particular each prime implicant, is an index in \
this table. The corresponding entry is the set of integers which are the \
indices to the original minterms in ",
 StyleBox["Fbits", "Input"],
 ". Thus, to determine which of the original minterms are covered by each \
prime implicant, we just look it up in the table."
}], "Text"],

Cell["\<\
We will model the coverage table as a matrix. Each row corresponds to a prime \
implicant, so there will be\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "allprimeImps", "]"}]], "Input"],

Cell[BoxData["6"], "Output"]
}, Open  ]],

Cell["rows. And each column corresponds to a minterm, so there are", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "Fbits", "]"}]], "Input"],

Cell[BoxData["10"], "Output"]
}, Open  ]],

Cell[TextData[{
 "columns. The entries in the matrix will be 0s and 1s with 1 in position ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"i", ",", "j"}], ")"}], TraditionalForm]]],
 " indicating that the prime implicant at position ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " in ",
 StyleBox["allprimeImps", "Input"],
 " covers the minterm at position ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " in ",
 StyleBox["Fbits", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "We use ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " with first argument 0 and second argument a list consisting of the number \
of rows and columns to form a matrix of all 0s of the appropriate size."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ConstantArray", "[", 
   RowBox[{"0", ",", 
    RowBox[{"{", 
     RowBox[{"6", ",", "10"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "0", "0", "0", "0", "0", "0", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
To enter 1s in the appropriate positions, we loop over the rows, considering \
each prime implicant in turn. For each prime implicant, we look up its entry \
in the coverage dictionary to obtain the set of minterms it covers. For each \
of those minterms, we place a 1 in the matrix.\
\>", "Text"],

Cell["The following function initializes the coverage table.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"initCoverageMatrix", "[", 
   RowBox[{"minterms_List", ",", "primeImps_List", ",", "coverDict_Symbol"}], 
   "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"M", ",", "i", ",", "C", ",", "j"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"M", "=", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"0", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Length", "[", "primeImps", "]"}], ",", 
          RowBox[{"Length", "[", "minterms", "]"}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "primeImps", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"C", "=", 
         RowBox[{"coverDict", "[", 
          RowBox[{"primeImps", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"M", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "j"}], "]"}], "]"}], "=", "1"}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", "C"}], "}"}]}], "]"}]}]}], "\[IndentingNewLine]",
       "]"}], ";", "\[IndentingNewLine]", "M"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Applied to our example, this produces the following coverage table.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"coverageTable", "=", 
   RowBox[{"initCoverageMatrix", "[", 
    RowBox[{"Fbits", ",", "allprimeImps", ",", "coverageDict"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MatrixForm", "[", "coverageTable", "]"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0", "0", "1", "1", "0", "1", "1"},
     {"0", "0", "0", "0", "1", "1", "0", "1", "1", "0"},
     {"0", "1", "1", "0", "0", "0", "0", "0", "0", "0"},
     {"0", "0", "1", "1", "0", "0", "0", "0", "0", "0"},
     {"0", "1", "0", "0", "0", "0", "0", "1", "0", "0"},
     {"1", "0", "0", "1", "0", "0", "1", "0", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Manipulating the Matrix", "Subsubsection"],

Cell["\<\
Once the coverage table is set up, we move to steps 6 and 7, determining \
which prime implicants to include in the minimal expression. In step 6, we \
identify the essential prime implicants and in step 7 we identify which of \
the non-essential prime implicants we will include. We will see how to \
identify the prime implicants to use in a moment.\
\>", "Text"],

Cell["\<\
To aid in performing both steps 6 and 7, we will be manipulating the coverage \
table. Once we have decided to include a particular prime implicant in the \
minimal expression, we take three actions.\
\>", "Text"],

Cell[TextData[{
 "First, record the decision by adding the prime implicant to a new list, say \
",
 StyleBox["minBits", "Input"],
 ", the list of bit strings to be included in the minimal expression."
}], "Text"],

Cell["\<\
Second, delete that prime implicant's row from the coverage table and delete \
the columns corresponding to the minterms it covered. We know the prime \
implicant will be in the expression and thus the minterms it covers are \
satisfied. Hence, there is no longer any need to keep track of that \
information.\
\>", "Text"],

Cell[TextData[{
 "Third, delete the prime implicant and the minterms it covers from the lists \
storing them (",
 StyleBox["allprimeImps", "Input"],
 " and ",
 StyleBox["Fbits", "Input"],
 "). This is to ensure that the indices of ",
 StyleBox["allprimeImps", "Input"],
 " and ",
 StyleBox["Fbits", "Input"],
 " continue to match the row and column numbers of the matrix."
}], "Text"],

Cell["\<\
We will now write a function that implements these actions. Its input will be \
the index to the prime implicant that has been chosen. It will also accept \
the names of the coverage matrix, the list of minterms, and the list of prime \
implicants. All of these will be modified in the function (refer to the \
subsection on \[OpenCurlyDoubleQuote]Modifying arguments\
\[CloseCurlyDoubleQuote] above). The function will return the bit string of \
the prime implicant that was chosen.\
\>", "Text"],

Cell[TextData[{
 "Our function will be called ",
 StyleBox["updateCT", "Input"],
 ", for \[OpenCurlyDoubleQuote]update coverage table.\[CloseCurlyDoubleQuote] \
The ",
 StyleBox["minBits", "Input"],
 " list, the list of chosen prime implicants, will be updated via the return \
value. This accomplishes the first task for this function."
}], "Text"],

Cell["\<\
Second, we must delete the row corresponding to the chosen prime implicant \
and the columns corresponding to the minterms covered by that implicant. \
Suppose, in our example, that we have decided to include the fourth prime \
implicant in the final result. This is:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"allprimeImps", "[", 
  RowBox[{"[", "4", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "\<\"-\"\>", ",", "0"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "From ",
 StyleBox["coverageTable", "Input"],
 ", we need to remove row 4 (since this corresponds to the prime implicant). \
We also need to remove the columns corresponding to the minterms covered by \
this prime implicant. To determine which columns are to be removed, we find \
the locations of the 1s in the row of the matrix."
}], "Text"],

Cell[TextData[{
 "To determine the locations of the 1s, we'll loop over the columns checking \
each position in row 4 to see if it is 1 or not. We use the ",
 StyleBox[ButtonBox["Dimensions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Dimensions"], "FunctionLink"],
 " function to determine the number of columns. For a matrix, ",
 StyleBox[ButtonBox["Dimensions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Dimensions"], "FunctionLink"],
 " returns a list whose elements are the number of rows and columns."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"covered", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"i", "=", "1"}], ",", 
    RowBox[{"i", "\[LessEqual]", 
     RowBox[{
      RowBox[{"Dimensions", "[", "coverageTable", "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}]}], ",", 
    RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"coverageTable", "[", 
        RowBox[{"[", 
         RowBox[{"4", ",", "i"}], "]"}], "]"}], "\[Equal]", "1"}], ",", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"covered", ",", "i"}], "]"}]}], "]"}]}], "\[IndentingNewLine]",
    "]"}], ";"}], "\[IndentingNewLine]", "covered"}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "4"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We now know that we need to remove row 4 and columns 3 and 4. To do this, \
we will use a complicated selection. Recall that ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") can be used with a list within the double brackets to select specific \
elements. For example, "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
   "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
    "\"\<e\>\"", ",", "\"\<f\>\"", ",", "\"\<g\>\""}], "}"}], "[", 
  RowBox[{"[", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "4", ",", "5", ",", "6"}], "}"}], "]"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"d\"\>", ",", "\<\"e\"\>", 
   ",", "\<\"f\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
The same is true for matrices. Given a matrix, a list of rows, and a list of \
columns, you can obtain the submatrix containing only the specified rows and \
columns as illustrated below.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"exampleMatrix", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     SubscriptBox["a", 
      RowBox[{"{", 
       RowBox[{"i", ",", "j"}], "}"}]], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "7"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "6"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"exampleMatrix", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"1", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"1", ",", "4"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"1", ",", "5"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"1", ",", "6"}], "}"}]]},
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"2", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"2", ",", "4"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"2", ",", "5"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"2", ",", "6"}], "}"}]]},
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"3", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"3", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"3", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"3", ",", "5"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"3", ",", "6"}], "}"}]]},
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"4", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"4", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"4", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"4", ",", "4"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"4", ",", "5"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"4", ",", "6"}], "}"}]]},
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"5", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"5", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"5", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"5", ",", "4"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"5", ",", "5"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"5", ",", "6"}], "}"}]]},
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"6", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"6", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"6", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"6", ",", "4"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"6", ",", "5"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"6", ",", "6"}], "}"}]]},
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"7", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"7", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"7", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"7", ",", "4"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"7", ",", "5"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"7", ",", "6"}], "}"}]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"exampleMatrix", "[", 
   RowBox[{"[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "4", ",", "6", ",", "7"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3", ",", "6"}], "}"}]}], "]"}], "]"}], "//",
   "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"1", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"1", ",", "6"}], "}"}]]},
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"2", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"2", ",", "6"}], "}"}]]},
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"4", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"4", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"4", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"4", ",", "6"}], "}"}]]},
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"6", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"6", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"6", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"6", ",", "6"}], "}"}]]},
     {
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"7", ",", "1"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"7", ",", "2"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"7", ",", "3"}], "}"}]], 
      SubscriptBox["a", 
       RowBox[{"{", 
        RowBox[{"7", ",", "6"}], "}"}]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
This is the matrix that results from deleting the 3rd and 5th rows, and the \
4th and 5th columns.\
\>", "Text"],

Cell[TextData[{
 "So for the coverage table, we can obtain the modified matrix by forming the \
lists of row and column numbers that should remain and then apply ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ")."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"remainingRows", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3", ",", "5", ",", "6"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "5", ",", "6"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"remainingColumns", "=", 
  RowBox[{"Complement", "[", 
   RowBox[{
    RowBox[{"Range", "[", 
     RowBox[{"Length", "[", "Fbits", "]"}], "]"}], ",", "covered"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "2", ",", "5", ",", "6", ",", "7", ",", "8", ",", "9", ",", 
   "10"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"coverageTable", "[", 
   RowBox[{"[", 
    RowBox[{"remainingRows", ",", "remainingColumns"}], "]"}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "1", "1", "0", "1", "1"},
     {"0", "0", "1", "1", "0", "1", "1", "0"},
     {"0", "1", "0", "0", "0", "0", "0", "0"},
     {"0", "1", "0", "0", "0", "1", "0", "0"},
     {"1", "0", "0", "0", "1", "0", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The reader is encouraged to compare this to the original matrix. Note that \
in this example, we have not actually modified ",
 StyleBox["coverageTable", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "The last tasks are to remove the selected prime implicant from the list of \
prime implicants, and remove the covered minterms from its list. Note that ",
 StyleBox["remainingRows", "Input"],
 " and ",
 StyleBox["remainingColumns", "Input"],
 " are indices into the ",
 StyleBox["allprimeImps", "Input"],
 " list and the list of minterms. Applying ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") produces the desired outcome."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"allprimeImps", "[", 
  RowBox[{"[", "remainingRows", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "\<\"-\"\>", ",", "0", ",", "\<\"-\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "\<\"-\"\>", ",", "\<\"-\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "\<\"-\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"-\"\>", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"-\"\>", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Fbits", "[", 
  RowBox[{"[", "remainingColumns", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["Here is the function.", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{"updateCT", ",", 
    RowBox[{"{", "HoldRest", "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"updateCT", "[", 
   RowBox[{
   "newPI_", ",", "coverTable_", ",", "minterms_", ",", "primeImps_"}], "]"}],
   ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "newPIbitstring", ",", "numRows", ",", "numCols", ",", "covered", ",", 
      "i", ",", "remainingRows", ",", "remainingCols"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"newPIbitstring", "=", 
      RowBox[{"primeImps", "[", 
       RowBox[{"[", "newPI", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"numRows", ",", "numCols"}], "}"}], "=", 
      RowBox[{"Dimensions", "[", "coverTable", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"covered", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", "numCols"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"coverTable", "[", 
           RowBox[{"[", 
            RowBox[{"newPI", ",", "i"}], "]"}], "]"}], "\[Equal]", "1"}], ",", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"covered", ",", "i"}], "]"}]}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"remainingRows", "=", 
      RowBox[{"Delete", "[", 
       RowBox[{
        RowBox[{"Range", "[", "numRows", "]"}], ",", "newPI"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"remainingCols", "=", 
      RowBox[{"Complement", "[", 
       RowBox[{
        RowBox[{"Range", "[", "numCols", "]"}], ",", "covered"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"coverTable", "=", 
      RowBox[{"coverTable", "[", 
       RowBox[{"[", 
        RowBox[{"remainingRows", ",", "remainingCols"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"primeImps", "=", 
      RowBox[{"primeImps", "[", 
       RowBox[{"[", "remainingRows", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"minterms", "=", 
      RowBox[{"minterms", "[", 
       RowBox[{"[", "remainingCols", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", "newPIbitstring"}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Finding Essential Prime Implicants", "Subsubsection"],

Cell["\<\
Next we write a function to identify the essential prime implicants. Recall \
that a prime implicant is essential when it is the only prime implicant to \
cover some minterm. In terms of the coverage table, this is equivalent to the \
existence of a column with only one 1.\
\>", "Text"],

Cell["\<\
We will locate the essential prime implicants as follows. First, we \
initialize the set of essential prime implicants to the empty list.\
\>", "Text"],

Cell[TextData[{
 "We proceed in a manner similar to the ",
 StyleBox["mergeBitstrings", "Input"],
 " function. We use a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop to step through the columns of the coverage table. Within this loop, \
we initialize a symbol, ",
 StyleBox["rowhas1", "Input"],
 ", to 0."
}], "Text"],

Cell[TextData[{
 "We then enter a second loop to step through the entries in the columns. \
When a 1 entry has been found, we check ",
 StyleBox["rowhas1.", "Input"],
 " If that symbol is 0, then it is assigned to the current row number. If it \
is not 0, then we have found a second 1 in the column and we assign ",
 StyleBox["rowhas1", "Input"],
 " to -1 and use ",
 StyleBox[ButtonBox["Break",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Break"], "FunctionLink"],
 " to terminate the inner loop. After the inner loop, we test ",
 StyleBox["rowhas1", "Input"],
 ". If it is positive, then we know that only one 1 was located in that \
column, and hence the row the solitary 1 was found in corresponds to an \
essential prime implicant. In this case, we add the row number (",
 StyleBox["rowhas1", "Input"],
 ") to essentials."
}], "Text"],

Cell["\<\
The following function implements this algorithm and returns the list of \
essential prime implicants.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findEssentials", "[", "coverTable_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"essentials", "=", 
       RowBox[{"{", "}"}]}], ",", "r", ",", "c", ",", "i", ",", "j", ",", 
      "rowhas1"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"r", ",", "c"}], "}"}], "=", 
      RowBox[{"Dimensions", "[", "coverTable", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", "c"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"rowhas1", "=", "0"}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"j", "=", "1"}], ",", 
          RowBox[{"j", "\[LessEqual]", "r"}], ",", 
          RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"coverTable", "[", 
              RowBox[{"[", 
               RowBox[{"j", ",", "i"}], "]"}], "]"}], "\[Equal]", "1"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"rowhas1", "\[Equal]", "0"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"rowhas1", "=", "j"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"rowhas1", "=", 
                RowBox[{"-", "1"}]}], ";", 
               RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", 
             "]"}]}], "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
         "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"rowhas1", ">", "0"}], ",", "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"essentials", ",", "rowhas1"}], "]"}]}], 
         "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Sort", "[", 
      RowBox[{
       RowBox[{"Union", "[", "essentials", "]"}], ",", "Greater"}], "]"}]}]}],
    "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " applied with the comparison function ",
 StyleBox[ButtonBox["Greater",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Greater"], "FunctionLink"],
 " ensures that the output will be in decreasing order. This will allow us to \
update the coverage table and other lists without affecting the index of \
smaller-indexed essential prime implicants."
}], "Text"],

Cell["\<\
We use this to determine the essential prime implicants of our example.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"essentialPIs", "=", 
  RowBox[{"findEssentials", "[", "coverageTable", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6", ",", "2"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now that we have the essential prime implicants, we can initialize ",
 StyleBox["minBits", "Input"],
 " and apply ",
 StyleBox["updateCT", "Input"],
 " to the essential prime implicants."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"minBits", "=", 
  RowBox[{"{", "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Do", "[", 
  RowBox[{
   RowBox[{"AppendTo", "[", 
    RowBox[{"minBits", ",", "\[IndentingNewLine]", 
     RowBox[{"updateCT", "[", 
      RowBox[{"i", ",", "coverageTable", ",", "Fbits", ",", "allprimeImps"}], 
      "]"}]}], "]"}], "\[IndentingNewLine]", ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "essentialPIs"}], "}"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData["minBits"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"-\"\>", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "\<\"-\"\>", ",", "\<\"-\"\>", ",", "1"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"coverageTable", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0"},
     {"1", "1"},
     {"0", "1"},
     {"1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Completing the Coverage", "Subsubsection"],

Cell["\<\
Provided that the essential prime implicants did not completely cover the \
original minterms, we must complete the coverage with non-essential prime \
implicants. First, we ensure that the coverage is not complete by checking \
the column dimension.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Dimensions", "[", "coverageTable", "]"}], "[", 
   RowBox[{"[", "2", "]"}], "]"}], ">", "0"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell["\<\
We will use a heuristic approach to find a minimal set of prime implicants \
rather than using an exhaustive search to determine the minimum. The \
heuristic we use will be to choose the prime implicant with the most \
extensive coverage of the remaining minterms.\
\>", "Text"],

Cell[TextData[{
 "To find such a prime implicant, we will do the following. First, initialize \
",
 StyleBox["maxCoverage", "Input"],
 " and ",
 StyleBox["bestImp", "Input"],
 " both to 0. Then loop over each row of the (modified) coverage table. For \
each row, we will compute the sum of the entries. If this sum is greater than \
",
 StyleBox["maxCoverage", "Input"],
 ", then set ",
 StyleBox["maxCoverage", "Input"],
 " to the sum and set ",
 StyleBox["bestImp", "Input"],
 " to the row number. Once the loop is complete, ",
 StyleBox["bestImp", "Input"],
 " will be the index to a row with maximum coverage and will be the next \
prime implicant added to the ",
 StyleBox["minBits", "Input"],
 " list."
}], "Text"],

Cell["Here is the function that implements this strategy.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findBestImp", "[", "coverTable_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"maxCoverage", "=", "0"}], ",", 
      RowBox[{"bestImp", "=", "0"}], ",", "i", ",", "j", ",", "sum"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{
         RowBox[{"Dimensions", "[", "coverTable", "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"sum", "=", 
         RowBox[{"Plus", "@@", 
          RowBox[{"coverTable", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"sum", ">", "maxCoverage"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"maxCoverage", "=", "sum"}], ";", "\[IndentingNewLine]", 
           RowBox[{"bestImp", "=", "i"}]}]}], "\[IndentingNewLine]", 
         "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     "bestImp"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "As long as the coverage table is not empty, we apply this function to it to \
obtain the next implicant. We add the implicant to the list ",
 StyleBox["minBits", "Input"],
 " representing the minimal expression and update the coverage table using ",
 StyleBox["updateCT", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{"While", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Dimensions", "[", "coverageTable", "]"}], "[", 
     RowBox[{"[", "2", "]"}], "]"}], ">", "0"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"nextPI", "=", 
     RowBox[{"findBestImp", "[", "coverageTable", "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"AppendTo", "[", 
     RowBox[{"minBits", ",", "\[IndentingNewLine]", 
      RowBox[{"updateCT", "[", 
       RowBox[{
       "nextPI", ",", "coverageTable", ",", "Fbits", ",", "allprimeImps"}], 
       "]"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData["minBits"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"-\"\>", ",", "\<\"-\"\>", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "\<\"-\"\>", ",", "\<\"-\"\>", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "\<\"-\"\>"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "All that's left is to translate ",
 StyleBox["minBits", "Input"],
 " back into a logical expression. This can be done using ",
 StyleBox["bitListToDNF", "Input"],
 " created earlier."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"bitListToDNF", "[", 
  RowBox[{"minBits", ",", 
   RowBox[{"{", 
    RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "y"}], "&&", 
    RowBox[{"!", "z"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", "z"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"w", "&&", 
    RowBox[{"!", "x"}], "&&", "y"}], ")"}]}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Putting It All Together", "Subsubsection"],

Cell["\<\
Finally, we assemble the pieces into a single function, which accepts a \
logical expression in disjunctive normal form and a list of its variables. It \
returns a minimal equivalent expression.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"quineMcCluskey", "[", 
   RowBox[{"F_", ",", "variables_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "fBits", ",", "fBitsL", ",", "coverageDict", ",", "groupsL", ",", 
      "primesL", ",", "newFbits", ",", "newprimes", ",", "i", ",", 
      "allprimeImps", ",", "j", ",", "coverageTable", ",", "essentialPIs", 
      ",", "minBits", ",", "nextPI"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"fBits", "=", 
      RowBox[{"dnfToBitList", "[", 
       RowBox[{"F", ",", "variables"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"initCoverDict", "[", 
      RowBox[{"coverageDict", ",", "fBits"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"i", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"fBitsL", "=", 
      RowBox[{"{", "fBits", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"groupsL", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"primesL", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"fBitsL", "[", 
         RowBox[{"[", 
          RowBox[{"-", "1"}], "]"}], "]"}], "=!=", 
        RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"AppendTo", "[", 
         RowBox[{"groupsL", ",", 
          RowBox[{"sortGroups", "[", 
           RowBox[{"fBitsL", "[", 
            RowBox[{"[", 
             RowBox[{"-", "1"}], "]"}], "]"}], "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"newFbits", ",", "newprimes"}], "}"}], "=", 
         RowBox[{"nextBitList", "[", 
          RowBox[{
           RowBox[{"groupsL", "[", 
            RowBox[{"[", 
             RowBox[{"-", "1"}], "]"}], "]"}], ",", "coverageDict"}], "]"}]}],
         ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"fBitsL", ",", "newFbits"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"primesL", ",", "newprimes"}], "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"allprimeImps", "=", 
      RowBox[{"Union", "@@", "primesL"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"coverageTable", "=", 
      RowBox[{"initCoverageMatrix", "[", 
       RowBox[{"fBits", ",", "allprimeImps", ",", "coverageDict"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"essentialPIs", "=", 
      RowBox[{"findEssentials", "[", "coverageTable", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"minBits", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{"minBits", ",", "\[IndentingNewLine]", 
         RowBox[{"updateCT", "[", 
          RowBox[{
          "i", ",", "coverageTable", ",", "fBits", ",", "allprimeImps"}], 
          "]"}]}], "]"}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "essentialPIs"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"MatrixQ", "[", "coverageTable", "]"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Dimensions", "[", "coverageTable", "]"}], "[", 
          RowBox[{"[", "2", "]"}], "]"}], ">", "0"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"nextPI", "=", 
         RowBox[{"findBestImp", "[", "coverageTable", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"minBits", ",", 
          RowBox[{"updateCT", "[", 
           RowBox[{
           "nextPI", ",", "coverageTable", ",", "fBits", ",", 
            "allprimeImps"}], "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"bitListToDNF", "[", 
      RowBox[{"minBits", ",", "variables"}], "]"}]}]}], "\[IndentingNewLine]",
    "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Define ",
 StyleBox["ex10", "Input"],
 " to be the expression in Example 10 from Section 12.4 of the text."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ex10", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"w", "&&", "x", "&&", "y", "&&", 
     RowBox[{"!", "z"}]}], ")"}], "||", 
   RowBox[{"(", 
    RowBox[{"w", "&&", 
     RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ")"}], "||", 
   RowBox[{"(", 
    RowBox[{"w", "&&", 
     RowBox[{"!", "x"}], "&&", "y", "&&", 
     RowBox[{"!", "z"}]}], ")"}], "||", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"!", "w"}], "&&", "x", "&&", "y", "&&", "z"}], ")"}], "||", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"!", "w"}], "&&", "x", "&&", 
     RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"!", "w"}], "&&", 
     RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ")"}], "||", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"!", "w"}], "&&", 
     RowBox[{"!", "x"}], "&&", 
     RowBox[{"!", "y"}], "&&", "z"}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"w", "&&", "x", "&&", "y", "&&", 
    RowBox[{"!", "z"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"w", "&&", 
    RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"w", "&&", 
    RowBox[{"!", "x"}], "&&", "y", "&&", 
    RowBox[{"!", "z"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", "x", "&&", "y", "&&", "z"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", "x", "&&", 
    RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", 
    RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", 
    RowBox[{"!", "x"}], "&&", 
    RowBox[{"!", "y"}], "&&", "z"}], ")"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"quineMcCluskey", "[", 
  RowBox[{"ex10", ",", 
   RowBox[{"{", 
    RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"w", "&&", "y", "&&", 
    RowBox[{"!", "z"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "w"}], "&&", "z"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"w", "&&", 
    RowBox[{"!", "x"}], "&&", "y"}], ")"}]}]], "Output"]
}, Open  ]],

Cell["\<\
Note that this is the first of the two answers given in the solution to \
Example 10.\
\>", "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Solutions to Computer Projects and Computations and Explorations\
\>", "Section",
 CellTags->"Solutions"],

Cell[CellGroupData[{

Cell["Computer Projects 2", "Subsection",
 CellTags->"SolutionsCP2"],

Cell["\<\
Construct a table listing the set of values of all 256 Boolean functions of \
degree three.\
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " The Boolean functions of degree three are in one-to-one correspondence \
with the subsets of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"{", 
     RowBox[{"true", ",", "false"}], "}"}], "3"], TraditionalForm]]],
 ". This is because each subset ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"{", 
     RowBox[{"true", ",", "false"}], "}"}], "3"], TraditionalForm]]],
 " can be identified with the unique Boolean function of degree three that \
returns true on the members of S and false on all other inputs."
}], "Text"],

Cell[TextData[{
 "The set ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"{", 
     RowBox[{"true", ",", "false"}], "}"}], "3"], TraditionalForm]]],
 " consists of 8 elements: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"true", ",", "true", ",", "true"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"true", ",", "true", ",", "false"}], ")"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"true", ",", "false", ",", "true"}], ")"}], TraditionalForm]]],
 ",...,",
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"false", ",", "false", ",", "false"}], ")"}], TraditionalForm]]],
 ". The power set can therefore be identified with bit strings of length 8, \
with a 1 indicating inclusion of the corresponding member of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"{", 
     RowBox[{"true", ",", "false"}], "}"}], "3"], TraditionalForm]]],
 ". For example, \[OpenCurlyDoubleQuote]10100000\[CloseCurlyDoubleQuote] \
would correspond to the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"true", ",", "true", ",", "true"}], ")"}], ",", 
     RowBox[{"(", 
      RowBox[{"true", ",", "false", ",", "true"}], ")"}]}], "}"}], 
   TraditionalForm]]],
 " which in turn corresponds to the Boolean function that returns true on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"true", ",", "true", ",", "true"}], ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"true", ",", "false", ",", "true"}], ")"}], TraditionalForm]]],
 " and false for all other input. The bit strings, in turn, can be identified \
with integers between 0 and 255, based on their binary representation."
}], "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " function takes advantage of this correspondence between integers and \
Boolean functions. Given two integers ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " as arguments, ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " produces the Boolean function, as a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 ", on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " variables corresponding to the binary representation of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanFunction", "[", 
  RowBox[{"132", ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 FrameBox[
  RowBox[{"BooleanFunction", "[", 
   RowBox[{"<", "3", ">"}], "]"}],
  BoxFrame->False,
  FrameMargins->{{False, False}, {False, False}}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The resulting ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " can be applied to truth values."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BooleanFunction", "[", 
   RowBox[{"132", ",", "3"}], "]"}], "[", 
  RowBox[{"True", ",", "False", ",", "True"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell["\<\
With a list of variables passed as a third argument, the output will be an \
expression for the Boolean function.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanFunction", "[", 
  RowBox[{"132", ",", "3", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"x", "&&", "y", "&&", "z"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "x"}], "&&", "y", "&&", 
    RowBox[{"!", "z"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Applying ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 ", described in the first section of this chapter, to the output of ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " produces the functions\[CloseCurlyQuote] truth table."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanTable", "[", 
  RowBox[{"BooleanFunction", "[", 
   RowBox[{"132", ",", "3"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "True", ",", "False", ",", "False", ",", "False", ",", "False", ",", "True",
    ",", "False", ",", "False"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We prefer a table that shows the input values along with the output. For \
this, we apply ",
 StyleBox[ButtonBox["BooleanTable",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanTable"], "FunctionLink"],
 " to a list consisting of variables as the second argument. Then in the \
first argument, we can use the variables."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanTable", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z", ",", 
     RowBox[{
      RowBox[{"BooleanFunction", "[", 
       RowBox[{"132", ",", "3"}], "]"}], "[", 
      RowBox[{"x", ",", "y", ",", "z"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"True", ",", "True", ",", "True", ",", "True"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"True", ",", "True", ",", "False", ",", "False"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"True", ",", "False", ",", "True", ",", "False"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"True", ",", "False", ",", "False", ",", "False"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"False", ",", "True", ",", "True", ",", "False"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"False", ",", "True", ",", "False", ",", "True"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"False", ",", "False", ",", "True", ",", "False"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"False", ",", "False", ",", "False", ",", "False"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The third element of the output indicates that the function 132 returns \
false on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"true", ",", "false", ",", "true"}], ")"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "We can now replace the specific application of ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " with a ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " to allow the integer to range. For demonstration purposes, we restrict the \
range to 100 to 105. We also apply ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 " with the ",
 StyleBox[ButtonBox["TableDepth",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableDepth"], "FunctionLink"],
 " option set to 1 to make the output readable."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{
   RowBox[{"BooleanTable", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "y", ",", "z", ",", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"BooleanFunction", "[", 
           RowBox[{"k", ",", "3"}], "]"}], "[", 
          RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "100", ",", "105"}], "}"}]}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}], ",", 
   RowBox[{"TableDepth", "\[Rule]", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{"{", 
       RowBox[{"True", ",", "True", ",", "True", ",", 
        RowBox[{"{", 
         RowBox[{
         "False", ",", "False", ",", "False", ",", "False", ",", "False", ",",
           "False"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{"True", ",", "True", ",", "False", ",", 
        RowBox[{"{", 
         RowBox[{
         "True", ",", "True", ",", "True", ",", "True", ",", "True", ",", 
          "True"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{"True", ",", "False", ",", "True", ",", 
        RowBox[{"{", 
         RowBox[{
         "True", ",", "True", ",", "True", ",", "True", ",", "True", ",", 
          "True"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{"True", ",", "False", ",", "False", ",", 
        RowBox[{"{", 
         RowBox[{
         "False", ",", "False", ",", "False", ",", "False", ",", "False", ",",
           "False"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{"False", ",", "True", ",", "True", ",", 
        RowBox[{"{", 
         RowBox[{
         "False", ",", "False", ",", "False", ",", "False", ",", "True", ",", 
          "True"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{"False", ",", "True", ",", "False", ",", 
        RowBox[{"{", 
         RowBox[{
         "True", ",", "True", ",", "True", ",", "True", ",", "False", ",", 
          "False"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{"False", ",", "False", ",", "True", ",", 
        RowBox[{"{", 
         RowBox[{
         "False", ",", "False", ",", "True", ",", "True", ",", "False", ",", 
          "False"}], "}"}]}], "}"}]},
     {
      RowBox[{"{", 
       RowBox[{"False", ",", "False", ",", "False", ",", 
        RowBox[{"{", 
         RowBox[{
         "False", ",", "True", ",", "False", ",", "True", ",", "False", ",", 
          "True"}], "}"}]}], "}"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$, TableDepth -> 1]]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The output above indicates that, on the input ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"True", ",", "False", ",", "True"}], ")"}], TraditionalForm]]],
 ", all 6 Boolean functions associated to the integers 100 through 105 output \
true."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 6", "Subsection",
 CellTags->"SolutionsCE6"],

Cell["\<\
Randomly generate 10 different Boolean expressions in four variables and \
determine the average number of steps required to minimize them using the \
Quine-McCluskey method.\
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " To solve this problem, we need to find a way to generate random Boolean \
expressions, and then we must find a method of examining the minimization \
process so that we can count the number of steps."
}], "Text"],

Cell[TextData[{
 "Using what we learned about ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " in the solution to Computer Projects 2 above, we can produce random \
Boolean functions by applying ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " to random integers. For an expression in four variables, there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     SuperscriptBox["2", "4"]], "=", "65536"}], TraditionalForm]]],
 " different Boolean functions. So we choose 10 different integers between 0 \
and 65535 using ",
 StyleBox[ButtonBox["RandomSample",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomSample"], "FunctionLink"],
 ". Recall that ",
 StyleBox[ButtonBox["RandomSample",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomSample"], "FunctionLink"],
 " accepts a list of elements, which in this case will be obtained from ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 ", and a positive integer. It returns a list of the specified number of \
elements randomly selected from the list of objects."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RandomSample", "[", 
  RowBox[{
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", "65535"}], "]"}], ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25323", ",", "34897", ",", "30135", ",", "40167", ",", "62488", ",", 
   "20582", ",", "34327", ",", "40283", ",", "61593", ",", "64736"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Applying ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " to each of the values output by ",
 StyleBox[ButtonBox["RandomSample",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomSample"], "FunctionLink"],
 " will produce a list of 10 randomly chosen Boolean expressions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"BooleanFunction", "[", 
     RowBox[{"k", ",", "4", ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", 
      RowBox[{"RandomSample", "[", 
       RowBox[{
        RowBox[{"Range", "[", 
         RowBox[{"0", ",", "65535"}], "]"}], ",", "10"}], "]"}]}], "}"}]}], 
   "]"}], "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", 
         RowBox[{"!", "x"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", 
         RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "x"}], "&&", 
         RowBox[{"!", "y"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "x"}], "&&", "z"}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{"w", "&&", 
         RowBox[{"!", "x"}], "&&", "y"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", "x"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{"x", "&&", 
         RowBox[{"!", "y"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{"x", "&&", "z"}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{"w", "&&", 
         RowBox[{"!", "x"}], "&&", 
         RowBox[{"!", "z"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", "x", "&&", "y", "&&", 
         RowBox[{"!", "z"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", 
         RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "x"}], "&&", 
         RowBox[{"!", "y"}], "&&", 
         RowBox[{"!", "z"}]}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{"w", "&&", "y", "&&", "z"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", 
         RowBox[{"!", "x"}], "&&", 
         RowBox[{"!", "y"}], "&&", 
         RowBox[{"!", "z"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{"x", "&&", "y"}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{"w", "&&", 
         RowBox[{"!", "y"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", "y", "&&", "z"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{"x", "&&", "y", "&&", 
         RowBox[{"!", "z"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{"x", "&&", 
         RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{"w", "&&", 
         RowBox[{"!", "y"}], "&&", 
         RowBox[{"!", "z"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", "x"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "x"}], "&&", "y", "&&", "z"}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{"w", "&&", "x", "&&", "y", "&&", "z"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", 
         RowBox[{"!", "y"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "x"}], "&&", 
         RowBox[{"!", "y"}]}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", "y"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{"x", "&&", 
         RowBox[{"!", "y"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{"x", "&&", 
         RowBox[{"!", "z"}]}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{"w", "&&", "x", "&&", "y", "&&", "z"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{"w", "&&", "x", "&&", 
         RowBox[{"!", "y"}], "&&", 
         RowBox[{"!", "z"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", "x", "&&", "y", "&&", 
         RowBox[{"!", "z"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", 
         RowBox[{"!", "x"}], "&&", 
         RowBox[{"!", "y"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", 
         RowBox[{"!", "x"}], "&&", "z"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", 
         RowBox[{"!", "y"}], "&&", "z"}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "x"}], "&&", 
         RowBox[{"!", "y"}], "&&", "z"}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{"w", "&&", "x", "&&", 
         RowBox[{"!", "z"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", 
         RowBox[{"!", "x"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"!", "w"}], "&&", 
         RowBox[{"!", "y"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{"x", "&&", 
         RowBox[{"!", "y"}], "&&", 
         RowBox[{"!", "z"}]}], ")"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell["\<\
Having determined how to generate random expressions, we need to find a way \
to count the number of steps taken during the minimization process. There are \
several approaches we could take to this part of the problem.\
\>", "Text"],

Cell["\<\
The first is to measure the time taken to execute the procedure. We have done \
this many times before.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"QMtimes", "=", 
   RowBox[{"{", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"randExprs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"BooleanFunction", "[", 
      RowBox[{"k", ",", "4", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"RandomSample", "[", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"0", ",", "65535"}], "]"}], ",", "10"}], "]"}]}], "}"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"i", "=", "1"}], ",", 
    RowBox[{"i", "\[LessEqual]", "10"}], ",", 
    RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{"AppendTo", "[", 
     RowBox[{"QMtimes", ",", 
      RowBox[{
       RowBox[{"Timing", "[", 
        RowBox[{"quineMcCluskey", "[", 
         RowBox[{
          RowBox[{"randExprs", "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "}"}]}], "]"}], "]"}],
        "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Mean", "[", "QMtimes", "]"}]}], "Input"],

Cell[BoxData["0.00104570000000000001491584633583897812`3.0400070212985075"], \
"Output"]
}, Open  ]],

Cell[TextData[{
 "The second approach is to modify the implementation of Quine-McCluskey to \
count the number of times certain operations are called. For example, we may \
be interested in the number of times that the ",
 StyleBox["updateCT", "Input"],
 " procedure is executed. In this case, we can alter ",
 StyleBox["quineMcCluskey", "Input"],
 " to include a variable that is incremented at the start of every execution \
of ",
 StyleBox["updateCT", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"quineMcCluskeycountCT", "[", 
   RowBox[{"F_", ",", "variables_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "fBits", ",", "fBitsL", ",", "coverageDict", ",", "groupsL", ",", 
      "primesL", ",", "newFbits", ",", "newprimes", ",", "i", ",", 
      "allprimeImps", ",", "j", ",", "coverageTable", ",", "essentialPIs", 
      ",", "minBits", ",", "nextPI", ",", 
      RowBox[{"count", "=", "0"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"fBits", "=", 
      RowBox[{"dnfToBitList", "[", 
       RowBox[{"F", ",", "variables"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"initCoverDict", "[", 
      RowBox[{"coverageDict", ",", "fBits"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"i", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"fBitsL", "=", 
      RowBox[{"{", "fBits", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"groupsL", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"primesL", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"fBitsL", "[", 
         RowBox[{"[", 
          RowBox[{"-", "1"}], "]"}], "]"}], "=!=", 
        RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"AppendTo", "[", 
         RowBox[{"groupsL", ",", 
          RowBox[{"sortGroups", "[", 
           RowBox[{"fBitsL", "[", 
            RowBox[{"[", 
             RowBox[{"-", "1"}], "]"}], "]"}], "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"newFbits", ",", "newprimes"}], "}"}], "=", 
         RowBox[{"nextBitList", "[", 
          RowBox[{
           RowBox[{"groupsL", "[", 
            RowBox[{"[", 
             RowBox[{"-", "1"}], "]"}], "]"}], ",", "coverageDict"}], "]"}]}],
         ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"fBitsL", ",", "newFbits"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"primesL", ",", "newprimes"}], "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"allprimeImps", "=", 
      RowBox[{"Union", "@@", "primesL"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"coverageTable", "=", 
      RowBox[{"initCoverageMatrix", "[", 
       RowBox[{"fBits", ",", "allprimeImps", ",", "coverageDict"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"essentialPIs", "=", 
      RowBox[{"findEssentials", "[", "coverageTable", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"minBits", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"AppendTo", "[", 
         RowBox[{"minBits", ",", "\[IndentingNewLine]", 
          RowBox[{"updateCT", "[", 
           RowBox[{
           "i", ",", "coverageTable", ",", "fBits", ",", "allprimeImps"}], 
           "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"count", "++"}]}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "essentialPIs"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"MatrixQ", "[", "coverageTable", "]"}], "&&", 
        RowBox[{
         RowBox[{
          RowBox[{"Dimensions", "[", "coverageTable", "]"}], "[", 
          RowBox[{"[", "2", "]"}], "]"}], ">", "0"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"nextPI", "=", 
         RowBox[{"findBestImp", "[", "coverageTable", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"minBits", ",", 
          RowBox[{"updateCT", "[", 
           RowBox[{
           "nextPI", ",", "coverageTable", ",", "fBits", ",", 
            "allprimeImps"}], "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"bitListToDNF", "[", 
      RowBox[{"minBits", ",", "variables"}], "]"}], ";", 
     "\[IndentingNewLine]", "count"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now execute ",
 StyleBox["quineMcCluskeycountCT", "Input"],
 " on 10 random expressions."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"QMtotal", "=", "0"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"randExprs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"BooleanFunction", "[", 
      RowBox[{"k", ",", "4", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", 
       RowBox[{"RandomSample", "[", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"0", ",", "65535"}], "]"}], ",", "10"}], "]"}]}], "}"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"i", "=", "1"}], ",", 
    RowBox[{"i", "\[LessEqual]", "10"}], ",", 
    RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"QMtotal", "=", 
      RowBox[{"QMtotal", "+", 
       RowBox[{"quineMcCluskeycountCT", "[", 
        RowBox[{
         RowBox[{"randExprs", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", "y", ",", "z", ",", "w"}], "}"}]}], "]"}]}]}], 
     ";"}]}], "\[IndentingNewLine]", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"N", "[", 
  RowBox[{"QMtotal", "/", "10"}], "]"}]}], "Input"],

Cell[BoxData["4.6`"], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "Section",
 CellTags->"Exercises"],

Cell[CellGroupData[{

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to verify De Morgan's Laws and the commutative and associative laws. (See \
Table 5 of Section 12.1.)"
}], "Item1Exercise"],

Cell["\<\
Construct truth tables for each of the following pairs of Boolean expressions \
and decide whether they are logically equivalent.\
\>", "Item1Exercise"],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[Rule]", "b"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[Rule]", "a"}], TraditionalForm]]]
}], "Item2Exercise"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[Rule]", 
    OverscriptBox["b", "_"]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[Rule]", 
    OverscriptBox["a", "_"]}], TraditionalForm]]]
}], "Item2Exercise"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "+", 
    RowBox[{"b", " ", "c"}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "+", "b", "+", "d"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"a", "+", "c", "+", "d"}], ")"}]}], TraditionalForm]]]
}], "Item2Exercise"]
}, Open  ]],

Cell[TextData[{
 "Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that, given a Boolean function, represents this function using \
only the ",
 StyleBox[ButtonBox["Nand",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Nand"], "FunctionLink"],
 " operator."
}], "Item1Exercise"],

Cell[TextData[{
 "Use the function in the previous exercise to represent the following \
Boolean functions using only the ",
 StyleBox[ButtonBox["Nand",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Nand"], "FunctionLink"],
 " operator."
}], "Item1Exercise"],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"F", "(", 
    RowBox[{"x", ",", "y", ",", "z"}], ")"}], "=", 
   RowBox[{
    RowBox[{"x", " ", "y"}], "+", 
    RowBox[{
     OverscriptBox["y", "_"], " ", "z"}]}]}], 
  TraditionalForm]]]], "Item2Exercise"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"G", "(", 
    RowBox[{"x", ",", "y", ",", "z"}], ")"}], "=", 
   RowBox[{"x", "+", 
    RowBox[{
     OverscriptBox["x", "_"], " ", "y"}], "+", 
    RowBox[{
     OverscriptBox[
      RowBox[{"y", " ", "z"}], "_"], Cell[""]}]}]}], 
  TraditionalForm]]]], "Item2Exercise"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"H", "(", 
    RowBox[{"x", ",", "y", ",", "z"}], ")"}], "=", 
   RowBox[{
    RowBox[{"x", " ", "y", " ", "z"}], "+", " ", 
    OverscriptBox[
     RowBox[{"x", " ", "y", " ", "z"}], "_"]}]}], 
  TraditionalForm]]]], "Item2Exercise"]
}, Open  ]],

Cell[TextData[{
 "Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that, given a Boolean function, represents this function using \
only the ",
 StyleBox[ButtonBox["Nor",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Nor"], "FunctionLink"],
 " operator."
}], "Item1Exercise"],

Cell[TextData[{
 "Use the function in the previous exercise to represent the Boolean \
functions in Exercise 4 using only the ",
 StyleBox[ButtonBox["Nor",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Nor"], "FunctionLink"],
 " operator."
}], "Item1Exercise"],

Cell[TextData[{
 "Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function for determining the output of a threshold gate, given the values \
of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " Boolean variables as input, and given the threshold value and a set of \
weights for the threshold gate. (See the Supplementary Exercises of Chapter \
12 for information on threshold gates. )"
}], "Item1Exercise"],

Cell[TextData[{
 "Develop a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that, given a Boolean function in four variables, determines \
whether it is a threshold function, and if so, finds the appropriate \
threshold gate representing this function. (See the Supplementary Exercises \
of Chapter 12.)"
}], "Item1Exercise"],

Cell[TextData[{
 "A Boolean expression ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " is called self dual if it is logically equivalent to its dual ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["e", "d"], TraditionalForm]]],
 ". Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function to test whether a given expression is self dual."
}], "Item1Exercise"],

Cell[TextData[{
 "Determine, for each integer ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[Element]", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6"}], "}"}]}],
    TraditionalForm]]],
 ", the total number of Boolean functions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " variables and the number of those functions that are self dual."
}], "Item1Exercise"],

Cell[TextData[{
 "Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that, given a positive integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", constructs a list of all Boolean functions of degree ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Use your function to find all Boolean functions of degree 4. Do not use \
",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 "."
}], "Item1Exercise"],

Cell[TextData[{
 "At the end of Section 12.3 of this manual, it was suggested that the \
procedure for producing tree representations of logical circuits could be \
improved by combining successive ",
 Cell[BoxData[
  FormBox["and", TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox["or", TraditionalForm]]],
 " gates into gates accepting multiple inputs. Implement this."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox[ButtonBox["BooleanFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BooleanFunction"], "FunctionLink"],
 " to compute a minimal sum of products expansion for the Boolean functions \
with don't care conditions specified by the Karnaugh maps shown in Exercises \
30 through 32 of Section 12.4."
}], "Item1Exercise"],

Cell[TextData[{
 "Use the function you wrote in Exercise 9 to write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function to generate random Boolean expressions in 4 variables and stop \
when it is has found one that is self dual. Run the program several times and \
time it. Find the average number of random expressions needed before \
stopping. Repeat for Boolean expressions in 5 and 6 variables. Can you make \
any conjectures from this information?"
}], "Item1Exercise"],

Cell[TextData[{
 "Modify ",
 StyleBox["quineMcCluskey", "Input"],
 " to allow for don't care conditions. "
}], "Item1Exercise"],

Cell[TextData[{
 "Modify ",
 StyleBox["quineMcCluskey", "Input"],
 " to use backtracking instead of the heuristic approach in order to \
determine the expression with the minimum number of terms. Use a large number \
of randomly generated expressions to compare the old function with the new \
and determine how often the heuristic produces non-optimal output."
}], "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
WindowSize->{725, 850},
WindowMargins->{{12, Automatic}, {Automatic, 24}},
DockedCells->Cell[
  BoxData[
   TagBox[
    GridBox[{{"\"Exploring Discrete Mathematics with Mathematica\"", 
       ActionMenuBox[
       "\"Navigation\"", {
        "\"CloseAll\"" :> Module[{$CellContext`DJSection$}, SelectionMove[
             SelectedNotebook[], All, Notebook]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken["OpenCloseGroup"]], {$CellContext`DJSection$, {
              "Section1a", "Section1b", "Section1c", "Section1d", "Section2a",
                "Section2b", "Section3a", "Section3b", "Section4a", 
               "Section4b", "Section4c", "SolutionsCE6", "SolutionsCP2", 
               "Section1", "Section2", "Section3", "Section4", "Solutions", 
               "Exercises"}}]; SelectionMove[
             SelectedNotebook[], Before, Notebook]], "\"Section1\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Solutions", 
                "Exercises"}}] NotebookLocate["Section1"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section2\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Solutions", 
                "Exercises"}}] NotebookLocate["Section2"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section3\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Solutions", 
                "Exercises"}}] NotebookLocate["Section3"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section4\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Solutions", 
                "Exercises"}}] NotebookLocate["Section4"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Solutions\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Solutions", 
                "Exercises"}}] NotebookLocate["Solutions"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Exercises\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Solutions", 
                "Exercises"}}] NotebookLocate["Exercises"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Open All\"" :> 
         Module[{$CellContext`DJSection$}, 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken[
               "SelectionOpenAllGroups"]], {$CellContext`DJSection$, {
              "Exercises", "Section1", "Section1a", "Section1b", "Section1c", 
               "Section1d", "Section2", "Section2a", "Section2b", "Section3", 
               "Section3a", "Section3b", "Section4", "Section4a", "Section4b",
                "Section4c", "Solutions", "SolutionsCE6", "SolutionsCP2"}}] 
           SelectionMove[
             SelectedNotebook[], Before, Notebook]]}]}}, 
     GridBoxAlignment -> {
      "Columns" -> {Left, Baseline}, "Rows" -> {Right, Baseline}}, AutoDelete -> 
     False, GridBoxItemSize -> {"Columns" -> {
         Scaled[0.8], 
         Scaled[0.2]}}], "Grid"]], "DockedCell", FontFamily -> "Times"],
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
ShowSelection->True,
Magnification->1,
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, \
2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Book"}, "Textbook.nb", CharacterEncoding -> 
       "UTF-8"]]], 
   Cell[
    StyleData[All, "Printout"], PageWidth -> PaperWidth, 
    CellHorizontalScrolling -> True, 
    CellLabelMargins -> {{2, Inherited}, {Inherited, Inherited}}, 
    ShowAutoStyles -> False, 
    AutoStyleOptions -> {"HighlightMisspelledWords" -> False}, ScriptMinSize -> 
    9, MenuSortingValue -> 1500, 
    PrivateFontOptions -> {"FontType" -> "Outline"}, Magnification -> 1], 
   Cell[
    StyleData["Title"], Editable -> False, ShowCellBracket -> Automatic, 
    CellGroupingRules -> {"TitleGrouping", 0}, PageBreakBelow -> False, 
    DefaultNewInlineCellStyle -> {}, LanguageCategory -> "NaturalLanguage", 
    FontFamily -> "Helvetica", FontSize -> 25, FontWeight -> "Bold", 
    FontColor -> GrayLevel[0.137255]], 
   Cell[
    StyleData["BookChapterTitle", StyleDefinitions -> StyleData["Title"]], 
    CellFrame -> {{0, 0}, {1, 1}}, CellDingbat -> Cell[
      BoxData[
       GraphicsBox[{{
          RGBColor[0.09375, 0.570313, 0.816406], 
          RectangleBox[{0, 0}]}, {
          GrayLevel[1], FontSize -> 35, FontWeight -> Bold, FontFamily -> 
          "Helvetica", 
          InsetBox[
           CounterBox["ChapterNumber"], {0.5, 0.45}, {Center, Center}]}}, 
        ImageSize -> {70, 70}]], "Output"], CellMargins -> {{75, 0}, {20, 5}},
     CellFrameColor -> GrayLevel[1]], 
   Cell[
    StyleData["DockedCell"], CellMargins -> {{0, 12}, {0, 0}}, 
    CellGroupingRules -> "TitleGrouping", 
    ConversionRules :> {"TeX" -> {"\n\\chapter{", "}"}}, 
    TaggingRules :> {"CounterSettings" -> {"CounterTarget" -> "CellGroup"}}, 
    PageBreakAbove -> False, PageBreakBelow -> False, LineSpacing -> {1, 2}, 
    MenuSortingValue -> 1105, FontFamily -> "Helvetica", FontSize -> 20, 
    FontWeight -> "Bold", Background -> 
    RGBColor[0.09375, 0.570313, 0.816406]], 
   Cell[
    StyleData["FunctionLink"], ConversionRules :> {"TeX" -> {{"", 
         $CellContext`ExportAsHyperRef[], ""}, {"", Automatic, ""}}}, 
    MenuSortingValue -> 1105, ButtonStyleMenuListing -> None, FontFamily -> 
    "Courier", FontVariations -> {"Underline" -> True}, FontColor -> 
    RGBColor[0, 0, 1], 
    ButtonBoxOptions -> {Active -> True, ButtonFunction :> (FrontEndExecute[{
         FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> ButtonData}], 
   Cell[
    StyleData["Problem"], CellMargins -> {{40, 12}, {15, 15}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 15010}, 
    ConversionRules :> {
     "TeX" -> {{"", Automatic, ""}, {"\n\n", Automatic, ""}}}, 
    TaggingRules :> {
     "ExtraConversionRules" -> {"General" -> {"RestartList" -> False}}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {30, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item1Exercise", 
       CounterAssignments -> {{"Item2Exercise", 0}}, MenuSortingValue -> 
       4100], 
      Cell[
       StyleData["Item1Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{35, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["ExerciseContinuation"], CellMargins -> {{50, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, MenuSortingValue -> 4101], 
      Cell[
       StyleData["ExerciseContinuation", "Printout"], 
       CellMargins -> {{35, 12}, {1, 3}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item2Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{75, 12}, {1, 2}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17100}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item2Exercise", MenuSortingValue -> 4200, 
       CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}], 
      Cell[
       StyleData["Item2Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {1, 2}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}]}, Open]], 
   Cell[
    StyleData["ExerciseEquation"], CellMargins -> {{36, 12}, {7, 7}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 17200}, 
    CellHorizontalScrolling -> True, 
    CellFrameLabels -> {{None, None}, {None, None}}, DefaultFormatType -> 
    TraditionalForm, InputAutoReplacements -> {}, TextAlignment -> Center, 
    MenuSortingValue -> 4300, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["myCode"], CellMargins -> {{66, 10}, {5, 10}}, 
    StyleKeyMapping -> {"=" -> "WolframAlphaShort"}, Evaluatable -> False, 
    CellGroupingRules -> "InputGrouping", TextClipboardType -> "InputText", 
    PageBreakWithin -> False, GroupPageBreakWithin -> False, 
    DefaultFormatType -> DefaultInputFormatType, ContextMenu -> 
    FEPrivate`FrontEndResource["ContextMenus", "myCode"], ShowAutoStyles -> 
    True, "TwoByteSyntaxCharacterAutoReplacement" -> True, 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    AutoItalicWords -> {}, LanguageCategory -> "Mathematica", FormatType -> 
    InputForm, NumberMarks -> True, 
    LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, CounterIncrements -> "Code", 
    MenuSortingValue -> 1495, MenuCommandKey -> "8", FontWeight -> 
    "Bold", $CellContext`StripStyleOnPaste -> 
    True, $CellContext`AutoQuoteCharacters -> {}, \
$CellContext`PasteAutoQuoteCharacters -> {}], 
   Cell[
    StyleData["CodeVar"], DefaultFormatType -> DefaultInputFormatType, 
    LineBreakWithin -> False, MenuSortingValue -> 10000, FontWeight -> "Bold",
     FontSlant -> "Italic", FontColor -> GrayLevel[0.4]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, ShowCellBracket -> 
       Automatic, CellMargins -> {{12, 12}, {3, 24}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, 
       ConversionRules :> {
        "TeX" -> {"\n\\section{", "}"}, 
         "HTML" -> {"<h3 class=\"Section\">", "</h3>"}}, 
       TaggingRules :> {
        "CounterSettings" -> {
          "CounterFormat" -> {"BookChapterNumber", ".", "Section"}, 
           "CounterTarget" -> "Cell", "XRefPrefix" -> "Section"}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{8, 8}, {4, 6}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], TabSpacings -> 
       2.1, CounterIncrements -> {
        "Section", "SectionHeader", "SubsectionHeader", "SubsubsectionHeader",
          "SubsubsubsectionHeader"}, FontFamily -> "Helvetica", FontSize -> 
       16, FontWeight -> "Bold", Background -> GrayLevel[0.999985]], 
      Cell[
       StyleData["Section", "Printout"], CellFrame -> {{4, 0}, {0, 0.5}}, 
       CellMargins -> {{2, 2}, {3, 21}}, PageBreakBelow -> False, 
       CellFrameMargins -> {{6, 6}, {3, 5}}, CellFrameColor -> 
       RGBColor[0.09375, 0.570313, 0.816406]], 
      Cell[
       StyleData["Section", "DoubleSpacedPrintout"], 
       CellFrame -> {{4, 0}, {0, 0.5}}, CellMargins -> {{2, 2}, {3, 21}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{6, 6}, {3, 5}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], FontSize -> 
       14]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellElementSpacings -> {"ClosedGroupTopMargin" -> 12}, 
       CellGroupingRules -> {"SectionGrouping", 50}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsection{", "}"}, 
         "HTML" -> {"<h5 class=\"Subsubsection\">", "</h5>"}}, PageBreakBelow -> 
       False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {
        "Subsubsection", "SubsubsectionHeader", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsection", "Printout"], PageBreakBelow -> False, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"]}, Open]], 
   Cell[
    StyleData["Equation"], CellMargins -> {{26, 10}, {7, 7}}, 
    ConversionRules :> {
     "HTML" -> {{"", ""}, {"<p class=\"Equation\">", "</p>"}}}, 
    CellHorizontalScrolling -> True, DefaultFormatType -> TraditionalForm, 
    InputAutoReplacements -> {}, TextAlignment -> Center, CounterIncrements -> 
    "Equation", ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["Subsubsubsection"], CellDingbat -> None, 
    CellMargins -> {{12, 12}, {0, 6}}, 
    CellGroupingRules -> {"SectionGrouping", 60}, 
    ConversionRules :> {
     "TeX" -> {"\n\\subsubsubsection{", "}"}, 
      "HTML" -> {"<h6 class=\"Subsubsubsection\">", "</h6>"}}, PageBreakBelow -> 
    False, CellFrameLabelMargins -> 4, 
    CounterIncrements -> {"Subsubsubsection", "SubsubsubsectionHeader"}, 
    FontFamily -> "Helvetica", FontSize -> 11, FontSlant -> "Italic"]}, 
  Visible -> False, FrontEndVersion -> 
  "9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, 2013)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Section1"->{
  Cell[1848, 57, 64, 1, 55, "Section",
   CellTags->"Section1"]},
 "Section1a"->{
  Cell[2230, 71, 59, 1, 32, "Subsection",
   CellTags->"Section1a"]},
 "Section1b"->{
  Cell[11622, 408, 87, 1, 26, "Subsection",
   CellTags->"Section1b"]},
 "Section1c"->{
  Cell[28432, 1046, 75, 1, 26, "Subsection",
   CellTags->"Section1c"]},
 "Section1d"->{
  Cell[37922, 1402, 53, 1, 26, "Subsection",
   CellTags->"Section1d"]},
 "Section2"->{
  Cell[43329, 1606, 77, 1, 35, "Section",
   CellTags->"Section2"]},
 "Section2a"->{
  Cell[43918, 1624, 88, 1, 32, "Subsection",
   CellTags->"Section2a"]},
 "Section2b"->{
  Cell[47691, 1770, 82, 1, 26, "Subsection",
   CellTags->"Section2b"]},
 "Section3"->{
  Cell[59737, 2180, 58, 1, 35, "Section",
   CellTags->"Section3"]},
 "Section3a"->{
  Cell[60269, 2198, 83, 1, 32, "Subsection",
   CellTags->"Section3a"]},
 "Section3b"->{
  Cell[162203, 4047, 83, 1, 26, "Subsection",
   CellTags->"Section3b"]},
 "Section4"->{
  Cell[190925, 4737, 71, 1, 35, "Section",
   CellTags->"Section4"]},
 "Section4a"->{
  Cell[191334, 4751, 118, 5, 33, "Subsection",
   CellTags->"Section4a"]},
 "Section4b"->{
  Cell[194744, 4879, 84, 1, 26, "Subsection",
   CellTags->"Section4b"]},
 "Section4c"->{
  Cell[202618, 5149, 61, 1, 26, "Subsection",
   CellTags->"Section4c"]},
 "Solutions"->{
  Cell[346853, 9846, 115, 3, 35, "Section",
   CellTags->"Solutions"]},
 "SolutionsCP2"->{
  Cell[346993, 9853, 68, 1, 32, "Subsection",
   CellTags->"SolutionsCP2"]},
 "SolutionsCE6"->{
  Cell[358635, 10243, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE6"]},
 "Exercises"->{
  Cell[375204, 10749, 52, 1, 35, "Section",
   CellTags->"Exercises"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Section1", 398380, 11311},
 {"Section1a", 398462, 11314},
 {"Section1b", 398548, 11317},
 {"Section1c", 398636, 11320},
 {"Section1d", 398725, 11323},
 {"Section2", 398813, 11326},
 {"Section2a", 398898, 11329},
 {"Section2b", 398987, 11332},
 {"Section3", 399075, 11335},
 {"Section3a", 399160, 11338},
 {"Section3b", 399249, 11341},
 {"Section4", 399338, 11344},
 {"Section4a", 399424, 11347},
 {"Section4b", 399515, 11350},
 {"Section4c", 399605, 11353},
 {"Solutions", 399695, 11356},
 {"SolutionsCP2", 399786, 11359},
 {"SolutionsCE6", 399882, 11362},
 {"Exercises", 399976, 11365}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 89, 1, 96, "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 12}}],
Cell[CellGroupData[{
Cell[693, 27, 31, 0, 55, "Section"],
Cell[727, 29, 1084, 23, 110, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1848, 57, 64, 1, 55, "Section",
 CellTags->"Section1"],
Cell[1915, 60, 290, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[2230, 71, 59, 1, 32, "Subsection",
 CellTags->"Section1a"],
Cell[2292, 74, 799, 28, 36, "Text"],
Cell[3094, 104, 758, 28, 36, "Text"],
Cell[CellGroupData[{
Cell[3877, 136, 128, 4, 29, "Input"],
Cell[4008, 142, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4077, 147, 177, 5, 28, "Input"],
Cell[4257, 154, 32, 0, 28, "Output"]
}, Open  ]],
Cell[4304, 157, 241, 6, 35, "Text"],
Cell[4548, 165, 655, 18, 52, "Text"],
Cell[5206, 185, 623, 15, 119, "Text"],
Cell[5832, 202, 619, 21, 37, "Text"],
Cell[6454, 225, 1129, 34, 67, "Text"],
Cell[CellGroupData[{
Cell[7608, 263, 59, 1, 28, "Input"],
Cell[7670, 266, 45, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7752, 272, 59, 1, 28, "Input"],
Cell[7814, 275, 45, 1, 28, "Output"]
}, Open  ]],
Cell[7874, 279, 1056, 26, 83, "Text"],
Cell[CellGroupData[{
Cell[8955, 309, 187, 5, 28, "Input"],
Cell[9145, 316, 188, 5, 28, "Output"]
}, Open  ]],
Cell[9348, 324, 662, 19, 37, "Text"],
Cell[10013, 345, 721, 25, 89, "Input"],
Cell[10737, 372, 181, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[10943, 382, 187, 5, 28, "Input"],
Cell[11133, 389, 152, 4, 28, "Output"]
}, Open  ]],
Cell[11300, 396, 285, 7, 35, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[11622, 408, 87, 1, 26, "Subsection",
 CellTags->"Section1b"],
Cell[11712, 411, 1164, 41, 55, "Text"],
Cell[CellGroupData[{
Cell[12901, 456, 156, 5, 29, "Input"],
Cell[13060, 463, 32, 0, 28, "Output"]
}, Open  ]],
Cell[13107, 466, 154, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[13286, 473, 108, 3, 29, "Input"],
Cell[13397, 478, 83, 2, 29, "Output"]
}, Open  ]],
Cell[13495, 483, 307, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[13827, 496, 294, 9, 29, "Input"],
Cell[14124, 507, 32, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14193, 512, 55, 0, 18, "Subsubsection"],
Cell[14251, 514, 157, 5, 20, "Text"],
Cell[14411, 521, 364, 12, 20, "Text"],
Cell[14778, 535, 139, 5, 20, "Text"],
Cell[14920, 542, 379, 12, 29, "Input",
 InitializationCell->True],
Cell[15302, 556, 253, 12, 21, "Text"],
Cell[CellGroupData[{
Cell[15580, 572, 87, 2, 29, "Input"],
Cell[15670, 576, 206, 7, 29, "Output"]
}, Open  ]],
Cell[15891, 586, 111, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[16027, 594, 97, 2, 28, "Input"],
Cell[16127, 598, 31, 0, 28, "Output"]
}, Open  ]],
Cell[16173, 601, 203, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[16401, 610, 90, 2, 29, "Input"],
Cell[16494, 614, 110, 3, 29, "Output"]
}, Open  ]],
Cell[16619, 620, 413, 13, 36, "Text"],
Cell[CellGroupData[{
Cell[17057, 637, 122, 3, 29, "Input"],
Cell[17182, 642, 51, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[17282, 649, 52, 0, 18, "Subsubsection"],
Cell[17337, 651, 466, 9, 51, "Text"],
Cell[17806, 662, 389, 8, 36, "Text"],
Cell[18198, 672, 574, 12, 51, "Text"],
Cell[CellGroupData[{
Cell[18797, 688, 326, 10, 29, "Input"],
Cell[19126, 700, 822, 22, 150, "Output"]
}, Open  ]],
Cell[19963, 725, 837, 24, 53, "Text"],
Cell[CellGroupData[{
Cell[20825, 753, 366, 11, 29, "Input"],
Cell[21194, 766, 710, 22, 152, "Output"]
}, Open  ]],
Cell[21919, 791, 672, 18, 37, "Text"],
Cell[CellGroupData[{
Cell[22616, 813, 573, 17, 45, "Input"],
Cell[23192, 832, 1191, 40, 168, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[24432, 878, 56, 0, 18, "Subsubsection"],
Cell[24491, 880, 228, 4, 35, "Text"],
Cell[24722, 886, 560, 18, 35, "Text"],
Cell[25285, 906, 155, 5, 29, "Input",
 InitializationCell->True],
Cell[25443, 913, 426, 17, 39, "Text"],
Cell[25872, 932, 206, 7, 29, "Input",
 InitializationCell->True],
Cell[26081, 941, 200, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[26306, 952, 80, 2, 29, "Input"],
Cell[26389, 956, 93, 3, 29, "Output"]
}, Open  ]],
Cell[26497, 962, 261, 10, 20, "Text"],
Cell[26761, 974, 174, 6, 29, "Input",
 InitializationCell->True],
Cell[26938, 982, 450, 16, 37, "Text"],
Cell[27391, 1000, 265, 9, 29, "Input",
 InitializationCell->True],
Cell[27659, 1011, 203, 7, 20, "Text"],
Cell[CellGroupData[{
Cell[27887, 1022, 111, 3, 29, "Input"],
Cell[28001, 1027, 28, 0, 28, "Output"]
}, Open  ]],
Cell[28044, 1030, 339, 10, 20, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[28432, 1046, 75, 1, 26, "Subsection",
 CellTags->"Section1c"],
Cell[28510, 1049, 388, 11, 37, "Text"],
Cell[28901, 1062, 735, 25, 35, "Text"],
Cell[29639, 1089, 182, 4, 35, "Text"],
Cell[29824, 1095, 394, 13, 45, "Input",
 InitializationCell->True],
Cell[30221, 1110, 499, 12, 37, "Text"],
Cell[CellGroupData[{
Cell[30745, 1126, 232, 6, 29, "Input"],
Cell[30980, 1134, 31, 0, 28, "Output"]
}, Open  ]],
Cell[31026, 1137, 57, 0, 20, "Text"],
Cell[31086, 1139, 398, 7, 36, "Text"],
Cell[31487, 1148, 579, 21, 21, "Text"],
Cell[CellGroupData[{
Cell[32091, 1173, 281, 9, 29, "Input"],
Cell[32375, 1184, 32, 0, 28, "Output"]
}, Open  ]],
Cell[32422, 1187, 220, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[32667, 1197, 318, 10, 29, "Input"],
Cell[32988, 1209, 104, 3, 28, "Output"]
}, Open  ]],
Cell[33107, 1215, 1322, 51, 50, "Text"],
Cell[34432, 1268, 557, 12, 51, "Text"],
Cell[CellGroupData[{
Cell[35014, 1284, 328, 10, 29, "Input"],
Cell[35345, 1296, 185, 6, 28, "Output"]
}, Open  ]],
Cell[35545, 1305, 340, 10, 21, "Text"],
Cell[35888, 1317, 57, 0, 20, "Text"],
Cell[35948, 1319, 248, 9, 29, "Equation"],
Cell[36199, 1330, 233, 8, 26, "Equation"],
Cell[36435, 1340, 51, 0, 20, "Text"],
Cell[36489, 1342, 416, 15, 45, "Input",
 InitializationCell->True],
Cell[36908, 1359, 590, 21, 21, "Text"],
Cell[CellGroupData[{
Cell[37523, 1384, 316, 10, 29, "Input"],
Cell[37842, 1396, 31, 0, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[37922, 1402, 53, 1, 26, "Subsection",
 CellTags->"Section1d"],
Cell[37978, 1405, 256, 8, 36, "Text"],
Cell[38237, 1415, 480, 13, 51, "Text"],
Cell[38720, 1430, 356, 10, 29, "Input",
 InitializationCell->True],
Cell[39079, 1442, 352, 9, 36, "Text"],
Cell[39434, 1453, 822, 27, 36, "Text"],
Cell[CellGroupData[{
Cell[40281, 1484, 369, 13, 29, "Input",
 InitializationCell->True],
Cell[40653, 1499, 266, 10, 29, "Output"]
}, Open  ]],
Cell[40934, 1512, 348, 12, 20, "Text"],
Cell[CellGroupData[{
Cell[41307, 1528, 369, 13, 29, "Input",
 InitializationCell->True],
Cell[41679, 1543, 266, 10, 29, "Output"]
}, Open  ]],
Cell[41960, 1556, 1033, 31, 51, "Text"],
Cell[CellGroupData[{
Cell[43018, 1591, 216, 6, 29, "Input"],
Cell[43237, 1599, 31, 0, 28, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[43329, 1606, 77, 1, 35, "Section",
 CellTags->"Section2"],
Cell[43409, 1609, 484, 11, 50, "Text"],
Cell[CellGroupData[{
Cell[43918, 1624, 88, 1, 32, "Subsection",
 CellTags->"Section2a"],
Cell[44009, 1627, 300, 7, 36, "Text"],
Cell[44312, 1636, 440, 16, 20, "Text"],
Cell[CellGroupData[{
Cell[44777, 1656, 176, 6, 29, "Input",
 InitializationCell->True],
Cell[44956, 1664, 117, 4, 29, "Output"]
}, Open  ]],
Cell[45088, 1671, 747, 17, 68, "Text"],
Cell[45838, 1690, 202, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[46065, 1700, 107, 2, 29, "Input"],
Cell[46175, 1704, 187, 7, 29, "Output"]
}, Open  ]],
Cell[46377, 1714, 274, 7, 35, "Text"],
Cell[46654, 1723, 251, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[46930, 1733, 74, 1, 29, "Input"],
Cell[47007, 1736, 187, 7, 29, "Output"]
}, Open  ]],
Cell[47209, 1746, 181, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[47415, 1756, 107, 2, 29, "Input"],
Cell[47525, 1760, 117, 4, 29, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[47691, 1770, 82, 1, 26, "Subsection",
 CellTags->"Section2b"],
Cell[47776, 1773, 275, 6, 35, "Text"],
Cell[48054, 1781, 543, 10, 51, "Text"],
Cell[48600, 1793, 90, 2, 20, "Text"],
Cell[48693, 1797, 516, 17, 165, "Text"],
Cell[49212, 1816, 1304, 38, 68, "Text"],
Cell[50519, 1856, 276, 9, 21, "Text"],
Cell[CellGroupData[{
Cell[50820, 1869, 1015, 30, 60, "Input",
 InitializationCell->True],
Cell[51838, 1901, 175, 5, 30, "Output"]
}, Open  ]],
Cell[52028, 1909, 92, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[52145, 1915, 105, 2, 29, "Input"],
Cell[52253, 1919, 32, 0, 28, "Output"]
}, Open  ]],
Cell[52300, 1922, 410, 10, 37, "Text"],
Cell[CellGroupData[{
Cell[52735, 1936, 1039, 31, 45, "Input"],
Cell[53777, 1969, 297, 11, 29, "Output"]
}, Open  ]],
Cell[54089, 1983, 266, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[54380, 1993, 1062, 32, 45, "Input"],
Cell[55445, 2027, 297, 11, 29, "Output"]
}, Open  ]],
Cell[55757, 2041, 564, 14, 52, "Text"],
Cell[CellGroupData[{
Cell[56346, 2059, 700, 21, 45, "Input"],
Cell[57049, 2082, 297, 11, 29, "Output"]
}, Open  ]],
Cell[57361, 2096, 67, 0, 20, "Text"],
Cell[57431, 2098, 247, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[57703, 2106, 270, 8, 29, "Input"],
Cell[57976, 2116, 297, 11, 29, "Output"]
}, Open  ]],
Cell[58288, 2130, 543, 12, 52, "Text"],
Cell[CellGroupData[{
Cell[58856, 2146, 223, 7, 29, "Input"],
Cell[59082, 2155, 594, 18, 86, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[59737, 2180, 58, 1, 35, "Section",
 CellTags->"Section3"],
Cell[59798, 2183, 446, 11, 50, "Text"],
Cell[CellGroupData[{
Cell[60269, 2198, 83, 1, 32, "Subsection",
 CellTags->"Section3a"],
Cell[60355, 2201, 57, 0, 20, "Text"],
Cell[60415, 2203, 51489, 849, 232, "Text"],
Cell[111907, 3054, 188, 5, 20, "Text"],
Cell[112098, 3061, 708, 24, 37, "Text"],
Cell[CellGroupData[{
Cell[112831, 3089, 88, 2, 29, "Input"],
Cell[112922, 3093, 62, 1, 29, "Output"]
}, Open  ]],
Cell[112999, 3097, 215, 4, 35, "Text"],
Cell[113217, 3103, 44319, 731, 235, "Text"],
Cell[157539, 3836, 867, 31, 50, "Text"],
Cell[CellGroupData[{
Cell[158431, 3871, 105, 3, 28, "Input"],
Cell[158539, 3876, 53, 1, 29, "Output"]
}, Open  ]],
Cell[158607, 3880, 312, 11, 20, "Text"],
Cell[CellGroupData[{
Cell[158944, 3895, 104, 3, 28, "Input"],
Cell[159051, 3900, 51, 1, 28, "Output"]
}, Open  ]],
Cell[159117, 3904, 366, 17, 20, "Text"],
Cell[CellGroupData[{
Cell[159508, 3925, 116, 3, 28, "Input"],
Cell[159627, 3930, 64, 1, 28, "Output"]
}, Open  ]],
Cell[159706, 3934, 345, 16, 20, "Text"],
Cell[CellGroupData[{
Cell[160076, 3954, 80, 2, 28, "Input"],
Cell[160159, 3958, 45, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[160241, 3964, 80, 2, 28, "Input"],
Cell[160324, 3968, 45, 1, 28, "Output"]
}, Open  ]],
Cell[160384, 3972, 193, 7, 20, "Text"],
Cell[CellGroupData[{
Cell[160602, 3983, 28, 0, 28, "Input"],
Cell[160633, 3985, 198, 7, 29, "Output"]
}, Open  ]],
Cell[160846, 3995, 519, 19, 20, "Text"],
Cell[161368, 4016, 798, 26, 50, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[162203, 4047, 83, 1, 26, "Subsection",
 CellTags->"Section3b"],
Cell[162289, 4050, 479, 14, 50, "Text"],
Cell[162771, 4066, 170, 3, 35, "Text"],
Cell[162944, 4071, 528, 8, 65, "Text"],
Cell[163475, 4081, 271, 5, 35, "Text"],
Cell[163749, 4088, 497, 8, 66, "Text"],
Cell[CellGroupData[{
Cell[164271, 4100, 157, 5, 29, "Input"],
Cell[164431, 4107, 448, 10, 24, "Message"],
Cell[164882, 4119, 34, 0, 28, "Output"]
}, Open  ]],
Cell[164931, 4122, 220, 4, 35, "Text"],
Cell[165154, 4128, 287, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[165466, 4142, 208, 6, 29, "Input"],
Cell[165677, 4150, 209, 6, 29, "Output"]
}, Open  ]],
Cell[165901, 4159, 68, 0, 20, "Text"],
Cell[165972, 4161, 13307, 223, 149, "Text"],
Cell[179282, 4386, 241, 4, 35, "Text"],
Cell[179526, 4392, 384, 9, 36, "Text"],
Cell[179913, 4403, 197, 4, 35, "Text"],
Cell[180113, 4409, 599, 16, 53, "Text"],
Cell[180715, 4427, 3844, 101, 395, "Input",
 InitializationCell->True],
Cell[184562, 4530, 220, 6, 36, "Text"],
Cell[184785, 4538, 3190, 89, 268, "Input",
 InitializationCell->True],
Cell[187978, 4629, 296, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[188299, 4642, 224, 7, 29, "Input"],
Cell[188526, 4651, 726, 18, 77, "Output"]
}, Open  ]],
Cell[189267, 4672, 64, 0, 20, "Text"],
Cell[189334, 4674, 879, 29, 53, "Text"],
Cell[CellGroupData[{
Cell[190238, 4707, 62, 1, 28, "Input"],
Cell[190303, 4710, 404, 14, 39, "Output"]
}, Open  ]],
Cell[190722, 4727, 154, 4, 21, "Text"]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[190925, 4737, 71, 1, 35, "Section",
 CellTags->"Section4"],
Cell[190999, 4740, 310, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[191334, 4751, 118, 5, 33, "Subsection",
 CellTags->"Section4a"],
Cell[191455, 4758, 284, 7, 36, "Text"],
Cell[191742, 4767, 284, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[192051, 4778, 245, 8, 29, "Input"],
Cell[192299, 4788, 167, 6, 29, "Output"]
}, Open  ]],
Cell[192481, 4797, 187, 6, 20, "Text"],
Cell[192671, 4805, 597, 13, 52, "Text"],
Cell[CellGroupData[{
Cell[193293, 4822, 238, 8, 29, "Input"],
Cell[193534, 4832, 119, 4, 29, "Output"]
}, Open  ]],
Cell[193668, 4839, 595, 15, 51, "Text"],
Cell[CellGroupData[{
Cell[194288, 4858, 285, 9, 29, "Input"],
Cell[194576, 4869, 119, 4, 29, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[194744, 4879, 84, 1, 26, "Subsection",
 CellTags->"Section4b"],
Cell[194831, 4882, 276, 8, 20, "Text"],
Cell[195110, 4892, 1663, 62, 65, "Text"],
Cell[196776, 4956, 416, 12, 38, "Text"],
Cell[197195, 4970, 274, 7, 35, "Text"],
Cell[197472, 4979, 569, 14, 173, "Text"],
Cell[198044, 4995, 721, 24, 35, "Text"],
Cell[198768, 5021, 714, 20, 52, "Text"],
Cell[CellGroupData[{
Cell[199507, 5045, 404, 12, 29, "Input"],
Cell[199914, 5059, 109, 3, 29, "Output"]
}, Open  ]],
Cell[200038, 5065, 434, 11, 36, "Text"],
Cell[200475, 5078, 392, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[200892, 5092, 1377, 38, 153, "Input"],
Cell[202272, 5132, 297, 11, 29, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[202618, 5149, 61, 1, 26, "Subsection",
 CellTags->"Section4c"],
Cell[202682, 5152, 236, 4, 35, "Text"],
Cell[202921, 5158, 172, 3, 35, "Text"],
Cell[203096, 5163, 1176, 35, 26, "Equation"],
Cell[204275, 5200, 91, 4, 21, "Text"],
Cell[204369, 5206, 1453, 47, 77, "Input",
 InitializationCell->True],
Cell[205825, 5255, 122, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[205972, 5262, 65, 0, 16, "Item1Numbered"],
Cell[206040, 5264, 67, 0, 16, "Item1Numbered"],
Cell[206110, 5266, 83, 1, 16, "Item1Numbered"],
Cell[206196, 5269, 100, 2, 16, "Item1Numbered"],
Cell[206299, 5273, 142, 3, 16, "Item1Numbered"],
Cell[206444, 5278, 95, 2, 16, "Item1Numbered"],
Cell[206542, 5282, 139, 3, 16, "Item1Numbered"]
}, Open  ]],
Cell[206696, 5288, 225, 6, 35, "Text"],
Cell[CellGroupData[{
Cell[206946, 5298, 44, 0, 18, "Subsubsection"],
Cell[206993, 5300, 473, 9, 51, "Text"],
Cell[207469, 5311, 327, 5, 50, "Text"],
Cell[207799, 5318, 466, 10, 51, "Text"],
Cell[208268, 5330, 887, 25, 141, "Input"],
Cell[CellGroupData[{
Cell[209180, 5359, 141, 3, 29, "Input"],
Cell[209324, 5364, 94, 2, 28, "Output"]
}, Open  ]],
Cell[209433, 5369, 414, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[209872, 5384, 38, 0, 29, "Input"],
Cell[209913, 5386, 29, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[209979, 5391, 38, 0, 29, "Input"],
Cell[210020, 5393, 94, 2, 28, "Output"]
}, Open  ]],
Cell[210129, 5398, 319, 10, 21, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[210485, 5413, 63, 0, 18, "Subsubsection"],
Cell[210551, 5415, 323, 7, 35, "Text"],
Cell[210877, 5424, 376, 6, 50, "Text"],
Cell[211256, 5432, 162, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[211443, 5441, 166, 5, 29, "Input",
 InitializationCell->True],
Cell[211612, 5448, 109, 3, 29, "Output"]
}, Open  ]],
Cell[211736, 5454, 491, 16, 37, "Text"],
Cell[CellGroupData[{
Cell[212252, 5474, 67, 1, 28, "Input"],
Cell[212322, 5477, 291, 10, 40, "Output"]
}, Open  ]],
Cell[212628, 5490, 672, 21, 37, "Text"],
Cell[CellGroupData[{
Cell[213325, 5515, 89, 2, 28, "Input"],
Cell[213417, 5519, 31, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[213485, 5524, 89, 2, 28, "Input"],
Cell[213577, 5528, 32, 0, 28, "Output"]
}, Open  ]],
Cell[213624, 5531, 166, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[213815, 5542, 120, 3, 28, "Input"],
Cell[213938, 5547, 31, 0, 28, "Output"]
}, Open  ]],
Cell[213984, 5550, 306, 5, 35, "Text"],
Cell[214293, 5557, 66, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[214384, 5561, 154, 4, 29, "Input",
 InitializationCell->True],
Cell[214541, 5567, 93, 2, 29, "Output"]
}, Open  ]],
Cell[214649, 5572, 96, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[214770, 5578, 151, 4, 29, "Input",
 InitializationCell->True],
Cell[214924, 5584, 105, 2, 28, "Output"]
}, Open  ]],
Cell[215044, 5589, 602, 13, 52, "Text"],
Cell[215649, 5604, 1066, 29, 93, "Input",
 InitializationCell->True],
Cell[216718, 5635, 114, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[216857, 5643, 35, 0, 29, "Input"],
Cell[216895, 5645, 93, 2, 28, "Output"]
}, Open  ]],
Cell[217003, 5650, 64, 0, 20, "Text"],
Cell[217070, 5652, 1834, 47, 220, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[218929, 5703, 164, 4, 29, "Input"],
Cell[219096, 5709, 93, 2, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[219238, 5717, 78, 0, 18, "Subsubsection"],
Cell[219319, 5719, 202, 4, 35, "Text"],
Cell[219524, 5725, 384, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[219933, 5739, 61, 1, 28, "Input"],
Cell[219997, 5742, 1895, 52, 88, "Output"]
}, Open  ]],
Cell[221907, 5797, 593, 19, 36, "Text"],
Cell[CellGroupData[{
Cell[222525, 5820, 53, 1, 28, "Input"],
Cell[222581, 5823, 1083, 35, 61, "Output"]
}, Open  ]],
Cell[223679, 5861, 1148, 34, 68, "Text"],
Cell[CellGroupData[{
Cell[224852, 5899, 273, 9, 29, "Input"],
Cell[225128, 5910, 843, 22, 44, "Output"]
}, Open  ]],
Cell[225986, 5935, 57, 0, 20, "Text"],
Cell[226046, 5937, 349, 10, 45, "Input",
 InitializationCell->True],
Cell[226398, 5949, 156, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[226579, 5958, 214, 6, 29, "Input",
 InitializationCell->True],
Cell[226796, 5966, 843, 22, 44, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[227688, 5994, 63, 0, 18, "Subsubsection"],
Cell[227754, 5996, 203, 4, 35, "Text"],
Cell[227960, 6002, 293, 5, 35, "Text"],
Cell[228256, 6009, 372, 6, 50, "Text"],
Cell[CellGroupData[{
Cell[228653, 6019, 156, 4, 28, "Input",
 InitializationCell->True],
Cell[228812, 6025, 101, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[228950, 6032, 100, 3, 28, "Input",
 InitializationCell->True],
Cell[229053, 6037, 45, 1, 28, "Output"]
}, Open  ]],
Cell[229113, 6041, 889, 23, 108, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[230027, 6068, 33, 0, 28, "Input"],
Cell[230063, 6070, 121, 4, 28, "Output"]
}, Open  ]],
Cell[230199, 6077, 510, 12, 51, "Text"],
Cell[230712, 6091, 516, 18, 21, "Text"],
Cell[CellGroupData[{
Cell[231253, 6113, 58, 1, 28, "Input"],
Cell[231314, 6116, 98, 3, 28, "Output"]
}, Open  ]],
Cell[231427, 6122, 59, 0, 20, "Text"],
Cell[231489, 6124, 1432, 36, 170, "Input",
 InitializationCell->True],
Cell[232924, 6162, 392, 14, 21, "Text"],
Cell[CellGroupData[{
Cell[233341, 6180, 227, 6, 29, "Input"],
Cell[233571, 6188, 109, 3, 29, "Output"]
}, Open  ]],
Cell[233695, 6194, 323, 14, 20, "Text"],
Cell[CellGroupData[{
Cell[234043, 6212, 235, 6, 29, "Input"],
Cell[234281, 6220, 80, 2, 28, "Output"]
}, Open  ]],
Cell[234376, 6225, 327, 7, 36, "Text"],
Cell[234706, 6234, 347, 10, 45, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[235090, 6249, 54, 0, 18, "Subsubsection"],
Cell[235147, 6251, 403, 6, 50, "Text"],
Cell[235553, 6259, 442, 10, 52, "Text"],
Cell[235998, 6271, 779, 16, 68, "Text"],
Cell[236780, 6289, 942, 25, 124, "Input",
 InitializationCell->True],
Cell[237725, 6316, 147, 4, 21, "Text"],
Cell[237875, 6322, 131, 3, 29, "Input",
 InitializationCell->True],
Cell[238009, 6327, 246, 7, 37, "Text"],
Cell[CellGroupData[{
Cell[238280, 6338, 132, 3, 29, "Input"],
Cell[238415, 6343, 50, 1, 28, "Output"]
}, Open  ]],
Cell[238480, 6347, 153, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[238658, 6355, 81, 2, 28, "Input"],
Cell[238742, 6359, 93, 2, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[238884, 6367, 53, 0, 18, "Subsubsection"],
Cell[238940, 6369, 94, 2, 20, "Text"],
Cell[239037, 6373, 435, 12, 50, "Text"],
Cell[239475, 6387, 645, 16, 66, "Text"],
Cell[240123, 6405, 376, 9, 52, "Text"],
Cell[CellGroupData[{
Cell[240524, 6418, 243, 8, 29, "Input"],
Cell[240770, 6428, 198, 7, 28, "Output"]
}, Open  ]],
Cell[240983, 6438, 153, 4, 21, "Text"],
Cell[241139, 6444, 166, 5, 21, "Text"],
Cell[241308, 6451, 635, 20, 92, "Input",
 InitializationCell->True],
Cell[241946, 6473, 57, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[242028, 6477, 161, 4, 28, "Input"],
Cell[242192, 6483, 28, 0, 28, "Output"]
}, Open  ]],
Cell[242235, 6486, 563, 17, 52, "Text"],
Cell[242801, 6505, 55, 0, 20, "Text"],
Cell[242859, 6507, 1289, 37, 125, "Input",
 InitializationCell->True],
Cell[244151, 6546, 96, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[244272, 6554, 122, 3, 29, "Input",
 InitializationCell->True],
Cell[244397, 6559, 1051, 30, 44, "Output"]
}, Open  ]],
Cell[245463, 6592, 490, 14, 36, "Text"],
Cell[CellGroupData[{
Cell[245978, 6610, 129, 3, 29, "Input"],
Cell[246110, 6615, 1643, 49, 104, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[247802, 6670, 46, 0, 18, "Subsubsection"],
Cell[247851, 6672, 392, 9, 36, "Text"],
Cell[248246, 6683, 232, 4, 35, "Text"],
Cell[248481, 6689, 129, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[248635, 6696, 168, 5, 28, "Input",
 InitializationCell->True],
Cell[248806, 6703, 114, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[248957, 6711, 168, 5, 28, "Input",
 InitializationCell->True],
Cell[249128, 6718, 114, 3, 28, "Output"]
}, Open  ]],
Cell[249257, 6724, 74, 0, 20, "Text"],
Cell[249334, 6726, 136, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[249495, 6733, 195, 6, 44, "Input",
 InitializationCell->True],
Cell[249693, 6741, 122, 3, 28, "Output"]
}, Open  ]],
Cell[249830, 6747, 1933, 55, 116, "Text"],
Cell[251766, 6804, 36, 0, 20, "Text"],
Cell[251805, 6806, 1738, 44, 234, "Input",
 InitializationCell->True],
Cell[253546, 6852, 86, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[253657, 6858, 268, 7, 29, "Input"],
Cell[253928, 6867, 122, 3, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[254099, 6876, 59, 0, 18, "Subsubsection"],
Cell[254161, 6878, 288, 7, 37, "Text"],
Cell[254452, 6887, 130, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[254607, 6894, 129, 3, 29, "Input"],
Cell[254739, 6899, 1643, 49, 104, "Output"]
}, Open  ]],
Cell[256397, 6951, 304, 6, 36, "Text"],
Cell[CellGroupData[{
Cell[256726, 6961, 99, 3, 28, "Input",
 InitializationCell->True],
Cell[256828, 6966, 45, 1, 28, "Output"]
}, Open  ]],
Cell[256888, 6970, 1003, 33, 68, "Text"],
Cell[257894, 7005, 50, 0, 20, "Text"],
Cell[257947, 7007, 754, 21, 93, "myCode"],
Cell[258704, 7030, 693, 23, 53, "Text"],
Cell[259400, 7055, 1234, 35, 138, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[260659, 7094, 32, 0, 28, "Input"],
Cell[260694, 7096, 1187, 29, 59, "Output"]
}, Open  ]],
Cell[261896, 7128, 363, 6, 50, "Text"],
Cell[262262, 7136, 715, 17, 68, "Text"],
Cell[CellGroupData[{
Cell[263002, 7157, 88, 2, 29, "Input"],
Cell[263093, 7161, 843, 22, 44, "Output"]
}, Open  ]],
Cell[263951, 7186, 617, 18, 36, "Text"],
Cell[CellGroupData[{
Cell[264593, 7208, 349, 11, 29, "Input"],
Cell[264945, 7221, 685, 18, 44, "Output"]
}, Open  ]],
Cell[265645, 7242, 189, 4, 35, "Text"],
Cell[265837, 7248, 2325, 59, 249, "Input"],
Cell[268165, 7309, 453, 7, 50, "Text"],
Cell[268621, 7318, 200, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[268846, 7328, 74, 1, 29, "Input"],
Cell[268923, 7331, 2779, 86, 308, "Output"]
}, Open  ]],
Cell[271717, 7420, 299, 5, 50, "Text"],
Cell[272019, 7427, 270, 9, 45, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[272314, 7440, 103, 2, 29, "Input",
 InitializationCell->True],
Cell[272420, 7444, 846, 24, 60, "Output"]
}, Open  ]],
Cell[273281, 7471, 393, 11, 60, "Input",
 InitializationCell->True],
Cell[273677, 7484, 1189, 28, 99, "Text"],
Cell[274869, 7514, 130, 2, 29, "Input"],
Cell[CellGroupData[{
Cell[275024, 7520, 76, 1, 29, "Input"],
Cell[275103, 7523, 959, 28, 91, "Output"]
}, Open  ]],
Cell[276077, 7554, 762, 20, 68, "Text"],
Cell[276842, 7576, 101, 4, 21, "Text"],
Cell[276946, 7582, 2711, 68, 280, "Input",
 InitializationCell->True],
Cell[279660, 7652, 169, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[279854, 7664, 212, 6, 29, "Input",
 InitializationCell->True],
Cell[280069, 7672, 1308, 32, 59, "Output"]
}, Open  ]],
Cell[281392, 7707, 37, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[281454, 7711, 64, 1, 29, "Input"],
Cell[281521, 7714, 29, 0, 28, "Output"]
}, Open  ]],
Cell[281565, 7717, 110, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[281700, 7724, 65, 1, 29, "Input"],
Cell[281768, 7727, 28, 0, 28, "Output"]
}, Open  ]],
Cell[281811, 7730, 129, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[281965, 7738, 74, 1, 29, "Input"],
Cell[282042, 7741, 6227, 203, 711, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[288318, 7950, 34, 0, 18, "Subsubsection"],
Cell[288355, 7952, 50, 0, 20, "Text"],
Cell[288408, 7954, 228, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[288661, 7968, 124, 3, 29, "Input",
 InitializationCell->True],
Cell[288788, 7973, 1423, 39, 59, "Output"]
}, Open  ]],
Cell[290226, 8015, 212, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[290463, 8029, 213, 6, 29, "Input",
 InitializationCell->True],
Cell[290679, 8037, 709, 20, 28, "Output"]
}, Open  ]],
Cell[291403, 8060, 152, 3, 20, "Text"],
Cell[291558, 8065, 96, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[291679, 8073, 124, 3, 29, "Input",
 InitializationCell->True],
Cell[291806, 8078, 508, 16, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[292351, 8099, 213, 6, 29, "Input",
 InitializationCell->True],
Cell[292567, 8107, 423, 12, 28, "Output"]
}, Open  ]],
Cell[293005, 8122, 181, 5, 21, "Text"],
Cell[293189, 8129, 109, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[293323, 8136, 171, 4, 29, "Input",
 InitializationCell->True],
Cell[293497, 8142, 602, 15, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[294148, 8163, 51, 0, 18, "Subsubsection"],
Cell[294202, 8165, 148, 3, 20, "Text"],
Cell[294353, 8170, 74, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[294452, 8174, 74, 1, 29, "Input"],
Cell[294529, 8177, 7111, 230, 804, "Output"]
}, Open  ]],
Cell[301655, 8410, 373, 7, 51, "Text"],
Cell[302031, 8419, 131, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[302187, 8426, 70, 1, 29, "Input"],
Cell[302260, 8429, 28, 0, 28, "Output"]
}, Open  ]],
Cell[302303, 8432, 76, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[302404, 8436, 63, 1, 29, "Input"],
Cell[302470, 8439, 29, 0, 28, "Output"]
}, Open  ]],
Cell[302514, 8442, 482, 17, 36, "Text"],
Cell[302999, 8461, 309, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[303333, 8472, 175, 5, 29, "Input"],
Cell[303511, 8479, 893, 21, 120, "Output"]
}, Open  ]],
Cell[304419, 8503, 307, 5, 50, "Text"],
Cell[304729, 8510, 70, 0, 20, "Text"],
Cell[304802, 8512, 1466, 41, 155, "Input",
 InitializationCell->True],
Cell[306271, 8555, 91, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[306387, 8561, 294, 7, 45, "Input",
 InitializationCell->True],
Cell[306684, 8570, 893, 21, 120, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[307626, 8597, 48, 0, 18, "Subsubsection"],
Cell[307677, 8599, 375, 6, 50, "Text"],
Cell[308055, 8607, 223, 4, 35, "Text"],
Cell[308281, 8613, 212, 5, 36, "Text"],
Cell[308496, 8620, 333, 6, 50, "Text"],
Cell[308832, 8628, 384, 11, 37, "Text"],
Cell[309219, 8641, 507, 8, 65, "Text"],
Cell[309729, 8651, 349, 8, 36, "Text"],
Cell[310081, 8661, 291, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[310397, 8670, 88, 2, 29, "Input"],
Cell[310488, 8674, 101, 2, 28, "Output"]
}, Open  ]],
Cell[310604, 8679, 360, 7, 51, "Text"],
Cell[310967, 8688, 528, 11, 51, "Text"],
Cell[CellGroupData[{
Cell[311520, 8703, 785, 22, 92, "Input",
 InitializationCell->True],
Cell[312308, 8727, 73, 2, 28, "Output"]
}, Open  ]],
Cell[312396, 8732, 404, 10, 37, "Text"],
Cell[CellGroupData[{
Cell[312825, 8746, 321, 9, 29, "Input"],
Cell[313149, 8757, 147, 3, 28, "Output"]
}, Open  ]],
Cell[313311, 8763, 211, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[313547, 8771, 434, 13, 49, "Input",
 InitializationCell->True],
Cell[313984, 8786, 4296, 148, 150, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[318317, 8939, 307, 9, 29, "Input"],
Cell[318627, 8950, 2346, 80, 114, "Output"]
}, Open  ]],
Cell[320988, 9033, 122, 3, 20, "Text"],
Cell[321113, 9038, 328, 9, 36, "Text"],
Cell[CellGroupData[{
Cell[321466, 9051, 165, 4, 29, "Input",
 InitializationCell->True],
Cell[321634, 9057, 103, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[321774, 9064, 236, 7, 29, "Input",
 InitializationCell->True],
Cell[322013, 9073, 141, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[322191, 9082, 178, 5, 29, "Input"],
Cell[322372, 9089, 786, 20, 104, "Output"]
}, Open  ]],
Cell[323173, 9112, 198, 5, 36, "Text"],
Cell[323374, 9119, 561, 15, 52, "Text"],
Cell[CellGroupData[{
Cell[323960, 9138, 100, 2, 29, "Input"],
Cell[324063, 9142, 515, 13, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[324615, 9160, 96, 2, 29, "Input"],
Cell[324714, 9164, 685, 18, 44, "Output"]
}, Open  ]],
Cell[325414, 9185, 37, 0, 20, "Text"],
Cell[325454, 9187, 2509, 66, 283, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[328000, 9258, 59, 0, 18, "Subsubsection"],
Cell[328062, 9260, 297, 5, 35, "Text"],
Cell[328362, 9267, 161, 3, 20, "Text"],
Cell[328526, 9272, 380, 11, 37, "Text"],
Cell[328909, 9285, 843, 18, 83, "Text"],
Cell[329755, 9305, 126, 3, 20, "Text"],
Cell[329884, 9310, 2271, 57, 292, "Input",
 InitializationCell->True],
Cell[332158, 9369, 485, 12, 36, "Text"],
Cell[332646, 9383, 95, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[332766, 9389, 140, 3, 29, "Input",
 InitializationCell->True],
Cell[332909, 9394, 73, 2, 28, "Output"]
}, Open  ]],
Cell[332997, 9399, 217, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[333239, 9409, 100, 3, 28, "Input",
 InitializationCell->True],
Cell[333342, 9414, 45, 1, 28, "Output"]
}, Open  ]],
Cell[333402, 9418, 397, 10, 61, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[333824, 9432, 33, 0, 28, "Input"],
Cell[333860, 9434, 246, 7, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[334143, 9446, 71, 1, 29, "Input"],
Cell[334217, 9449, 619, 19, 88, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[334885, 9474, 48, 0, 18, "Subsubsection"],
Cell[334936, 9476, 274, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[335235, 9485, 151, 4, 29, "Input"],
Cell[335389, 9491, 31, 0, 28, "Output"]
}, Open  ]],
Cell[335435, 9494, 288, 5, 35, "Text"],
Cell[335726, 9501, 720, 20, 68, "Text"],
Cell[336449, 9523, 67, 0, 20, "Text"],
Cell[336519, 9525, 1273, 32, 171, "Input",
 InitializationCell->True],
Cell[337795, 9559, 316, 7, 36, "Text"],
Cell[338114, 9568, 648, 17, 93, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[338787, 9589, 33, 0, 28, "Input"],
Cell[338823, 9591, 333, 9, 28, "Output"]
}, Open  ]],
Cell[339171, 9603, 213, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[339409, 9613, 163, 4, 29, "Input"],
Cell[339575, 9619, 297, 11, 29, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[339921, 9636, 48, 0, 18, "Subsubsection"],
Cell[339972, 9638, 218, 4, 35, "Text"],
Cell[340193, 9644, 4094, 101, 458, "Input",
 InitializationCell->True],
Cell[344290, 9747, 137, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[344452, 9755, 887, 28, 45, "Input"],
Cell[345342, 9785, 836, 27, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[346215, 9817, 162, 4, 29, "Input"],
Cell[346380, 9823, 288, 10, 29, "Output"]
}, Open  ]],
Cell[346683, 9836, 109, 3, 20, "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[346853, 9846, 115, 3, 35, "Section",
 CellTags->"Solutions"],
Cell[CellGroupData[{
Cell[346993, 9853, 68, 1, 32, "Subsection",
 CellTags->"SolutionsCP2"],
Cell[347064, 9856, 118, 3, 42, "Problem"],
Cell[347185, 9861, 675, 21, 56, "Text"],
Cell[347863, 9884, 1815, 58, 86, "Text"],
Cell[349681, 9944, 922, 29, 52, "Text"],
Cell[CellGroupData[{
Cell[350628, 9977, 93, 2, 28, "Input"],
Cell[350724, 9981, 175, 5, 30, "Output"]
}, Open  ]],
Cell[350914, 9989, 204, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[351143, 9999, 165, 4, 28, "Input"],
Cell[351311, 10005, 32, 0, 28, "Output"]
}, Open  ]],
Cell[351358, 10008, 137, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[351520, 10015, 162, 4, 29, "Input"],
Cell[351685, 10021, 209, 7, 29, "Output"]
}, Open  ]],
Cell[351909, 10031, 409, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[352343, 10045, 133, 3, 28, "Input"],
Cell[352479, 10050, 170, 4, 28, "Output"]
}, Open  ]],
Cell[352664, 10057, 356, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[353045, 10069, 360, 10, 29, "Input"],
Cell[353408, 10081, 802, 19, 59, "Output"]
}, Open  ]],
Cell[354225, 10103, 239, 8, 20, "Text"],
Cell[354467, 10113, 720, 19, 52, "Text"],
Cell[CellGroupData[{
Cell[355212, 10136, 624, 17, 61, "Input"],
Cell[355839, 10155, 2460, 72, 152, "Output"]
}, Open  ]],
Cell[358314, 10230, 284, 8, 35, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[358635, 10243, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE6"],
Cell[358718, 10246, 201, 4, 55, "Problem"],
Cell[358922, 10252, 278, 6, 35, "Text"],
Cell[359203, 10260, 1269, 31, 90, "Text"],
Cell[CellGroupData[{
Cell[360497, 10295, 151, 4, 29, "Input"],
Cell[360651, 10301, 203, 5, 28, "Output"]
}, Open  ]],
Cell[360869, 10309, 384, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[361278, 10323, 468, 14, 45, "Input"],
Cell[361749, 10339, 5384, 178, 184, "Output"]
}, Open  ]],
Cell[367148, 10520, 243, 4, 35, "Text"],
Cell[367394, 10526, 127, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[367546, 10533, 1320, 38, 124, "Input"],
Cell[368869, 10573, 88, 1, 28, "Output"]
}, Open  ]],
Cell[368972, 10577, 481, 11, 52, "Text"],
Cell[369456, 10590, 4271, 105, 488, "Input",
 InitializationCell->True],
Cell[373730, 10697, 119, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[373874, 10705, 1235, 35, 124, "Input"],
Cell[375112, 10742, 31, 0, 28, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[375204, 10749, 52, 1, 35, "Section",
 CellTags->"Exercises"],
Cell[CellGroupData[{
Cell[375281, 10754, 200, 6, 20, "Item1Exercise"],
Cell[375484, 10762, 162, 3, 20, "Item1Exercise"],
Cell[CellGroupData[{
Cell[375671, 10769, 205, 8, 15, "Item2Exercise"],
Cell[375879, 10779, 255, 10, 19, "Item2Exercise"],
Cell[376137, 10791, 335, 13, 15, "Item2Exercise"]
}, Open  ]],
Cell[376487, 10807, 301, 10, 21, "Item1Exercise"],
Cell[376791, 10819, 259, 7, 21, "Item1Exercise"],
Cell[CellGroupData[{
Cell[377075, 10830, 269, 9, 15, "Item2Exercise"],
Cell[377347, 10841, 332, 11, 15, "Item2Exercise"],
Cell[377682, 10854, 294, 9, 15, "Item2Exercise"]
}, Open  ]],
Cell[377991, 10866, 299, 10, 21, "Item1Exercise"],
Cell[378293, 10878, 261, 7, 21, "Item1Exercise"],
Cell[378557, 10887, 429, 11, 46, "Item1Exercise"],
Cell[378989, 10900, 341, 8, 33, "Item1Exercise"],
Cell[379333, 10910, 381, 12, 36, "Item1Exercise"],
Cell[379717, 10924, 413, 12, 33, "Item1Exercise"],
Cell[380133, 10938, 512, 16, 34, "Item1Exercise"],
Cell[380648, 10956, 397, 10, 33, "Item1Exercise"],
Cell[381048, 10968, 356, 8, 34, "Item1Exercise"],
Cell[381407, 10978, 488, 9, 59, "Item1Exercise"],
Cell[381898, 10989, 127, 4, 21, "Item1Exercise"],
Cell[382028, 10995, 382, 7, 47, "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
